// SPDX-License-Identifier: GPL-2.0-or-later
/*
 *  History:
 *  Started: Aug 9 by Lawrence Foard (entropy@world.std.com),
 *           to allow user process control of SCSI devices.
 *  Development Sponsored by Killy Corp. NY NY   [1992 ??]
 *
 * Original driver (sg.c):
 *        Copyright (C) 1992 Lawrence Foard
 * Version 2, 3 and 4 extensions to driver:
 *        Copyright (C) 1998 - 2019 Douglas Gilbert
 *
 */

static int sg_version_num = 40036;  /* [x]xyyzz where [x] empty when x=0 */
#define SG_VERSION_STR "4.0.36"		/* [x]x.[y]y.zz */
static char *sg_version_date = "20191003";

#include <linux/module.h>

#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/mtio.h>
#include <linux/ioctl.h>
#include <linux/slab.h>
#include <linux/fcntl.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/moduleparam.h>
#include <linux/cdev.h>
#include <linux/idr.h>
#include <linux/file.h>		/* for fget() and fput() */
#include <linux/seq_file.h>
#include <linux/blkdev.h>
#include <linux/delay.h>
#include <linux/blktrace_api.h>
#include <linux/mutex.h>
#include <linux/atomic.h>
#include <linux/ratelimit.h>
#include <linux/uio.h>
#include <linux/cred.h>			/* for sg_check_file_access() */
#include <linux/timekeeping.h>
#include <linux/proc_fs.h>		/* used if CONFIG_SCSI_PROC_FS */
#include <linux/debugfs.h>
#include <linux/rculist.h>

#include <scsi/scsi.h>
#include <scsi/scsi_eh.h>
#include <scsi/scsi_dbg.h>
#include <scsi/scsi_host.h>
#include <scsi/scsi_driver.h>
#include <scsi/scsi_ioctl.h>
#include <scsi/sg.h>

#include "scsi_logging.h"

#define SG_ALLOW_DIO_DEF 0

#define SG_MAX_DEVS 32768
#define SG_MAX_MULTI_REQ_SZ (2 * 1024 * 1024)

/* Comment out the following line to compile out SCSI_LOGGING stuff */
#define SG_DEBUG 1

#if !IS_ENABLED(SG_DEBUG)
#if IS_ENABLED(DEBUG)	/* If SG_DEBUG not defined, check for DEBUG */
#define SG_DEBUG DEBUG
#endif
#endif

#if IS_ENABLED(CONFIG_SCSI_PROC_FS) || IS_ENABLED(CONFIG_DEBUG_FS)
#define SG_PROC_OR_DEBUG_FS 1
#endif

/*
 * SG_MAX_CDB_SIZE should be 260 (spc4r37 section 3.1.30) however the type
 * of sg_io_hdr::cmd_len can only represent 255. All SCSI commands greater
 * than 16 bytes are "variable length" whose length is a multiple of 4, so:
 */
#define SG_MAX_CDB_SIZE 252

static struct kmem_cache *sg_sense_cache;
#define SG_MEMPOOL_MIN_NR 4
static mempool_t *sg_sense_pool;

#define uptr64(usp_val) ((void __user *)(uintptr_t)(usp_val))
#define cuptr64(usp_val) ((const void __user *)(uintptr_t)(usp_val))

/* Following enum contains the states of sg_request::rq_st */
enum sg_rq_state {
	SG_RS_INACTIVE = 0,	/* request not in use (e.g. on fl) */
	SG_RS_INFLIGHT,		/* active: cmd/req issued, no response yet */
	SG_RS_AWAIT_RCV,	/* have response from LLD, awaiting receive */
	SG_RS_RCV_DONE,		/* receive is ongoing or done */
	SG_RS_BUSY,		/* temporary state should rarely be seen */
	SG_RS_SHR_SWAP,		/* master: is finished, await swap to slave */
	SG_RS_SHR_SLAVE,	/* master: waits while slave inflight */
	SG_RS_SHR_SWAIT,	/* slave: waiting for master to complete */
};

/* slave sets up sharing: ioctl(sl_fd,SG_SET_GET_EXTENDED(SHARE_FD(ma_fd))) */
enum sg_shr_var {
	SG_SHR_NONE = 0,	/* no sharing on owning fd */
	SG_SHR_MA_NOT_SRQ,	/* master sharing on fd but not on this req */
	SG_SHR_MA_RQ,		/* master sharing on this req */
	SG_SHR_SL_NOT_SRQ,	/* slave sharing on fd but not on this req */
	SG_SHR_SL_RQ,		/* slave sharing on this req */
};

/* If sum_of(dlen) of a fd exceeds this, write() will yield E2BIG */
#define SG_TOT_FD_THRESHOLD (32 * 1024 * 1024)

#define SG_TIME_UNIT_MS 0	/* milliseconds */
/* #define SG_TIME_UNIT_NS 1	   nanoseconds */
#define SG_DEF_TIME_UNIT SG_TIME_UNIT_MS
#define SG_DEFAULT_TIMEOUT mult_frac(SG_DEFAULT_TIMEOUT_USER, HZ, USER_HZ)
#define SG_FD_Q_AT_HEAD 0
#define SG_DEFAULT_Q_AT SG_FD_Q_AT_HEAD /* for backward compatibility */
#define SG_FL_MMAP_DIRECT (SG_FLAG_MMAP_IO | SG_FLAG_DIRECT_IO)
#define SG_FL_IMMED_NO_WAITQ (SGV4_FLAG_IMMED | SGV4_FLAG_NO_WAITQ)

#define SG_MAX_RSV_REQS 4

/* Only take lower 4 bits of driver byte, all host byte and sense byte */
#define SG_ML_RESULT_MSK 0x0fff00ff	/* mid-level's 32 bit result value */

#define SG_SHARE_FD_UNUSED (-1)
#define SG_SHARE_FD_MASTER (-2)		/* values >= 0 imply FD_SLAVE */

#define SG_PACK_ID_WILDCARD (-1)
#define SG_TAG_WILDCARD (-1)

#define SG_ADD_RQ_MAX_RETRIES 40	/* to stop infinite _trylock(s) */

/* Bit positions (flags) for sg_request::frq_bm bitmask follow */
#define SG_FRQ_IS_V4I		0	/* true (set) when is v4 interface */
#define SG_FRQ_IS_ORPHAN	1	/* owner of request gone */
#define SG_FRQ_SYNC_INVOC	2	/* synchronous (blocking) invocation */
#define SG_FRQ_DIO_IN_USE	3	/* false->indirect_IO,mmap; 1->dio */
#define SG_FRQ_NO_US_XFER	4	/* no user space transfer of data */
#define SG_FRQ_ABORTING		5	/* in process of aborting this cmd */
#define SG_FRQ_DEFER_XFR	6	/* slave waiting will defer transfer */
#define SG_FRQ_DEACT_ORPHAN	7	/* not keeping orphan so de-activate */
#define SG_FRQ_MULTI_REQ	8	/* part of a multiple request series */
#define SG_FRQ_BLK_PUT_REQ	9	/* set when blk_put_request() called */
#define SG_FRQ_RESERVED		10	/* marks a reserved request */
#define SG_FRQ_WAKE_MASTER	11	/* svb: send wake up on master's fd */

/* Bit positions (flags) for sg_fd::ffd_bm bitmask follow */
#define SG_FFD_FORCE_PACKID	0	/* receive only given pack_id/tag */
#define SG_FFD_NO_CMD_Q		1	/* set: only 1 active req per fd */
#define SG_FFD_KEEP_ORPHAN	2	/* policy for this fd */
#define SG_FFD_MMAP_CALLED	3	/* mmap(2) system call made on fd */
#define SG_FFD_TIME_IN_NS	4	/* set: time in nanoseconds, else ms */
#define SG_FFD_Q_AT_TAIL	5	/* set: queue reqs at tail of blk q */
#define SG_FFD_MASTER_ERR	6	/* master side of share had error */
#define SG_FFD_PREFER_TAG	7	/* prefer tag over pack_id (def) */
#define SG_FFD_RELEASE		8	/* release (close) underway */
#define SG_FFD_NO_DURATION	9	/* don't do command duration calc */
#define SG_FFD_MORE_ASYNC	10	/* yield EBUSY more often */
#define SG_FFD_MRQ_ABORT	11	/* SG_IOABORT + FLAG_MULTIPLE_REQS */
#define SG_FFD_EXCL_WAITQ	12	/* append _exclusive to wait_event */
#define SG_FFD_SVB_ACTIVE	13	/* special variable blocking active */

/* Bit positions (flags) for sg_device::fdev_bm bitmask follow */
#define SG_FDEV_EXCLUDE		0	/* have fd open with O_EXCL */
#define SG_FDEV_DETACHING	1	/* may be unexpected device removal */
#define SG_FDEV_LOG_SENSE	2	/* set by ioctl(SG_SET_DEBUG) */

int sg_big_buff = SG_DEF_RESERVED_SIZE;
/*
 * This variable is accessible via /proc/scsi/sg/def_reserved_size . Each
 * time sg_open() is called a sg_request of this size (or less if there is
 * not enough memory) will be reserved for use by this file descriptor.
 */
static int def_reserved_size = -1;	/* picks up init parameter */
static int sg_allow_dio = SG_ALLOW_DIO_DEF;

static int scatter_elem_sz = SG_SCATTER_SZ;

#define SG_DEF_SECTOR_SZ 512

static int sg_add_device(struct device *, struct class_interface *);
static void sg_remove_device(struct device *, struct class_interface *);

static DEFINE_IDR(sg_index_idr);
static DEFINE_RWLOCK(sg_index_lock); /* Also used to lock fd list for device */

static struct class_interface sg_interface = {
	.add_dev        = sg_add_device,
	.remove_dev     = sg_remove_device,
};

static DEFINE_MUTEX(snapped_mutex);
static char *snapped_buf;

/* Subset of sg_io_hdr found in <scsi/sg.h>, has only [i] and [i->o] fields */
struct sg_slice_hdr3 {
	int interface_id;
	int dxfer_direction;
	u8 cmd_len;
	u8 mx_sb_len;
	u16 iovec_count;
	unsigned int dxfer_len;
	void __user *dxferp;
	u8 __user *cmdp;
	void __user *sbp;
	unsigned int timeout;
	unsigned int flags;
	int pack_id;
	void __user *usr_ptr;
};

struct sg_slice_hdr4 {	/* parts of sg_io_v4 object needed in async usage */
	void __user *sbp;	/* derived from sg_io_v4::response */
	u64 usr_ptr;		/* hold sg_io_v4::usr_ptr as given (u64) */
	int out_resid;
	s16 dir;		/* data xfer direction; SG_DXFER_*  */
	u16 cmd_len;		/* truncated of sg_io_v4::request_len */
	u16 max_sb_len;		/* truncated of sg_io_v4::max_response_len */
	u16 mrq_ind;		/* mrq submit order, origin 0 */
	atomic_t pack_id_of_mrq;	/* mrq pack_id, active when > 0 */
};

struct sg_scatter_hold {     /* holding area for scsi scatter gather info */
	struct page **pages;	/* num_sgat element array of struct page* */
	int buflen;		/* capacity in bytes (dlen<=buflen) */
	int dlen;		/* current valid data length of this req */
	u16 page_order;		/* byte_len = (page_size*(2**page_order)) */
	u16 num_sgat;		/* actual number of scatter-gather segments */
};

struct sg_device;		/* forward declarations */
struct sg_fd;

struct sg_request {	/* active SCSI command or inactive on free list (fl) */
	struct list_head rq_entry;	/* member of rq_list (active cmd) */
	struct list_head fl_entry;	/* member of rq_fl */
	spinlock_t req_lck;
	struct sg_scatter_hold sgat_h;	/* hold buffer, perhaps scatter list */
	struct sg_scatter_hold *sgatp;	/* ptr to prev unless slave shr req */
	union {
		struct sg_slice_hdr3 s_hdr3;  /* subset of sg_io_hdr */
		struct sg_slice_hdr4 s_hdr4; /* reduced size struct sg_io_v4 */
	};
	u32 duration;		/* cmd duration in milli or nano seconds */
	u32 rq_flags;		/* flags given in v3 and v4 */
	u32 rq_info;		/* info supplied by v3 and v4 interfaces */
	u32 rq_result;		/* packed scsi request result from LLD */
	int in_resid;		/* requested-actual byte count on data-in */
	int pack_id;		/* v3 pack_id or in v4 request_extra field */
	int sense_len;		/* actual sense buffer length (data-in) */
	atomic_t rq_st;		/* request state, holds a enum sg_rq_state */
	enum sg_shr_var sh_var;	/* sharing variety, SG_SHR_NONE=0 if none */
	u8 cmd_opcode;		/* first byte of SCSI cdb */
	int tag;		/* block layer identifier of request */
	u64 start_ns;		/* starting point of command duration calc */
	unsigned long frq_bm[1];	/* see SG_FRQ_* defines above */
	u8 *sense_bp;		/* mempool alloc-ed sense buffer, as needed */
	struct sg_fd *parentfp;	/* pointer to owning fd, even when on fl */
	struct request *rq;	/* released in sg_rq_end_io(), bio kept */
	struct bio *bio;	/* kept until this req -->SG_RS_INACTIVE */
	struct execute_work ew_orph;	/* harvest orphan request */
};

struct sg_rsv_slave {	/* element in sg_fd::rsv_arr */
	struct sg_request *rsv_srp;/* reserve request owned by this fd */
	struct sg_request *slave_srp;	/* non-NULL when request sharing */
};

struct sg_fd {		/* holds the state of a file descriptor */
	struct list_head sfd_entry;	/* member sg_device::sfds list */
	struct sg_device *parentdp;	/* owning device */
	wait_queue_head_t cmpl_wait;	/* queue awaiting req completion */
	struct mutex f_mutex;	/* serialize ioctls on this fd */
	spinlock_t rq_list_lock;/* for rw-lock on sg_request lists [rcu] */
	struct list_head rq_list; /* head of inflight sg_request list */
	struct list_head rq_fl; /* head of sg_request free list */
	int timeout;		/* defaults to SG_DEFAULT_TIMEOUT      */
	int timeout_user;	/* defaults to SG_DEFAULT_TIMEOUT_USER */
	atomic_t submitted;	/* number inflight or awaiting receive */
	atomic_t waiting;	/* number of requests awaiting receive */
	atomic_t sum_fd_dlens;	/* when tot_fd_thresh>0 this is sum_of(dlen) */
	atomic_t mrq_id_abort;	/* inactive when 0, else id if aborted */
	atomic_t svb_cnt;	/* event counter used on svb master */
	int tot_fd_thresh;	/* E2BIG if sum_of(dlen) > this, 0: ignore */
	int sgat_elem_sz;	/* initialized to scatter_elem_sz */
	int shr_fd;		/* init: SG_SHARE_FD_UNUSED; master: -2 */
	pid_t tid;		/* thread id when opened */
	u8 next_cmd_len;	/* 0: automatic, >0: use on next write() */
	unsigned long ffd_bm[1];	/* see SG_FFD_* defines above */
	struct file *filp;	/* my identity when sharing */
	struct sg_fd *share_sfp;/* master+slave shares set this, else NULL */
	struct fasync_struct *async_qp; /* used by asynchronous notification */
	struct sg_rsv_slave rsv_arr[SG_MAX_RSV_REQS];
	struct kref f_ref;
	struct execute_work ew_fd;  /* harvest all fd resources and lists */
};

struct sg_device { /* holds the state of each scsi generic device */
	struct scsi_device *device;
	wait_queue_head_t open_wait;    /* queue open() when O_EXCL present */
	struct mutex open_rel_lock;     /* held when in open() or release() */
	struct list_head sfds;
	rwlock_t sfd_lock;      /* protect access to sfd list */
	int max_sgat_elems;     /* adapter's max number of elements in sgat */
	int max_sgat_sz;	/* max number of bytes in sgat list */
	u32 index;		/* device index number */
	atomic_t open_cnt;	/* count of opens (perhaps < num(sfds) ) */
	unsigned long fdev_bm[1];	/* see SG_FDEV_* defines above */
	struct gendisk *disk;
	struct cdev *cdev;	/* char_dev [sysfs: /sys/cdev/major/sg<n>] */
	struct kref d_ref;
};

struct sg_comm_wr_t {  /* arguments to sg_common_write() */
	int timeout;
	int cmd_len;
	unsigned long frq_bm[1];	/* see SG_FRQ_* defines above */
	union {		/* selector is frq_bm.SG_FRQ_IS_V4I */
		struct sg_io_hdr *h3p;
		struct sg_io_v4 *h4p;
	};
	struct sg_fd *sfp;
	const u8 __user *u_cmdp;
	const u8 *cmdp;
};

struct sg_mrq_hold {	/* for passing context between mrq functions */
	bool blocking;
	bool chk_abort;
	bool immed;
	bool stop_if;
	int id_of_mrq;
	int s_res;		/* secondary error: some-good-then-error */
	u32 cdb_mxlen;		/* cdb length in cdb_ap, actual be may less */
	u32 tot_reqs;		/* total number of requests and cdb_s */
	struct sg_comm_wr_t *cwrp;
	u8 *cdb_ap;		/* array of commands */
	struct sg_io_v4 *a_hds;	/* array of request to execute */
};

/* tasklet or soft irq callback */
static void sg_rq_end_io(struct request *rq, blk_status_t status);
/* Declarations of other static functions used before they are defined */
static int sg_proc_init(void);
static void sg_dfs_init(void);
static void sg_dfs_exit(void);
static int sg_start_req(struct sg_request *srp, struct sg_comm_wr_t *cwrp,
			int dxfer_dir);
static void sg_finish_scsi_blk_rq(struct sg_request *srp);
static int sg_mk_sgat(struct sg_request *srp, struct sg_fd *sfp, int minlen);
static int sg_v3_receive(struct sg_fd *sfp, struct sg_request *srp,
			 void __user *p);
static int sg_v3_submit(struct sg_fd *sfp, struct sg_io_hdr *hp, bool sync,
			struct sg_request **o_srp);
static struct sg_request *sg_common_write(struct sg_comm_wr_t *cwrp);
static int sg_wait_event_srp(struct sg_fd *sfp, void __user *p,
			     struct sg_io_v4 *h4p, struct sg_request *srp);
static int sg_v4_receive(struct sg_fd *sfp, struct sg_request *srp,
			 void __user *p, struct sg_io_v4 *h4p);
static int sg_rd_append(struct sg_request *srp, void __user *outp,
			int num_xfer);
static void sg_remove_sgat(struct sg_request *srp);
static struct sg_fd *sg_add_sfp(struct sg_device *sdp, struct file *filp);
static void sg_remove_sfp(struct kref *);
static struct sg_request *sg_find_srp_by_id(struct sg_fd *sfp, int id,
					    bool is_tag);
static bool sg_mrq_get_ready_srp(struct sg_fd *sfp, struct sg_request **srpp);
static struct sg_request *sg_add_request(struct sg_comm_wr_t *cwrp,
					 enum sg_shr_var sh_var, int dxfr_len);
static int sg_rq_map_kern(struct sg_request *srp, struct request_queue *q,
			  struct request *rq);
static void sg_deact_request(struct sg_fd *sfp, struct sg_request *srp);
static struct sg_device *sg_get_dev(int min_dev);
static void sg_device_destroy(struct kref *kref);
static struct sg_request *sg_mk_srp_sgat(struct sg_fd *sfp, bool first,
					 int db_len);
static void sg_calc_sgat_param(struct sg_device *sdp);
static const char *sg_rq_st_str(enum sg_rq_state rq_st, bool long_str);
static const char *sg_shr_str(enum sg_shr_var sh_var, bool long_str);
#if IS_ENABLED(SG_PROC_OR_DEBUG_FS)
static int sg_proc_debug_sdev(struct sg_device *sdp, char *obp, int len,
			      int *fd_counterp, bool reduced);
#endif

#define SZ_SG_HEADER ((int)sizeof(struct sg_header))	/* v1 and v2 header */
#define SZ_SG_IO_HDR ((int)sizeof(struct sg_io_hdr))	/* v3 header */
#define SZ_SG_IO_V4 ((int)sizeof(struct sg_io_v4))  /* v4 header (in bsg.h) */
#define SZ_SG_REQ_INFO ((int)sizeof(struct sg_req_info))
#define SZ_SG_EXTENDED_INFO ((int)sizeof(struct sg_extended_info))

/* There is a assert that SZ_SG_IO_V4 >= SZ_SG_IO_HDR in first function */

#define SG_IS_DETACHING(sdp) test_bit(SG_FDEV_DETACHING, (sdp)->fdev_bm)
#define SG_HAVE_EXCLUDE(sdp) test_bit(SG_FDEV_EXCLUDE, (sdp)->fdev_bm)
#define SG_IS_O_NONBLOCK(sfp) (!!((sfp)->filp->f_flags & O_NONBLOCK))
#define SG_RS_ACTIVE(srp) (atomic_read(&(srp)->rq_st) != SG_RS_INACTIVE)
#define SG_SYNC_INVOC(srp) test_bit(SG_FRQ_SYNC_INVOC, (srp)->frq_bm)

/*
 * Kernel needs to be built with CONFIG_SCSI_LOGGING to see log messages.
 * 'depth' is a number between 1 (most severe) and 7 (most noisy, most
 * information). All messages are logged as informational (KERN_INFO). In
 * the unexpected situation where sfp or sdp is NULL the macro reverts to
 * a pr_info and ignores SCSI_LOG_TIMEOUT and always prints to the log.
 * Example: this invocation: 'scsi_logging_level -s -T 3' will print
 * depth (aka level) 1 and 2 SG_LOG() messages.
 */

#define SG_PROC_DEBUG_SZ 8192
#define SG_SNAP_BUFF_SZ (SG_PROC_DEBUG_SZ * 4)

#if IS_ENABLED(CONFIG_SCSI_LOGGING) && IS_ENABLED(SG_DEBUG)
#define SG_LOG_BUFF_SZ 48
#define SG_LOG_ACTIVE 1

#define SG_LOG(depth, sfp, fmt, a...)					\
	do {								\
		char _b[SG_LOG_BUFF_SZ];				\
		int _tid = (current ? current->pid : -1);		\
		struct sg_fd *_fp = sfp;				\
		struct sg_device *_sdp = _fp ? _fp->parentdp : NULL;	\
									\
		if (likely(_sdp && _sdp->disk)) {			\
			snprintf(_b, sizeof(_b), "sg%u: tid=%d",	\
				 _sdp->index, _tid);			\
			SCSI_LOG_TIMEOUT(depth,				\
					 sdev_prefix_printk(KERN_INFO,	\
					 _sdp->device, _b, fmt, ##a));	\
		} else							\
			pr_info("sg: sdp or sfp NULL, " fmt, ##a);	\
	} while (0)
#else
#define SG_LOG(depth, sfp, fmt, a...) do { } while (0)
#endif	/* end of CONFIG_SCSI_LOGGING && SG_DEBUG conditional */

/*
 * Unless otherwise noted, functions that return int will return 0 for
 * good/successful or a negated errno value. Here is list of errno_s generated
 * by this driver:
 *
 * E2BIG    sum(dlen) > tot_fd_thresh ; slave dxfer_len > master dxfer_len
 * EACCES   user (process) does not have sufficient privilege or capabilities
 * EADDRINUSE	sharing: slave file descriptor already in share
 * EADDRNOTAVAIL   sharing: master file descriptor already in share
 *		   slave request but no preceding master request
 * EAGAIN   [aka EWOULDBLOCK]; occurs when O_NONBLOCK set on open() or
 *	    SGV4_FLAG_IMMED given, and SG_IORECEIVE (or read(2)) not ready
 * EBUSY    'Device or resource busy'; this uses open(O_NONBLOCK) but another
 *	    has open(O_EXCL); reserve request in use (e.g. when mmap() called)
 * EDOM     numerical error, command queueing false and second command
 *	    attempted when one already outstanding, mrq pack_id
 * EFAULT   problem moving data to or from user space
 * EFBIG    too many reserve requests on this file descriptor
 * EIDRM    block request unexpectedly missing
 * EINTR    interrupted system call (generated by kernel, not this driver)
 * EINVAL   flags or other input information contradicts or disallowed
 * EIO      only kept for backward compatibility, too generic to be useful
 * ELOOP    sharing: file descriptor can't share with itself
 * EMSGSIZE    cdb too long (> 252 bytes) or too short (less than 6 bytes)
 * ENODATA  sharing: no data xfer requested; mmap or direct io problem
 *          SG_IOABORT: no match on pack_id or tag; mrq: no active reqs
 * ENODEV   target (SCSI) device associated with the fd has "disappeared"
 * ENOMEM   obvious; could be some pre-allocated cache that is full
 * ENOMSG   data transfer setup needed or (direction) disallowed (sharing)
 * ENOSTR   slave request abandoned due to master error or state
 * ENOTSOCK   sharing: file descriptor for sharing unassociated with sg driver
 * ENXIO    'no such device or address' SCSI mid-level processing errors
 *          (e.g. command timeouts); also sg info not in 'file' struct
 * EPERM    not permitted (even if has ACCES); v1+2,v3,v4 interface usage
 *	    violation, opened read-only but SCSI command not listed read-only
 * EPROTO   logic error (in driver); like "shouldn't get here"
 * EPROTOTYPE    atomic state change failed unexpectedly
 * ERANGE   multiple requests: usually bad flag values
 * ERESTARTSYS   should not be seen in user space, associated with an
 *		 interruptable wait; will restart system call or give EINTR
 * EWOULDBLOCK   [aka EAGAIN]; additionally if the 'more async' flag is set
 *		 SG_IOSUBMIT may yield this error
 */

/*
 * The SCSI interfaces that use read() and write() as an asynchronous variant of
 * ioctl(..., SG_IO, ...) are fundamentally unsafe, since there are lots of ways
 * to trigger read() and write() calls from various contexts with elevated
 * privileges. This can lead to kernel memory corruption (e.g. if these
 * interfaces are called through splice()) and privilege escalation inside
 * userspace (e.g. if a process with access to such a device passes a file
 * descriptor to a SUID binary as stdin/stdout/stderr).
 *
 * This function provides protection for the legacy API by restricting the
 * calling context.
 */
static int
sg_check_file_access(struct file *filp, const char *caller)
{
	/* can't put following in declarations where it belongs */
	compiletime_assert(SZ_SG_IO_V4 >= SZ_SG_IO_HDR,
			   "struct sg_io_v4 should be larger than sg_io_hdr");

	if (unlikely(filp->f_cred != current_real_cred())) {
		pr_err_once("%s: process %d (%s) changed security contexts after opening file descriptor, this is not allowed.\n",
			caller, task_tgid_vnr(current), current->comm);
		return -EPERM;
	}
	if (unlikely(uaccess_kernel())) {
		pr_err_once("%s: process %d (%s) called from kernel context, this is not allowed.\n",
			caller, task_tgid_vnr(current), current->comm);
		return -EACCES;
	}
	return 0;
}

static int
sg_wait_open_event(struct sg_device *sdp, bool o_excl)
		__must_hold(&sdp->open_rel_lock)
{
	int res = 0;

	if (unlikely(o_excl)) {
		while (atomic_read(&sdp->open_cnt) > 0) {
			mutex_unlock(&sdp->open_rel_lock);
			res = wait_event_interruptible
					(sdp->open_wait,
					 (unlikely(SG_IS_DETACHING(sdp)) ||
					  atomic_read(&sdp->open_cnt) == 0));
			mutex_lock(&sdp->open_rel_lock);

			if (unlikely(res)) /* -ERESTARTSYS */
				return res;
			if (unlikely(SG_IS_DETACHING(sdp)))
				return -ENODEV;
		}
	} else {
		while (unlikely(SG_HAVE_EXCLUDE(sdp))) {
			mutex_unlock(&sdp->open_rel_lock);
			res = wait_event_interruptible
					(sdp->open_wait,
					 (unlikely(SG_IS_DETACHING(sdp)) ||
					  !SG_HAVE_EXCLUDE(sdp)));
			mutex_lock(&sdp->open_rel_lock);

			if (unlikely(res)) /* -ERESTARTSYS */
				return res;
			if (unlikely(SG_IS_DETACHING(sdp)))
				return -ENODEV;
		}
	}
	return res;
}

/*
 * scsi_block_when_processing_errors() returns 0 when dev was taken offline by
 * error recovery, 1 otherwise (i.e. okay). Even if in error recovery, let
 * user continue if O_NONBLOCK set. Permits SCSI commands to be issued during
 * error recovery. Tread carefully.
 * Returns 0 for ok (i.e. allow), -EPROTO if sdp is NULL, otherwise -ENXIO .
 */
static inline int
sg_allow_if_err_recovery(struct sg_device *sdp, bool non_block)
{
	if (unlikely(!sdp))
		return -EPROTO;
	if (unlikely(SG_IS_DETACHING(sdp)))
		return -ENODEV;
	if (non_block)
		return 0;
	if (likely(scsi_block_when_processing_errors(sdp->device)))
		return 0;
	return -ENXIO;
}

/*
 * Corresponds to the open() system call on sg devices. Implements O_EXCL on
 * a per device basis using 'open_cnt'. If O_EXCL and O_NONBLOCK and there is
 * already a sg handle open on this device then it fails with an errno of
 * EBUSY. Without the O_NONBLOCK flag then this thread enters an interruptible
 * wait until the other handle(s) are closed.
 */
static int
sg_open(struct inode *inode, struct file *filp)
{
	bool o_excl, non_block;
	int res;
	int min_dev = iminor(inode);
	int op_flags = filp->f_flags;
	struct sg_device *sdp;
	struct sg_fd *sfp;

	nonseekable_open(inode, filp);
	o_excl = !!(op_flags & O_EXCL);
	non_block = !!(op_flags & O_NONBLOCK);
	if (unlikely(o_excl) && ((op_flags & O_ACCMODE) == O_RDONLY))
		return -EPERM;/* not permitted, need write access for O_EXCL */
	sdp = sg_get_dev(min_dev);
	if (IS_ERR(sdp))
		return PTR_ERR(sdp);

	/* Prevent the device driver from vanishing while we sleep */
	res = scsi_device_get(sdp->device);
	if (unlikely(res))
		goto sg_put;
	res = scsi_autopm_get_device(sdp->device);
	if (unlikely(res))
		goto sdp_put;
	res = sg_allow_if_err_recovery(sdp, non_block);
	if (unlikely(res))
		goto error_out;

	mutex_lock(&sdp->open_rel_lock);
	if (op_flags & O_NONBLOCK) {
		if (unlikely(o_excl)) {
			if (atomic_read(&sdp->open_cnt) > 0) {
				res = -EBUSY;
				goto error_mutex_locked;
			}
		} else {
			if (unlikely(SG_HAVE_EXCLUDE(sdp))) {
				res = -EBUSY;
				goto error_mutex_locked;
			}
		}
	} else {
		res = sg_wait_open_event(sdp, o_excl);
		if (unlikely(res)) /* -ERESTARTSYS or -ENODEV */
			goto error_mutex_locked;
	}

	/* N.B. at this point we are holding the open_rel_lock */
	if (unlikely(o_excl))
		set_bit(SG_FDEV_EXCLUDE, sdp->fdev_bm);

	if (atomic_read(&sdp->open_cnt) < 1)	/* no existing opens */
		sg_calc_sgat_param(sdp);
	sfp = sg_add_sfp(sdp, filp);
	if (IS_ERR(sfp)) {
		res = PTR_ERR(sfp);
		goto out_undo;
	}

	filp->private_data = sfp;
	sfp->tid = (current ? current->pid : -1);
	atomic_inc(&sdp->open_cnt);
	mutex_unlock(&sdp->open_rel_lock);
	SG_LOG(3, sfp, "%s: minor=%d, op_flags=0x%x; %s count prior=%d%s\n",
	       __func__, min_dev, op_flags, "device open",
	       atomic_read(&sdp->open_cnt),
	       ((op_flags & O_NONBLOCK) ? " O_NONBLOCK" : ""));

	res = 0;
sg_put:
	kref_put(&sdp->d_ref, sg_device_destroy);
	return res;

out_undo:
	if (o_excl) {		/* undo if error */
		clear_bit(SG_FDEV_EXCLUDE, sdp->fdev_bm);
		wake_up_interruptible(&sdp->open_wait);
	}
error_mutex_locked:
	mutex_unlock(&sdp->open_rel_lock);
error_out:
	scsi_autopm_put_device(sdp->device);
sdp_put:
	scsi_device_put(sdp->device);
	goto sg_put;
}

/*
 * Release resources associated with a prior, successful sg_open(). It can be
 * seen as the (final) close() call on a sg device file descriptor in the user
 * space. The real work releasing all resources associated with this file
 * descriptor is done by sg_uc_remove_sfp() which is scheduled by
 * sg_remove_sfp().
 */
static int
sg_release(struct inode *inode, struct file *filp)
{
	struct sg_device *sdp;
	struct sg_fd *sfp;

	sfp = filp->private_data;
	sdp = sfp->parentdp;
	SG_LOG(3, sfp, "%s: device open count prior=%d\n", __func__,
	       atomic_read(&sdp->open_cnt));
	if (unlikely(!sdp))
		return -ENXIO;
	set_bit(SG_FFD_RELEASE, sfp->ffd_bm);

	mutex_lock(&sdp->open_rel_lock);
	scsi_autopm_put_device(sdp->device);
	kref_put(&sfp->f_ref, sg_remove_sfp);
	atomic_dec(&sdp->open_cnt);

	/*
	 * Possibly many open()s waiting on exlude clearing, start many;
	 * only open(O_EXCL)s wait on 0==open_cnt so only start one.
	 */
	if (test_and_clear_bit(SG_FDEV_EXCLUDE, sdp->fdev_bm))
		wake_up_interruptible_all(&sdp->open_wait);
	else if (atomic_read(&sdp->open_cnt) == 0)
		wake_up_interruptible(&sdp->open_wait);
	mutex_unlock(&sdp->open_rel_lock);
	return 0;
}

/*
 * ***********************************************************************
 * write(2) related functions follow. They are shown before read(2) related
 * functions. That is because SCSI commands/requests are first "written" to
 * the SCSI device by using write(2), ioctl(SG_IOSUBMIT) or the first half
 * of the synchronous ioctl(SG_IO) system call.
 * ***********************************************************************
 */

/* This is the write(2) system call entry point. v4 interface disallowed. */
static ssize_t
sg_write(struct file *filp, const char __user *p, size_t count, loff_t *ppos)
{
	int mxsize, cmd_size, input_size, res;
	u8 opcode;
	struct sg_device *sdp;
	struct sg_fd *sfp;
	struct sg_request *srp;
	struct sg_header ov2hdr;
	struct sg_io_hdr v3hdr;
	struct sg_header *ohp = &ov2hdr;
	struct sg_io_hdr *h3p = &v3hdr;
	struct sg_comm_wr_t cwr;

	res = sg_check_file_access(filp, __func__);
	if (unlikely(res))
		return res;

	sfp = filp->private_data;
	sdp = sfp->parentdp;
	SG_LOG(3, sfp, "%s: write(3rd arg) count=%d\n", __func__, (int)count);
	res = sg_allow_if_err_recovery(sdp, !!(filp->f_flags & O_NONBLOCK));
	if (unlikely(res))
		return res;

	if (unlikely(count < SZ_SG_HEADER))
		return -EIO;
	if (unlikely(copy_from_user(ohp, p, SZ_SG_HEADER)))
		return -EFAULT;
	if (ohp->reply_len < 0) {	/* assume this is v3 */
		struct sg_io_hdr *reinter_2p = (struct sg_io_hdr *)ohp;
		u8 *h3u8p = (u8 *)&v3hdr;

		if (unlikely(count < SZ_SG_IO_HDR))
			return -EIO;
		if (unlikely(reinter_2p->interface_id != 'S')) {
			pr_info_once("sg: %s: v3 interface only here\n",
				     __func__);
			return -EPERM;
		}
		WARN_ONCE(true, "Please use %s instead of write(),\n%s\n",
			  "ioctl(SG_SUBMIT_V3)",
			  "  See: http://sg.danny.cz/sg/sg_v40.html");
		memcpy(h3p, ohp, SZ_SG_HEADER);
		if (copy_from_user(h3u8p + SZ_SG_HEADER, p + SZ_SG_HEADER,
				   SZ_SG_IO_HDR - SZ_SG_HEADER))
			return -EFAULT;
		res = sg_v3_submit(sfp, h3p, false, NULL);
		return unlikely(res < 0) ? res : (int)count;
	}
	/* v1 and v2 interfaces processed below this point */
	if (unlikely(count < (SZ_SG_HEADER + 6)))
		return -EIO;    /* minimum scsi command length is 6 bytes */
	p += SZ_SG_HEADER;
	__get_user(opcode, p);
	mutex_lock(&sfp->f_mutex);
	if (unlikely(sfp->next_cmd_len > 0)) {
		cmd_size = sfp->next_cmd_len;
		sfp->next_cmd_len = 0;	/* reset, only this write() effected */
	} else {
		cmd_size = COMMAND_SIZE(opcode);  /* old: SCSI command group */
		if (opcode >= 0xc0 && ohp->twelve_byte)
			cmd_size = 12;
	}
	mutex_unlock(&sfp->f_mutex);
	SG_LOG(4, sfp, "%s:   scsi opcode=0x%02x, cmd_size=%d\n", __func__,
	       (unsigned int)opcode, cmd_size);
	input_size = count - cmd_size;
	mxsize = max_t(int, input_size, ohp->reply_len);
	mxsize -= SZ_SG_HEADER;
	input_size -= SZ_SG_HEADER;
	if (unlikely(input_size < 0))
		return -EIO; /* Insufficient bytes passed for this command. */
	memset(h3p, 0, sizeof(*h3p));
	h3p->interface_id = '\0';/* indicate v1 or v2 interface (tunnelled) */
	h3p->cmd_len = (u8)cmd_size;
	h3p->iovec_count = 0;
	h3p->mx_sb_len = 0;
	if (input_size > 0)
		h3p->dxfer_direction = (ohp->reply_len > SZ_SG_HEADER) ?
		    SG_DXFER_TO_FROM_DEV : SG_DXFER_TO_DEV;
	else
		h3p->dxfer_direction = (mxsize > 0) ? SG_DXFER_FROM_DEV :
						      SG_DXFER_NONE;
	h3p->dxfer_len = mxsize;
	if (h3p->dxfer_direction == SG_DXFER_TO_DEV ||
	    h3p->dxfer_direction == SG_DXFER_TO_FROM_DEV)
		h3p->dxferp = (u8 __user *)p + cmd_size;
	else
		h3p->dxferp = NULL;
	h3p->sbp = NULL;
	h3p->timeout = ohp->reply_len;	/* structure abuse ... */
	h3p->flags = input_size;	/* structure abuse ... */
	h3p->pack_id = ohp->pack_id;
	h3p->usr_ptr = NULL;
	/*
	 * SG_DXFER_TO_FROM_DEV is functionally equivalent to SG_DXFER_FROM_DEV,
	 * but it is possible that the app intended SG_DXFER_TO_DEV, because
	 * there is a non-zero input_size, so emit a warning.
	 */
	if (h3p->dxfer_direction == SG_DXFER_TO_FROM_DEV) {
		printk_ratelimited
			(KERN_WARNING
			 "%s: data in/out %d/%d bytes for SCSI command 0x%x-- guessing data in;\n"
			 "   program %s not setting count and/or reply_len properly\n",
			 __func__, ohp->reply_len - (int)SZ_SG_HEADER,
			 input_size, (unsigned int)opcode, current->comm);
	}
	memset(&cwr, 0, sizeof(cwr));
	cwr.h3p = h3p;
	cwr.timeout = sfp->timeout;
	cwr.cmd_len = cmd_size;
	cwr.sfp = sfp;
	cwr.u_cmdp = p;
	srp = sg_common_write(&cwr);
	return (IS_ERR(srp)) ? PTR_ERR(srp) : (int)count;
}

static int
sg_fetch_cmnd(struct sg_fd *sfp, const u8 __user *u_cdbp, int len, u8 *cdbp)
{
	if (unlikely(!u_cdbp || len < 6 || len > SG_MAX_CDB_SIZE))
		return -EMSGSIZE;
	if (unlikely(copy_from_user(cdbp, u_cdbp, len)))
		return -EFAULT;
	if (O_RDWR != (sfp->filp->f_flags & O_ACCMODE)) {	/* read-only */
		switch (sfp->parentdp->device->type) {
		case TYPE_DISK:
		case TYPE_RBC:
		case TYPE_ZBC:
			return blk_verify_command(cdbp, sfp->filp->f_mode);
		default:	/* SSC, SES, etc cbd_s may differ from SBC */
			break;
		}
	}
	return 0;
}

static int
sg_v3_submit(struct sg_fd *sfp, struct sg_io_hdr *hp, bool sync,
	     struct sg_request **o_srp)
{
	unsigned long ul_timeout;
	struct sg_request *srp;
	struct sg_comm_wr_t cwr;

	/* now doing v3 blocking (sync) or non-blocking submission */
	if (unlikely(hp->flags & SGV4_FLAG_MULTIPLE_REQS))
		return -ERANGE;		/* need to use v4 interface */
	if (hp->flags & SG_FLAG_MMAP_IO) {
		if (unlikely(!list_empty(&sfp->rq_list)))
			return -EBUSY;  /* already active requests on fd */
		if (unlikely(hp->dxfer_len >
			     sfp->rsv_arr[0].rsv_srp->sgatp->buflen))
			return -ENOMEM; /* MMAP_IO size must fit in reserve */
		if (unlikely(hp->flags & SG_FLAG_DIRECT_IO))
			return -EINVAL; /* not both MMAP_IO and DIRECT_IO */
	}
	/* when v3 seen, allow cmd_q on this fd (def: cmd_q) */
	if (test_bit(SG_FFD_NO_CMD_Q, sfp->ffd_bm))
		clear_bit(SG_FFD_NO_CMD_Q, sfp->ffd_bm);
	ul_timeout = msecs_to_jiffies(hp->timeout);
	memset(&cwr, 0, sizeof(cwr));
	__assign_bit(SG_FRQ_SYNC_INVOC, cwr.frq_bm, (int)sync);
	cwr.h3p = hp;
	cwr.timeout = min_t(unsigned long, ul_timeout, INT_MAX);
	cwr.cmd_len = hp->cmd_len;
	cwr.sfp = sfp;
	cwr.u_cmdp = hp->cmdp;
	srp = sg_common_write(&cwr);
	if (IS_ERR(srp))
		return PTR_ERR(srp);
	if (o_srp)
		*o_srp = srp;
	return 0;
}

static void
sg_sgv4_out_zero(struct sg_io_v4 *h4p)
{
	const int off = offsetof(struct sg_io_v4, driver_status);

	/* clear from and including driver_status to end of object */
	memset((u8 *)h4p + off, 0, SZ_SG_IO_V4 - off);
}

/*
 * Takes a pointer (cop) to the multiple request (mrq) control object and
 * a pointer to the command array. The command array (with tot_reqs elements)
 * is written out (flushed) to user space pointer cop->din_xferp. The
 * secondary error value (s_res) is placed in the array last element's
 * spare_out field iff it was zero. Returns 0 on success.
 */
static int
sg_mrq_arr_flush(struct sg_io_v4 *cop, struct sg_io_v4 *a_hds,
		 u32 tot_reqs, int s_res)
{
	u32 sz = tot_reqs * SZ_SG_IO_V4;
	void __user *p = uptr64(cop->din_xferp);

	if (unlikely(s_res)) {
		a_hds[tot_reqs - 1].spare_out = -s_res;
		cop->spare_out = -s_res;
	}
	if (!p)
		return 0;
	if (sz > cop->din_xfer_len)
		sz = cop->din_xfer_len;
	if (sz > 0) {
		if (unlikely(copy_to_user(p, a_hds, sz)))
			return -EFAULT;
	}
	return 0;
}

static int
sg_mrq_1complet(struct sg_io_v4 *cop, struct sg_io_v4 *a_hds,
		struct sg_fd *w_sfp, struct sg_fd *sec_sfp, int tot_reqs,
		struct sg_request *srp)
{
	int s_res, indx;
	struct sg_io_v4 *siv4p;

	SG_LOG(3, w_sfp, "%s: start\n", __func__);
	if (unlikely(!srp))
		return -EPROTO;
	indx = srp->s_hdr4.mrq_ind;
	if (unlikely(indx < 0 || indx >= tot_reqs))
		return -EPROTO;
	siv4p = a_hds + indx;
	s_res = sg_v4_receive(w_sfp, srp, NULL, siv4p);
	if (unlikely(s_res == -EFAULT))
		return s_res;
	siv4p->info |= SG_INFO_MRQ_FINI;
	if (siv4p->flags & SGV4_FLAG_SIG_ON_OTHER) {
		s_res = sg_mrq_arr_flush(cop, a_hds, tot_reqs, s_res);
		if (unlikely(s_res))	/* can only be -EFAULT */
			return s_res;
		kill_fasync(&sec_sfp->async_qp, SIGPOLL, POLL_IN);
	}
	return 0;
}

static int
sg_wait_mrq_event(struct sg_fd *sfp, struct sg_request **srpp)
{
	if (test_bit(SG_FFD_EXCL_WAITQ, sfp->ffd_bm))
		return __wait_event_interruptible_exclusive
					(sfp->cmpl_wait,
					 sg_mrq_get_ready_srp(sfp, srpp));
	return __wait_event_interruptible(sfp->cmpl_wait,
					  sg_mrq_get_ready_srp(sfp, srpp));
}

/*
 * With special variable blocking (svb), events for both master and slave are
 * forwarded on the master's file descriptor. Each event increments svb_cnt.
 */
static int
sg_wait_mrq_svb(struct sg_fd *sfp)
{
	if (test_bit(SG_FFD_EXCL_WAITQ, sfp->ffd_bm))
		return __wait_event_interruptible_exclusive
					(sfp->cmpl_wait,
					 atomic_read(&sfp->svb_cnt) > 0);
	return __wait_event_interruptible(sfp->cmpl_wait,
					  atomic_read(&sfp->svb_cnt) > 0);
}

/*
 * This is a fair-ish algorithm for an interruptible wait on two file
 * descriptors. It favours the main fd over the secondary fd (sec_sfp).
 * Increments cop->info for each successful completion.
 */
static int
sg_mrq_complets(struct sg_io_v4 *cop, struct sg_io_v4 *a_hds,
		struct sg_fd *sfp, struct sg_fd *sec_sfp, int tot_reqs,
		int mreqs, int sec_reqs, bool is_svb)
{
	int res = 0;
	int rres;
	struct sg_request *srp;

	SG_LOG(3, sfp, "%s: mreqs=%d, sec_reqs=%d\n", __func__, mreqs,
	       sec_reqs);
	while (mreqs + sec_reqs > 0) {
		while (mreqs > 0 && sg_mrq_get_ready_srp(sfp, &srp)) {
			if (IS_ERR(srp)) {	/* -ENODATA: no mrqs here */
				if (PTR_ERR(srp) == -ENODATA)
					break;
				res = PTR_ERR(srp);
				break;
			} else {
				--mreqs;
				res = sg_mrq_1complet(cop, a_hds, sfp,
						      sec_sfp, tot_reqs, srp);
				if (unlikely(res))
					return res;
				++cop->info;
				if (cop->din_xfer_len > 0)
					--cop->din_resid;
				if (is_svb)
					atomic_dec(&sfp->svb_cnt);
			}
		}
		while (sec_reqs > 0 && sg_mrq_get_ready_srp(sec_sfp, &srp)) {
			if (IS_ERR(srp)) {
				if (PTR_ERR(srp) == -ENODATA)
					break;
				res = PTR_ERR(srp);
				break;
			} else {
				--sec_reqs;
				rres = sg_mrq_1complet(cop, a_hds, sec_sfp,
						       sec_sfp, tot_reqs, srp);
				if (unlikely(rres))
					return rres;
				++cop->info;
				if (cop->din_xfer_len > 0)
					--cop->din_resid;
				if (is_svb)	/* svb count is on _master_ */
					atomic_dec(&sfp->svb_cnt);
			}
		}
		if (res)
			break;
		if (is_svb) {
			struct sg_fd *ma_sfp = sfp;

			if (mreqs + sec_reqs <= 0)
				continue;
			res = sg_wait_mrq_svb(ma_sfp);
			if (unlikely(res))
				return res;	/* signal --> -ERESTARTSYS */
			continue;	/* while loops above should process */
		}
		/* only ! is_svb case from here to end of loop */
		if (mreqs > 0) {
			res = sg_wait_mrq_event(sfp, &srp);
			if (unlikely(res))
				return res;	/* signal --> -ERESTARTSYS */
			if (IS_ERR(srp)) {
				mreqs = 0;
			} else {
				--mreqs;
				res = sg_mrq_1complet(cop, a_hds, sfp,
						      sec_sfp, tot_reqs, srp);
				if (unlikely(res))
					return res;
				++cop->info;
				if (cop->din_xfer_len > 0)
					--cop->din_resid;
			}
		}
		if (sec_reqs > 0) {
			res = sg_wait_mrq_event(sec_sfp, &srp);
			if (unlikely(res))
				return res;	/* signal --> -ERESTARTSYS */
			if (IS_ERR(srp)) {
				sec_reqs = 0;
			} else {
				--sec_reqs;
				res = sg_mrq_1complet(cop, a_hds, sec_sfp,
						      sec_sfp, tot_reqs, srp);
				if (unlikely(res))
					return res;
				++cop->info;
				if (cop->din_xfer_len > 0)
					--cop->din_resid;
			}
		}
	}	/* end of outer while loop (while requests still inflight) */
	return res;
}

static int
sg_mrq_sanity(struct sg_device *sdp, struct sg_io_v4 *cop,
	      struct sg_io_v4 *a_hds, u8 *cdb_ap, struct sg_fd *sfp,
	      bool immed, u32 tot_reqs, bool *share_on_othp)
{
	bool have_mrq_sense = (cop->response && cop->max_response_len);
	bool share_on_oth = false;
	bool share;
	int k;
	u32 cdb_alen = cop->request_len;
	u32 cdb_mxlen = cdb_alen / tot_reqs;
	u32 flags;
	struct sg_fd *o_sfp = sfp->share_sfp;
	struct sg_io_v4 *hp;
	__maybe_unused const char *rip = "request index";

	/* Pre-check each request for anomalies, plus some preparation */
	for (k = 0, hp = a_hds; k < tot_reqs; ++k, ++hp) {
		flags = hp->flags;
		sg_sgv4_out_zero(hp);
		if (unlikely(hp->guard != 'Q' || hp->protocol != 0 ||
			     hp->subprotocol != 0)) {
			SG_LOG(1, sfp, "%s: req index %u: %s or protocol\n",
			       __func__, k, "bad guard");
			return -ERANGE;
		}
		if (unlikely(flags & SGV4_FLAG_MULTIPLE_REQS)) {
			SG_LOG(1, sfp, "%s: %s %u: no nested multi-reqs\n",
			       __func__, rip, k);
			return -ERANGE;
		}
		share = !!(flags & SGV4_FLAG_SHARE);
		if (immed) {	/* only accept async submits on current fd */
			if (unlikely(flags & SGV4_FLAG_DO_ON_OTHER)) {
				SG_LOG(1, sfp, "%s: %s %u, %s\n", __func__,
				       rip, k, "no IMMED with ON_OTHER");
				return -ERANGE;
			} else if (unlikely(share)) {
				SG_LOG(1, sfp, "%s: %s %u, %s\n", __func__,
				       rip, k, "no IMMED with FLAG_SHARE");
				return -ERANGE;
			} else if (unlikely(flags & SGV4_FLAG_COMPLETE_B4)) {
				SG_LOG(1, sfp, "%s: %s %u, %s\n", __func__,
				       rip, k, "no IMMED with COMPLETE_B4");
				return -ERANGE;
			}
			/* N.B. SGV4_FLAG_SIG_ON_OTHER is allowed */
		}
		if (o_sfp) {
			if (!share_on_oth && share)
				share_on_oth = true;
		} else {
			if (unlikely(share)) {
				SG_LOG(1, sfp, "%s: %s %u, no share\n",
				       __func__, rip, k);
				return -ERANGE;
			} else if (unlikely(flags & SGV4_FLAG_SIG_ON_OTHER)) {
				SG_LOG(1, sfp, "%s: %s %u, %s signal on\n",
				       __func__, rip, k, "no other fd to");
				return -ERANGE;
			} else if (unlikely(flags & SGV4_FLAG_DO_ON_OTHER)) {
				SG_LOG(1, sfp, "%s: %s %u, %s do on\n",
				       __func__, rip, k, "no other fd to");
				return -ERANGE;
			}
		}
		if (cdb_ap) {
			if (unlikely(hp->request_len > cdb_mxlen)) {
				SG_LOG(1, sfp, "%s: %s %u, cdb too long\n",
				       __func__, rip, k);
				return -ERANGE;
			}
		}
		if (have_mrq_sense && hp->response == 0 &&
		    hp->max_response_len == 0) {
			hp->response = cop->response;
			hp->max_response_len = cop->max_response_len;
		}
		/*
		 * if NO_WAITQ flag set on cop (mrq control object) then set
		 * that flag on each element of the request array _unless_ the
		 * IMMED flag is already set on that element.
		 */
		if ((cop->flags & SGV4_FLAG_NO_WAITQ) &&
		    !(hp->flags & SGV4_FLAG_IMMED))
			hp->flags |= SGV4_FLAG_NO_WAITQ;
	}
	if (share_on_othp)
		*share_on_othp = share_on_othp;
	return 0;
}

static bool
sg_mrq_svb_chk(struct sg_io_v4 *a_hds, u32 tot_reqs)
{
	bool expect_rd;
	int k;
	u32 flags;
	struct sg_io_v4 *hp;

	/* expect read-write pairs, all with SGV4_FLAG_NO_DXFER set */
	for (k = 0, hp = a_hds, expect_rd = true; k < tot_reqs; ++k, ++hp) {
		flags = hp->flags;
		if (flags & (SGV4_FLAG_COMPLETE_B4 | SGV4_FLAG_SIG_ON_OTHER))
			return false;
		if (expect_rd) {
			if (hp->dout_xfer_len > 0)
				return false;
			if (hp->din_xfer_len > 0) {
				if (!(flags & SGV4_FLAG_SHARE))
					return false;
				if (flags & SGV4_FLAG_DO_ON_OTHER)
					return false;
				expect_rd = false;
			}
			/* allowing commands with no dxfer */
		} else {	/* checking write side */
			if (hp->dout_xfer_len > 0) {
				if (~flags &
				    (SGV4_FLAG_NO_DXFER | SGV4_FLAG_SHARE |
				     SGV4_FLAG_DO_ON_OTHER))
					return false;
				expect_rd = true;
			}
			if (hp->din_xfer_len > 0)
				return false;
		}
	}
	if (!expect_rd)
		return false;
	return true;
}

static int
sg_process_most_mrq(struct sg_fd *fp, struct sg_fd *o_sfp,
		    struct sg_mrq_hold *mhp)
{
	int flags;
	int num_subm = 0;
	int num_cmpl = 0;
	int res = 0;
	int other_fp_sent = 0;
	int this_fp_sent = 0;
	const int shr_complet_b4 = SGV4_FLAG_SHARE | SGV4_FLAG_COMPLETE_B4;
	unsigned long ul_timeout;
	struct sg_fd *rq_sfp;
	struct sg_io_v4 *cop = mhp->cwrp->h4p;
	struct sg_io_v4 *hp;		/* ptr to request object in a_hds */
	struct sg_request *srp;
	struct sg_comm_wr_t r_cwr;
	struct sg_comm_wr_t *r_cwrp = &r_cwr;

	SG_LOG(3, fp, "%s: id_of_mrq=%d, tot_reqs=%d, enter\n", __func__,
	       mhp->id_of_mrq, mhp->tot_reqs);
	/* Dispatch (submit) requests and optionally wait for response */
	for (hp = mhp->a_hds; num_subm < mhp->tot_reqs; ++hp) {
		if (mhp->chk_abort && test_and_clear_bit(SG_FFD_MRQ_ABORT,
							 fp->ffd_bm)) {
			SG_LOG(1, fp, "%s: id_of_mrq=%d aborting at ind=%d\n",
			       __func__, mhp->id_of_mrq, num_subm);
			break;	/* N.B. rest not submitted */
		}
		flags = hp->flags;
		rq_sfp = (flags & SGV4_FLAG_DO_ON_OTHER) ? o_sfp : fp;
		if (mhp->cdb_ap) {	/* already have array of cdbs */
			r_cwrp->cmdp = mhp->cdb_ap +
				       (num_subm * mhp->cdb_mxlen);
			r_cwrp->u_cmdp = NULL;
		} else {	/* fetch each cdb from user space */
			r_cwrp->cmdp = NULL;
			r_cwrp->u_cmdp = cuptr64(hp->request);
		}
		r_cwrp->cmd_len = hp->request_len;
		ul_timeout = msecs_to_jiffies(hp->timeout);
		r_cwrp->frq_bm[0] = 0;
		__assign_bit(SG_FRQ_SYNC_INVOC, r_cwrp->frq_bm,
			     (int)mhp->blocking);
		__set_bit(SG_FRQ_IS_V4I, r_cwrp->frq_bm);
		__set_bit(SG_FRQ_MULTI_REQ, r_cwrp->frq_bm);
		r_cwrp->h4p = hp;
		r_cwrp->timeout = min_t(unsigned long, ul_timeout, INT_MAX);
		r_cwrp->sfp = rq_sfp;
		srp = sg_common_write(r_cwrp);
		if (IS_ERR(srp)) {
			mhp->s_res = PTR_ERR(srp);
			break;
		}
		srp->s_hdr4.mrq_ind = num_subm++;
		if (mhp->chk_abort)
			atomic_set(&srp->s_hdr4.pack_id_of_mrq,
				   mhp->id_of_mrq);
		if (mhp->immed ||
		    (!(mhp->blocking || (flags & shr_complet_b4)))) {
			if (fp == rq_sfp)
				++this_fp_sent;
			else
				++other_fp_sent;
			continue;  /* defer completion until all submitted */
		}
		mhp->s_res = sg_wait_event_srp(rq_sfp, NULL, hp, srp);
		if (unlikely(mhp->s_res)) {
			if (mhp->s_res == -ERESTARTSYS)
				return mhp->s_res;
			break;
		}
		++num_cmpl;
		hp->info |= SG_INFO_MRQ_FINI;
		if (mhp->stop_if && (hp->driver_status ||
				     hp->transport_status ||
				     hp->device_status)) {
			SG_LOG(2, fp, "%s: %s=0x%x/0x%x/0x%x] cause exit\n",
			       __func__, "STOP_IF and status [drv/tran/scsi",
			       hp->driver_status, hp->transport_status,
			       hp->device_status);
			break;	/* cop->driver_status <-- 0 in this case */
		}
		if (hp->flags & SGV4_FLAG_SIG_ON_OTHER) {
			res = sg_mrq_arr_flush(cop, mhp->a_hds, mhp->tot_reqs,
					       mhp->s_res);
			if (unlikely(res))
				break;
			kill_fasync(&o_sfp->async_qp, SIGPOLL, POLL_IN);
		}
	}	/* end of dispatch request and optionally wait response loop */
	cop->dout_resid = mhp->tot_reqs - num_subm;
	cop->info = mhp->immed ? num_subm : num_cmpl;
	if (cop->din_xfer_len > 0) {
		cop->din_resid = mhp->tot_reqs - num_cmpl;
		cop->spare_out = -mhp->s_res;
	}

	if (mhp->immed)
		return res;
	if (likely(res == 0 && (this_fp_sent + other_fp_sent) > 0)) {
		mhp->s_res = sg_mrq_complets(cop, mhp->a_hds, fp, o_sfp,
					     mhp->tot_reqs, this_fp_sent,
					     other_fp_sent, false);
		if (unlikely(mhp->s_res == -EFAULT ||
			     mhp->s_res == -ERESTARTSYS))
			res = mhp->s_res;	/* this may leave orphans */
	}
	if (mhp->id_of_mrq)	/* can no longer do a mrq abort */
		atomic_set(&fp->mrq_id_abort, 0);
	return res;
}

/*
 * Processes special variable blocking. The inner loop should use all the
 * available reserve requests on the master's sfp as it asynchronously
 * submits those master and their corresponding slave requests. After all
 * those responses are received, the outer loop works its way through the
 * multiple requests. So this is a semi-async algorithm.
 */
static int
sg_process_svb_mrq(struct sg_fd *fp, struct sg_fd *o_sfp,
		   struct sg_mrq_hold *mhp)
{
	int k, j, flags, rcv_before;
	int num_subm = 0;
	int num_cmpl = 0;
	int res = 0;
	int other_fp_sent = 0;
	int this_fp_sent = 0;
	const int inner_max = 2 * SG_MAX_RSV_REQS;
	unsigned long ul_timeout;
	struct sg_fd *rq_sfp;
	struct sg_io_v4 *cop = mhp->cwrp->h4p;
	struct sg_io_v4 *hp;		/* ptr to request object in a_hds */
	struct sg_request *srp;
	struct sg_comm_wr_t r_cwr;
	struct sg_comm_wr_t *r_cwrp = &r_cwr;

	SG_LOG(3, fp, "%s: id_of_mrq=%d, tot_reqs=%d, enter\n", __func__,
	       mhp->id_of_mrq, mhp->tot_reqs);
	/* Dispatch (submit) requests and optionally wait for response */
	for (hp = mhp->a_hds; num_subm < mhp->tot_reqs; ) {
		this_fp_sent = 0;
		other_fp_sent = 0;
		for (k = 0, j = 0; k < inner_max && num_subm < mhp->tot_reqs;
		     ++hp, ++j) {
			if (mhp->chk_abort &&
			    test_and_clear_bit(SG_FFD_MRQ_ABORT, fp->ffd_bm)) {
				SG_LOG(1, fp,
				       "%s: id_of_mrq=%d aborting at pos=%d\n",
				       __func__, mhp->id_of_mrq, num_subm);
				break;	/* N.B. rest not submitted */
			}
			flags = hp->flags;
			if (flags & SGV4_FLAG_DO_ON_OTHER) {
				SG_LOG(6, o_sfp, "%s: submitting on slave\n",
				       __func__);
				rq_sfp = o_sfp;
				if (hp->dout_xfer_len > 0)
					++k;
			} else {
				SG_LOG(6, fp, "%s: submitting on master\n",
				       __func__);
				rq_sfp = fp;
				if (hp->din_xfer_len > 0)
					++k;
			}
			if (mhp->cdb_ap) {	/* already have array of cdbs */
				r_cwrp->cmdp = mhp->cdb_ap +
					       (num_subm * mhp->cdb_mxlen);
				r_cwrp->u_cmdp = NULL;
			} else {	/* fetch each cdb from user space */
				r_cwrp->cmdp = NULL;
				r_cwrp->u_cmdp = cuptr64(hp->request);
			}
			r_cwrp->cmd_len = hp->request_len;
			ul_timeout = msecs_to_jiffies(hp->timeout);
			r_cwrp->frq_bm[0] = 0;
			__clear_bit(SG_FRQ_SYNC_INVOC, r_cwrp->frq_bm);
			__set_bit(SG_FRQ_IS_V4I, r_cwrp->frq_bm);
			__set_bit(SG_FRQ_WAKE_MASTER, r_cwrp->frq_bm);
			__set_bit(SG_FRQ_MULTI_REQ, r_cwrp->frq_bm);
			r_cwrp->h4p = hp;
			r_cwrp->timeout = min_t(unsigned long, ul_timeout,
						INT_MAX);
			r_cwrp->sfp = rq_sfp;
			srp = sg_common_write(r_cwrp);
			if (IS_ERR(srp)) {
				mhp->s_res = PTR_ERR(srp);
				res = mhp->s_res;	/* don't loop again */
				break;
			}
			srp->s_hdr4.mrq_ind = num_subm++;
			if (mhp->chk_abort)
				atomic_set(&srp->s_hdr4.pack_id_of_mrq,
					   mhp->id_of_mrq);
			if (fp == rq_sfp)
				++this_fp_sent;
			else
				++other_fp_sent;
		}
		if (this_fp_sent + other_fp_sent <= 0)
			break;
		rcv_before = cop->info;
		mhp->s_res = sg_mrq_complets(cop, mhp->a_hds, fp, o_sfp,
					     mhp->tot_reqs, this_fp_sent,
					     other_fp_sent, true);
		if (unlikely(mhp->s_res == -EFAULT ||
			     mhp->s_res == -ERESTARTSYS))
			res = mhp->s_res;	/* this may leave orphans */
		num_cmpl += (cop->info - rcv_before);
		if (res)
			break;
	}	/* end of outer for loop */

	cop->dout_resid = mhp->tot_reqs - num_subm;
	if (cop->din_xfer_len > 0) {
		cop->din_resid = mhp->tot_reqs - num_cmpl;
		cop->spare_out = -mhp->s_res;
	}
	if (mhp->id_of_mrq)	/* can no longer do a mrq abort */
		atomic_set(&fp->mrq_id_abort, 0);
	return res;
}

/*
 * Implements the multiple request functionality. When 'blocking' is true
 * invocation was via ioctl(SG_IO), otherwise it was via ioctl(SG_IOSUBMIT).
 * Submit non-blocking if IMMED flag given or when ioctl(SG_IOSUBMIT)
 * is used with O_NONBLOCK set on its file descriptor. Waitless non-blocking
 * is when the NO_WAITQ flag is given.
 */
static int
sg_do_multi_req(struct sg_comm_wr_t *cwrp, bool blocking)
{
	bool f_non_block, no_waitq, share_on_oth;
	int res = 0;
	int existing_id;
	u32 cdb_mxlen;
	struct sg_io_v4 *cop = cwrp->h4p;	/* controlling object */
	u32 blen = cop->dout_xfer_len;
	u32 cdb_alen = cop->request_len;
	u32 tot_reqs = blen / SZ_SG_IO_V4;
	u8 *cdb_ap = NULL;
	struct sg_io_v4 *a_hds;		/* array of request objects */
	struct sg_fd *fp = cwrp->sfp;
	struct sg_fd *o_sfp = fp->share_sfp;
	struct sg_device *sdp = fp->parentdp;
	struct sg_mrq_hold mh;
	struct sg_mrq_hold *mhp = &mh;

	memset(mhp, 0, sizeof(*mhp));
	mhp->cwrp = cwrp;
	mhp->blocking = blocking;
	f_non_block = !!(fp->filp->f_flags & O_NONBLOCK);
	mhp->immed = !!(cop->flags & SG_FL_IMMED_NO_WAITQ);
	no_waitq = !!(cop->flags & SGV4_FLAG_NO_WAITQ);
	mhp->stop_if = !!(cop->flags & SGV4_FLAG_STOP_IF);
	mhp->id_of_mrq = (int)cop->request_extra;
	mhp->tot_reqs = tot_reqs;
	if (mhp->id_of_mrq) {
		existing_id = atomic_cmpxchg(&fp->mrq_id_abort, 0,
					     mhp->id_of_mrq);
		if (existing_id && existing_id != mhp->id_of_mrq) {
			SG_LOG(1, fp, "%s: existing id=%d id_of_mrq=%d\n",
			       __func__, existing_id, mhp->id_of_mrq);
			return -EDOM;
		}
		clear_bit(SG_FFD_MRQ_ABORT, fp->ffd_bm);
		mhp->chk_abort = true;
	}
	if (blocking) {		/* came from ioctl(SG_IO) */
		if (unlikely(mhp->immed)) {
			SG_LOG(1, fp, "%s: ioctl(SG_IO) %s contradicts\n",
			       __func__, "with SGV4_FLAG_IMMED");
			return -ERANGE;
		}
		if (unlikely(f_non_block)) {
			SG_LOG(6, fp, "%s: ioctl(SG_IO) %s O_NONBLOCK\n",
			       __func__, "ignoring");
			f_non_block = false;
		}
	}
	if (!mhp->immed && f_non_block)
		mhp->immed = true;
	SG_LOG(3, fp, "%s: %s, tot_reqs=%u, id_of_mrq=%d\n", __func__,
	       (mhp->immed ? (no_waitq ? "NO_WAITQ" : "IMMED")
			   : (blocking ?  "ordered blocking" :
					  "variable blocking")),
	       tot_reqs, mhp->id_of_mrq);
	sg_sgv4_out_zero(cop);

	if (unlikely(tot_reqs > U16_MAX)) {
		return -ERANGE;
	} else if (unlikely(blen > SG_MAX_MULTI_REQ_SZ ||
			    cdb_alen > SG_MAX_MULTI_REQ_SZ)) {
		return  -E2BIG;
	} else if (unlikely(mhp->immed && mhp->stop_if)) {
		return -ERANGE;
	} else if (unlikely(tot_reqs == 0)) {
		return 0;
	} else if (unlikely(!!cdb_alen != !!cop->request)) {
		return -ERANGE;	/* both must be zero or both non-zero */
	} else if (cdb_alen) {
		if (unlikely(cdb_alen % tot_reqs))
			return -ERANGE;
		cdb_mxlen = cdb_alen / tot_reqs;
		if (unlikely(cdb_mxlen < 6))
			return -ERANGE;	/* too short for SCSI cdbs */
	} else {
		cdb_mxlen = 0;
	}

	if (unlikely(SG_IS_DETACHING(sdp)))
		return -ENODEV;
	else if (unlikely(o_sfp && SG_IS_DETACHING((o_sfp->parentdp))))
		return -ENODEV;

	a_hds = kcalloc(tot_reqs, SZ_SG_IO_V4, GFP_KERNEL | __GFP_NOWARN);
	if (unlikely(!a_hds))
		return -ENOMEM;
	if (unlikely(copy_from_user(a_hds, cuptr64(cop->dout_xferp),
				    tot_reqs * SZ_SG_IO_V4))) {
		res = -EFAULT;
		goto fini;
	}
	if (cdb_alen > 0) {
		cdb_ap = kcalloc(tot_reqs, cdb_alen / tot_reqs,
				 GFP_KERNEL | __GFP_NOWARN);
		if (unlikely(!cdb_ap)) {
			res = -ENOMEM;
			goto fini;
		}
		if (unlikely(copy_from_user(cdb_ap, cuptr64(cop->request),
					    cdb_alen))) {
			res = -EFAULT;
			goto fini;
		}
	}
	/* do sanity checks on all requests before starting */
	res = sg_mrq_sanity(sdp, cop, a_hds, cdb_ap, fp, mhp->immed,
			    tot_reqs, &share_on_oth);
	if (unlikely(res))
		goto fini;

	/* override cmd queuing setting to allow */
	clear_bit(SG_FFD_NO_CMD_Q, fp->ffd_bm);
	if (o_sfp)
		clear_bit(SG_FFD_NO_CMD_Q, o_sfp->ffd_bm);

	mhp->cdb_ap = cdb_ap;
	mhp->a_hds = a_hds;
	mhp->cdb_mxlen = cdb_mxlen;

	if (!mhp->immed && !blocking && share_on_oth) {
		bool ok;
		const char *svb_s = "special variable blocking";

		/* check for 'special' variable blocking (svb) */
		ok = sg_mrq_svb_chk(a_hds, tot_reqs);
		if (!ok) {
			SG_LOG(1, fp, "%s: %s failed on req(s)\n", __func__,
			       svb_s);
			res = -ERANGE;
			goto fini;
		}
		if (test_and_set_bit(SG_FFD_SVB_ACTIVE, fp->ffd_bm)) {
			SG_LOG(1, fp, "%s: %s already active\n", __func__,
			       svb_s);
			res = -EBUSY;
			goto fini;
		}
		res = sg_process_svb_mrq(fp, o_sfp, mhp);
		clear_bit(SG_FFD_SVB_ACTIVE, fp->ffd_bm);
	} else {
		res = sg_process_most_mrq(fp, o_sfp, mhp);
	}

fini:
	if (res == 0 && !mhp->immed)
		res = sg_mrq_arr_flush(cop, a_hds, tot_reqs, mhp->s_res);
	kfree(cdb_ap);
	kfree(a_hds);
	return res;
}

static int
sg_v4_submit(struct sg_fd *sfp, void __user *p, struct sg_io_v4 *h4p,
	     bool sync, struct sg_request **o_srp)
{
	int res = 0;
	unsigned long ul_timeout;
	struct sg_request *srp;
	struct sg_comm_wr_t cwr;

	memset(&cwr, 0, sizeof(cwr));
	cwr.sfp = sfp;
	cwr.h4p = h4p;
	if (h4p->flags & SGV4_FLAG_MULTIPLE_REQS) {
		/* want v4 async or sync with guard, din and dout and flags */
		if (!h4p->dout_xferp || h4p->din_iovec_count ||
		    h4p->dout_iovec_count ||
		    (h4p->dout_xfer_len % SZ_SG_IO_V4))
			return -ERANGE;
		if (o_srp)
			*o_srp = NULL;
		res = sg_do_multi_req(&cwr, sync);
		if (unlikely(res))
			return res;
		if (likely(p)) {
			/* Write back sg_io_v4 object for error/warning info */
			if (unlikely(copy_to_user(p, h4p, SZ_SG_IO_V4)))
				return -EFAULT;
		}
		return 0;
	}
	if (h4p->flags & SG_FLAG_MMAP_IO) {
		int len = 0;

		if (h4p->din_xferp)
			len = h4p->din_xfer_len;
		else if (h4p->dout_xferp)
			len = h4p->dout_xfer_len;
		if (unlikely(!list_empty(&sfp->rq_list)))
			return -EBUSY;  /* already active requests on fd */
		if (unlikely(len > sfp->rsv_arr[0].rsv_srp->sgatp->buflen))
			return -ENOMEM; /* MMAP_IO size must fit in reserve */
		if (unlikely(h4p->flags & SG_FLAG_DIRECT_IO))
			return -EINVAL; /* not both MMAP_IO and DIRECT_IO */
	}
	/* once v4 (or v3) seen, allow cmd_q on this fd (def: no cmd_q) */
	if (test_bit(SG_FFD_NO_CMD_Q, sfp->ffd_bm))
		clear_bit(SG_FFD_NO_CMD_Q, sfp->ffd_bm);
	ul_timeout = msecs_to_jiffies(h4p->timeout);
	__assign_bit(SG_FRQ_SYNC_INVOC, cwr.frq_bm, (int)sync);
	__set_bit(SG_FRQ_IS_V4I, cwr.frq_bm);
	cwr.h4p = h4p;
	cwr.timeout = min_t(unsigned long, ul_timeout, INT_MAX);
	cwr.cmd_len = h4p->request_len;
	cwr.u_cmdp = cuptr64(h4p->request);
	srp = sg_common_write(&cwr);
	if (IS_ERR(srp))
		return PTR_ERR(srp);
	if (o_srp)
		*o_srp = srp;
	if (p && !sync && (srp->rq_flags & SGV4_FLAG_YIELD_TAG)) {
		u64 gen_tag = srp->tag;
		struct sg_io_v4 __user *h4_up = (struct sg_io_v4 __user *)p;

		if (unlikely(copy_to_user(&h4_up->generated_tag, &gen_tag,
					  sizeof(gen_tag))))
			return -EFAULT;
	}
	return res;
}

static int
sg_ctl_iosubmit(struct sg_fd *sfp, void __user *p)
{
	int res;
	struct sg_io_v4 h4;
	struct sg_io_v4 *h4p = &h4;
	struct sg_device *sdp = sfp->parentdp;

	res = sg_allow_if_err_recovery(sdp, SG_IS_O_NONBLOCK(sfp));
	if (unlikely(res))
		return res;
	if (unlikely(copy_from_user(h4p, p, SZ_SG_IO_V4)))
		return -EFAULT;
	if (likely(h4p->guard == 'Q'))
		return sg_v4_submit(sfp, p, h4p, false, NULL);
	return -EPERM;
}

static int
sg_ctl_iosubmit_v3(struct sg_fd *sfp, void __user *p)
{
	int res;
	struct sg_io_hdr h3;
	struct sg_io_hdr *h3p = &h3;
	struct sg_device *sdp = sfp->parentdp;

	res = sg_allow_if_err_recovery(sdp, SG_IS_O_NONBLOCK(sfp));
	if (unlikely(res))
		return res;
	if (unlikely(copy_from_user(h3p, p, SZ_SG_IO_HDR)))
		return -EFAULT;
	if (likely(h3p->interface_id == 'S'))
		return sg_v3_submit(sfp, h3p, false, NULL);
	return -EPERM;
}

/*
 * Assumes sharing has been established at the file descriptor level and now we
 * check the rq_flags of a new request/command. SGV4_FLAG_NO_DXFER may or may
 * not be used on the master side, it must be used on the slave side. Also
 * returns (via *sh_varp) the proposed sg_request::sh_var of the new request
 * yet to be built/re-used.
 */
static int
sg_share_chk_flags(struct sg_fd *sfp, u32 rq_flags, int dxfer_len, int dir,
		   enum sg_shr_var *sh_varp)
{
	bool is_master = (sfp->shr_fd == SG_SHARE_FD_MASTER);
	int result = 0;
	enum sg_shr_var sh_var = SG_SHR_NONE;

	if (rq_flags & SGV4_FLAG_SHARE) {
		if (unlikely(rq_flags & SG_FLAG_DIRECT_IO))
			result = -EINVAL; /* since no control of data buffer */
		else if (unlikely(dxfer_len < 1))
			result = -ENODATA;
		else if (is_master) {		/* fd is reader */
			sh_var = SG_SHR_MA_RQ;
			if (unlikely(dir != SG_DXFER_FROM_DEV))
				result = -ENOMSG;
			if (rq_flags & SGV4_FLAG_NO_DXFER) {
				/* rule out some contradictions */
				if (unlikely(rq_flags & SG_FL_MMAP_DIRECT))
					result = -ENODATA;
			}
		} else {			/* fd is slave, writer */
			sh_var = SG_SHR_SL_RQ;
			if (unlikely(dir != SG_DXFER_TO_DEV))
				result = -ENOMSG;
			if (unlikely(!(rq_flags & SGV4_FLAG_NO_DXFER)))
				result = -ENOMSG;
			if (unlikely(rq_flags & SG_FL_MMAP_DIRECT))
				result = -ENODATA;
		}
	} else if (is_master) {
		sh_var = SG_SHR_MA_NOT_SRQ;
	} else {
		sh_var = SG_SHR_SL_NOT_SRQ;
	}
	*sh_varp = sh_var;
	return result;
}

static void
sg_execute_cmd(struct sg_fd *sfp, struct sg_request *srp)
{
	bool at_head, is_v4h, sync;
	unsigned long iflags;
	struct sg_device *sdp = sfp->parentdp;

	if (unlikely(srp->sh_var == SG_SHR_SL_RQ && sfp->share_sfp)) {
		if (test_and_clear_bit(SG_FRQ_DEFER_XFR, srp->frq_bm)) {
			int res = sg_rq_map_kern(srp,
						 sdp->device->request_queue,
						 srp->rq);

			SG_LOG(3, sfp, "%s: deferred xfer\n", __func__);
			if (unlikely(res))
				pr_warn("%s: sg_rq_map_kern() --> %d\n",
					__func__, res);
		}
	}
	is_v4h = test_bit(SG_FRQ_IS_V4I, srp->frq_bm);
	sync = SG_SYNC_INVOC(srp);
	SG_LOG(3, sfp, "%s: is_v4h=%d\n", __func__, (int)is_v4h);
	if (test_bit(SG_FFD_NO_DURATION, sfp->ffd_bm))
		srp->start_ns = 0;
	else
		srp->start_ns = ktime_get_boottime_ns();/* assume always > 0 */
	srp->duration = 0;

	if (!is_v4h && srp->s_hdr3.interface_id == '\0')
		at_head = true;	/* backward compatibility: v1+v2 interfaces */
	else if (test_bit(SG_FFD_Q_AT_TAIL, sfp->ffd_bm))
		/* cmd flags can override sfd setting */
		at_head = !!(srp->rq_flags & SG_FLAG_Q_AT_HEAD);
	else            /* this sfd is defaulting to head */
		at_head = !(srp->rq_flags & SG_FLAG_Q_AT_TAIL);

	kref_get(&sfp->f_ref); /* sg_rq_end_io() does kref_put(). */

	/* >>>>>>> send cmd/req off to other levels <<<<<<<< */
	if (!sync)
		atomic_inc(&sfp->submitted);
	blk_execute_rq_nowait(sdp->device->request_queue, sdp->disk,
			      srp->rq, (int)at_head, sg_rq_end_io);

	/* Should we try to find the req's tag? Only pay if answer is yes */
	if (is_v4h && (srp->rq_flags & SGV4_FLAG_YIELD_TAG)) {
		spin_lock_irqsave(&srp->req_lck, iflags);
		{		/* we might be too late or too early! */
			struct request *rq = srp->rq;

			srp->tag = rq ? rq->tag : SG_TAG_WILDCARD;
		}
		spin_unlock_irqrestore(&srp->req_lck, iflags);
	}
}

#if IS_ENABLED(SG_LOG_ACTIVE)
static void
sg_rep_rq_state_fail(struct sg_fd *sfp, enum sg_rq_state exp_old_st,
		     enum sg_rq_state want_st, enum sg_rq_state act_old_st)
{
	const char *eors = "expected old rq_st: ";
	const char *aors = "actual old rq_st: ";

	if (IS_ENABLED(CONFIG_SCSI_PROC_FS))
		SG_LOG(1, sfp, "%s: %s%s, %s%s, wanted rq_st: %s\n", __func__,
		       eors, sg_rq_st_str(exp_old_st, false),
		       aors, sg_rq_st_str(act_old_st, false),
		       sg_rq_st_str(want_st, false));
	else
		pr_info("sg: %s: %s%d, %s%d, wanted rq_st: %d\n", __func__,
			eors, (int)exp_old_st, aors, (int)act_old_st,
			(int)want_st);
}
#endif

static inline int
sg_rstate_chg(struct sg_request *srp, enum sg_rq_state old_st,
	      enum sg_rq_state new_st)
{
	enum sg_rq_state act_old_st = (enum sg_rq_state)
				atomic_cmpxchg(&srp->rq_st, old_st, new_st);

	if (act_old_st == old_st)
		return 0;	/* implies new_st --> srp->rq_st */
#if IS_ENABLED(SG_LOG_ACTIVE)
	else
		sg_rep_rq_state_fail(srp->parentfp, old_st, new_st,
				     act_old_st);
#endif
	return -EPROTOTYPE;
}

/*
 * Returns index of an unused element in sfp's rsv_arr, or -1 if it is full.
 * Marks that element's rsv_srp with ERR_PTR(-EBUSY) to reserve that index.
 */
static int
sg_get_idx_new(struct sg_fd *sfp)
{
	int k;
	struct sg_rsv_slave *rsp;

	for (k = 0, rsp = sfp->rsv_arr; k < SG_MAX_RSV_REQS; ++k, ++rsp) {
		if (!rsp->rsv_srp) {
			/* any non-zero pointer, used to hold this position */
			rsp->rsv_srp = ERR_PTR(-EBUSY);
			break;
		}
	}
	return (k < SG_MAX_RSV_REQS) ? k : -1;
}

static int
sg_get_idx_new_lck(struct sg_fd *sfp)
{
	int res;
	unsigned long iflags;

	spin_lock_irqsave(&sfp->rq_list_lock, iflags);
	res = sg_get_idx_new(sfp);
	spin_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	return res;
}

/*
 * Looks for an available element index in sfp's rsv_arr. That element's
 * slave_srp must be NULL and will be set to ERR_PTR(-EBUSY). If no element
 * is available then returns -1.
 */
static int
sg_get_idx_available(struct sg_fd *sfp)
		__must_hold(&sfp->rq_list_lock)
{
	int k;
	struct sg_rsv_slave *rsp;
	struct sg_request *srp;

	for (k = 0, rsp = sfp->rsv_arr; k < SG_MAX_RSV_REQS; ++k, ++rsp) {
		srp = rsp->rsv_srp;
		if (srp && !IS_ERR(srp) && !rsp->slave_srp &&
		    !SG_RS_ACTIVE(srp)) {
			rsp->slave_srp = ERR_PTR(-EBUSY);
			break;
		}
	}
	return (k < SG_MAX_RSV_REQS) ? k : -1;
}

/*
 * If idx >= 0 then writes rsv_srp and slave_srp at that index in sfp's
 * rsv_arr[]. If idx < 0 then searches for match on rsv_srp and if found
 * places slave_srp in the same element.
 */
static void
sg_place_at_idx(struct sg_fd *sfp, int idx, struct sg_request *rsv_srp,
		struct sg_request *slave_srp)
{
	if (idx >= 0) {
		sfp->rsv_arr[idx].rsv_srp = rsv_srp;
		sfp->rsv_arr[idx].slave_srp = rsv_srp ? slave_srp : NULL;
	} else {
		struct sg_rsv_slave *rsp = sfp->rsv_arr;
		const struct sg_rsv_slave *end_rsp = rsp + SG_MAX_RSV_REQS;

		for ( ; rsp < end_rsp; ++rsp) {
			if (rsv_srp == rsp->rsv_srp) {
				rsp->slave_srp = slave_srp;
				return;
			}
		}
	}
}

static inline void
sg_place_at_idx_lck(struct sg_fd *sfp, int idx, struct sg_request *rsv_srp,
		    struct sg_request *slave_srp)
{
	unsigned long iflags;

	spin_lock_irqsave(&sfp->rq_list_lock, iflags);
	sg_place_at_idx(sfp, idx, rsv_srp, slave_srp);
	spin_unlock_irqrestore(&sfp->rq_list_lock, iflags);
}

/* Returns master's reserve request ptr whose slave_srp matches, else NULL. */
static struct sg_request *
sg_get_rsv_by_slave(struct sg_fd *sfp, struct sg_request *slave_srp)
{
	unsigned long iflags;
	struct sg_rsv_slave *rsp;
	const struct sg_rsv_slave *end_rsp;

	spin_lock_irqsave(&sfp->rq_list_lock, iflags);
	rsp = sfp->rsv_arr;
	end_rsp = rsp + SG_MAX_RSV_REQS;
	for ( ; rsp < end_rsp; ++rsp) {
		if (rsp->rsv_srp && !IS_ERR(rsp->rsv_srp) &&
		    rsp->slave_srp == slave_srp)
			break;
	}
	spin_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	return (rsp < end_rsp) ? rsp->rsv_srp : NULL;
}

/*
 * Searches for rsv_srp in sfp's rsv_arr. If found and corresponding
 * slave_srp isn't an error, then it returns slave_srp, otherwise it
 * returns NULL.
 */
static struct sg_request *
sg_get_slave_by_rsv(struct sg_fd *sfp, struct sg_request *rsv_srp)
{
	struct sg_request *srp;
	const struct sg_rsv_slave *rsp = sfp->rsv_arr;
	const struct sg_rsv_slave *end_rsp = rsp + SG_MAX_RSV_REQS;

	for ( ; rsp < end_rsp; ++rsp) {
		if (rsp->rsv_srp == rsv_srp) {
			srp = rsp->slave_srp;
			if (IS_ERR(srp))
				srp = NULL;
			break;
		}
	}
	return (rsp < end_rsp) ? srp : NULL;
}

static struct sg_request *
sg_get_slave_by_rsv_lck(struct sg_fd *sfp, struct sg_request *rsv_srp)
{
	unsigned long iflags;
	struct sg_request *srp;

	spin_lock_irqsave(&sfp->rq_list_lock, iflags);
	srp = sg_get_slave_by_rsv(sfp, rsv_srp);
	spin_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	return srp;
}

static bool
sg_any_rsv_in_use(struct sg_fd *sfp)
{
	unsigned long iflags;
	enum sg_rq_state rq_st;
	struct sg_rsv_slave *rsp;
	const struct sg_rsv_slave *end_rsp;

	spin_lock_irqsave(&sfp->rq_list_lock, iflags);
	rsp = sfp->rsv_arr;
	end_rsp = rsp + SG_MAX_RSV_REQS;
	for ( ; rsp < end_rsp; ++rsp) {
		if (rsp->rsv_srp && rsp->rsv_srp != ERR_PTR(-EBUSY)) {
			rq_st = atomic_read(&rsp->rsv_srp->rq_st);
			if (!(rq_st == SG_RS_INACTIVE ||
			      rq_st == SG_RS_SHR_SWAP))
				break;
			if (rsp->slave_srp)
				break;
		}
	}
	spin_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	return (rsp < end_rsp) ? false : true;
}

/*
 * Returns string of the form: <leadin>rsv<num><leadout> if srp is one of
 * the reserve requests. Otherwise a blank string of length <leadin> plus
 * length of <leadout> is returned.
 */
static char *
sg_get_rsv_str(struct sg_request *srp, const char *leadin,
	       const char *leadout, int b_len, char *b)
{
	int k, i_len, o_len, len;
	struct sg_fd *sfp;
	struct sg_rsv_slave *rsp;

	if (!b || b_len < 1)
		return b;
	if (!leadin)
		leadin = "";
	if (!leadout)
		leadout = "";
	i_len = strlen(leadin);
	o_len = strlen(leadout);
	if (!srp)
		goto blank;
	sfp = srp->parentfp;
	if (!sfp)
		goto blank;
	rsp = sfp->rsv_arr;
	for (k = 0; k < SG_MAX_RSV_REQS; ++k, ++rsp) {
		if (rsp->rsv_srp == srp)
			break;
	}
	if (k >= SG_MAX_RSV_REQS)
		goto blank;
	scnprintf(b, b_len, "%srsv%d%s", leadin, k, leadout);
	return b;
blank:
	len = min_t(int, i_len + o_len, b_len - 1);
	for (k = 0; k < len; ++k)
		b[k] = ' ';
	b[len] = '\0';
	return b;
}

static inline char *
sg_get_rsv_str_lck(struct sg_request *srp, const char *leadin,
		   const char *leadout, int b_len, char *b)
{
	unsigned long iflags;
	char *cp;

	spin_lock_irqsave(&srp->parentfp->rq_list_lock, iflags);
	cp = sg_get_rsv_str(srp, leadin, leadout, b_len, b);
	spin_unlock_irqrestore(&srp->parentfp->rq_list_lock, iflags);
	return cp;
}

/*
 * Here when share slave request is being built and SG_FRQ_DEFER_XFR is set.
 * Returns 0 for SWAIT (i.e. deferred execution), SZ_SG_IO_V4 (positive) for
 * execute now, and negated errno value for error.
 */
static int
sg_process_swait(struct sg_fd *sl_sfp, struct sg_request *sl_srp)
{
	int res = 0;
	unsigned long iflags;
	enum sg_rq_state ma_sr_st;
	struct sg_fd *ma_sfp = sl_sfp->share_sfp;
	struct sg_request *ma_rsv_srp;

	if (unlikely(!ma_sfp))
		return -EPROTO;
	ma_rsv_srp = sg_get_rsv_by_slave(ma_sfp, sl_srp);
	if (!ma_rsv_srp)
		return -EPROTO;

	spin_lock_irqsave(&ma_rsv_srp->req_lck, iflags);
	ma_sr_st = atomic_read(&ma_rsv_srp->rq_st);
	SG_LOG(3, sl_sfp, "%s: ma_rsv_srp=0x%pK, ma_sr_st: %s\n", __func__,
	       ma_rsv_srp, sg_rq_st_str(ma_sr_st, false));

	switch (ma_sr_st) {	/* master's state held constant by spinlock */
	case SG_RS_INFLIGHT:
		res = sg_rstate_chg(sl_srp, SG_RS_INFLIGHT, SG_RS_SHR_SWAIT);
		break;		/* if that worked res=0 and slave now SWAIT */
	case SG_RS_AWAIT_RCV:
	case SG_RS_RCV_DONE:
	case SG_RS_SHR_SWAP:
		res = (ma_rsv_srp->rq_result & SG_ML_RESULT_MSK) ?
				-ENOSTR : SZ_SG_IO_V4;
		break;
	case SG_RS_BUSY:
		res = -EBUSY;
		break;
	case SG_RS_INACTIVE:
		res = -EAGAIN;
		break;
	case SG_RS_SHR_SLAVE:
		res = -EDOM;
		break;
	case SG_RS_SHR_SWAIT:
		res = -ENOSTR;
		break;
	default:
		res = -EPROTO;
		break;
	}
	if (unlikely(res == -EPROTOTYPE))
		res = -ENOSTR;
	spin_unlock_irqrestore(&ma_rsv_srp->req_lck, iflags);
	return res;
}

/*
 * All writes and submits converge on this function to launch the SCSI
 * command/request (via blk_execute_rq_nowait). Returns a pointer to a
 * sg_request object holding the request just issued or a negated errno
 * value twisted by ERR_PTR.
 * N.B. pack_id placed in sg_io_v4::request_extra field.
 */
static struct sg_request *
sg_common_write(struct sg_comm_wr_t *cwrp)
{
	int res = 0;
	int dxfr_len, dir;
	int pack_id = SG_PACK_ID_WILDCARD;
	u32 rq_flags;
	enum sg_shr_var sh_var;
	struct sg_fd *fp = cwrp->sfp;
	struct sg_device *sdp = fp->parentdp;
	struct sg_request *srp;
	struct sg_io_hdr *hi_p;
	struct sg_io_v4 *h4p;

	if (test_bit(SG_FRQ_IS_V4I, cwrp->frq_bm)) {
		h4p = cwrp->h4p;
		hi_p = NULL;
		dxfr_len = 0;
		dir = SG_DXFER_NONE;
		rq_flags = h4p->flags;
		pack_id = h4p->request_extra;
		if (unlikely(h4p->din_xfer_len && h4p->dout_xfer_len)) {
			return ERR_PTR(-EOPNOTSUPP);
		} else if (h4p->din_xfer_len) {
			dxfr_len = h4p->din_xfer_len;
			dir = SG_DXFER_FROM_DEV;
		} else if (h4p->dout_xfer_len) {
			dxfr_len = h4p->dout_xfer_len;
			dir = SG_DXFER_TO_DEV;
		}
		/* Only allow SGV4_FLAG_DO_ON_OTHER on mrq */
		if (!test_bit(SG_FRQ_MULTI_REQ, cwrp->frq_bm) &&
		    (unlikely(rq_flags & SGV4_FLAG_DO_ON_OTHER)))
			return ERR_PTR(-ERANGE);
	} else {			/* sg v3 interface so hi_p valid */
		h4p = NULL;
		hi_p = cwrp->h3p;
		dir = hi_p->dxfer_direction;
		dxfr_len = hi_p->dxfer_len;
		rq_flags = hi_p->flags;
		pack_id = hi_p->pack_id;
	}
	if (unlikely(rq_flags & SGV4_FLAG_MULTIPLE_REQS))
		return ERR_PTR(-ERANGE);  /* only control object sets this */
	if (fp->shr_fd == SG_SHARE_FD_UNUSED) {
		/* no sharing established on this fd */
		sh_var = SG_SHR_NONE;
		if (unlikely(rq_flags & SGV4_FLAG_SHARE))
			return ERR_PTR(-ENOMSG);    /* no file share found */
	} else {
		res = sg_share_chk_flags(fp, rq_flags, dxfr_len, dir, &sh_var);
		if (unlikely(res < 0))
			return ERR_PTR(res);
	}
	if (unlikely(dxfr_len >= SZ_256M))
		return ERR_PTR(-EINVAL);

	srp = sg_add_request(cwrp, sh_var, dxfr_len);
	if (IS_ERR(srp))
		return srp;
	srp->rq_flags = rq_flags;
	srp->pack_id = pack_id;

	if (h4p) {
		memset(&srp->s_hdr4, 0, sizeof(srp->s_hdr4));
		srp->s_hdr4.usr_ptr = h4p->usr_ptr;
		srp->s_hdr4.sbp = uptr64(h4p->response);
		srp->s_hdr4.max_sb_len = h4p->max_response_len;
		srp->s_hdr4.cmd_len = h4p->request_len;
		srp->s_hdr4.dir = dir;
	} else {	/* v3 interface active */
		memcpy(&srp->s_hdr3, hi_p, sizeof(srp->s_hdr3));
	}
	res = sg_start_req(srp, cwrp, dir);
	if (unlikely(res < 0))	/* probably out of space --> -ENOMEM */
		goto err_out;
	SG_LOG(4, fp, "%s: opcode=0x%02x, cdb_sz=%d, pack_id=%d\n", __func__,
	       srp->cmd_opcode, cwrp->cmd_len, pack_id);
	if (unlikely(SG_IS_DETACHING(sdp))) {
		res = -ENODEV;
		goto err_out;
	}
	if (unlikely(test_bit(SG_FRQ_BLK_PUT_REQ, srp->frq_bm) || !srp->rq)) {
		res = -EIDRM;	/* this failure unexpected but observed */
		goto err_out;
	}
	srp->rq->timeout = cwrp->timeout;

	if (sh_var == SG_SHR_SL_RQ &&
	    test_bit(SG_FRQ_DEFER_XFR, srp->frq_bm)) {
		res = sg_process_swait(fp, srp);
		if (res == 0)		/* in SWAIT state, deferring */
			return srp;
		else if (unlikely(res < 0))
			goto err_out;
		/* fallthrough when res is SZ_SG_IO_V4 to execute now */
	}
	sg_execute_cmd(fp, srp);
	return srp;
err_out:
	sg_finish_scsi_blk_rq(srp);
	sg_deact_request(fp, srp);
	return ERR_PTR(res);
}

/*
 * ***********************************************************************
 * read(2) related functions follow. They are shown after write(2) related
 * functions. Apart from read(2) itself, ioctl(SG_IORECEIVE) and the second
 * half of the ioctl(SG_IO) share code with read(2).
 * ***********************************************************************
 */

/*
 * This function is called by wait_event_interruptible in sg_read() and
 * sg_ctl_ioreceive(). wait_event_interruptible will return if this one
 * returns true (or an event like a signal (e.g. control-C) occurs).
 */
static inline bool
sg_get_ready_srp(struct sg_fd *sfp, struct sg_request **srpp, int id,
		 bool is_tag)
{
	struct sg_request *srp;

	if (unlikely(SG_IS_DETACHING(sfp->parentdp))) {
		*srpp = NULL;
		return true;
	}
	srp = sg_find_srp_by_id(sfp, id, is_tag);
	*srpp = srp;
	return !!srp;
}

/*
 * Returns number of bytes copied to user space provided sense buffer or
 * negated errno value.
 */
static int
sg_copy_sense(struct sg_request *srp, bool v4_active)
{
	int sb_len_wr = 0;
	int scsi_stat;

	/* If need be, copy the sense buffer to the user space */
	scsi_stat = srp->rq_result & 0xff;
	if ((scsi_stat & SAM_STAT_CHECK_CONDITION) ||
	    (driver_byte(srp->rq_result) & DRIVER_SENSE)) {
		int sb_len = min_t(int, SCSI_SENSE_BUFFERSIZE, srp->sense_len);
		int mx_sb_len;
		u8 *sbp = srp->sense_bp;
		void __user *up;

		srp->sense_bp = NULL;
		if (v4_active) {
			up = uptr64(srp->s_hdr4.sbp);
			mx_sb_len = srp->s_hdr4.max_sb_len;
		} else {
			up = (void __user *)srp->s_hdr3.sbp;
			mx_sb_len = srp->s_hdr3.mx_sb_len;
		}
		if (up && mx_sb_len > 0 && sbp) {
			sb_len = min_t(int, sb_len, mx_sb_len);
			/* Additional sense length field */
			sb_len_wr = 8 + (int)sbp[7];
			sb_len_wr = min_t(int, sb_len, sb_len_wr);
			if (copy_to_user(up, sbp, sb_len_wr))
				sb_len_wr = -EFAULT;
		}
		mempool_free(sbp, sg_sense_pool);
	}
	return sb_len_wr;
}

static int
sg_rec_v3v4_state(struct sg_fd *sfp, struct sg_request *srp, bool v4_active)
{
	int err = 0;
	u32 rq_res = srp->rq_result;
	enum sg_shr_var sh_var = srp->sh_var;

	if (unlikely(srp->rq_result & 0xff)) {
		int sb_len_wr = sg_copy_sense(srp, v4_active);

		if (sb_len_wr < 0)
			return sb_len_wr;
	}
	if (rq_res & SG_ML_RESULT_MSK)
		srp->rq_info |= SG_INFO_CHECK;
	if (unlikely(test_bit(SG_FRQ_ABORTING, srp->frq_bm)))
		srp->rq_info |= SG_INFO_ABORTED;

	if (sh_var == SG_SHR_SL_RQ && sfp->share_sfp) {
		enum sg_rq_state mar_st;
		struct sg_fd *ma_sfp = sfp->share_sfp;
		struct sg_request *ma_srp = sg_get_rsv_by_slave(ma_sfp, srp);

		if (!ma_srp)
			return -EPROTO;
		mar_st = atomic_read(&ma_srp->rq_st);

		switch (mar_st) {
		case SG_RS_SHR_SWAP:
		case SG_RS_SHR_SLAVE:
			/* already on master's free list, make re-usable */
			ma_srp->tag = SG_TAG_WILDCARD;
			ma_srp->sh_var = SG_SHR_NONE;
			err = sg_rstate_chg(ma_srp, mar_st, SG_RS_INACTIVE);
			break;
		case SG_RS_AWAIT_RCV:
			/* slave completing while master awaiting: leave */
			break;
		case SG_RS_INACTIVE:
			/* remove master-slave mapping */
			sg_place_at_idx_lck(ma_sfp, -1, ma_srp, NULL);
			break;
		default:
			err = -EPROTO;	/* Logic error */
			SG_LOG(1, sfp,
			       "%s: SHR_SL_ACTIVE, bad master state: %s\n",
			       __func__, sg_rq_st_str(mar_st, true));
			break;	/* nothing to do */
		}
	}
	if (unlikely(SG_IS_DETACHING(sfp->parentdp)))
		srp->rq_info |= SG_INFO_DEVICE_DETACHING;
	return err;
}

static int
sg_v3v4_complete(struct sg_fd *sfp, struct sg_request *srp, bool other_err)
{
	int err = 0;
	enum sg_rq_state sr_st = atomic_read(&srp->rq_st);

	/* advance state machine, send signal to slave if appropriate */
	SG_LOG(4, sfp, "%s: %pK: sh_var=%s, sr_st=%s\n", __func__, srp,
	       sg_shr_str(srp->sh_var, true), sg_rq_st_str(sr_st, true));
	switch (srp->sh_var) {
	case SG_SHR_MA_RQ:
		{
			int poll_type = POLL_OUT;

			if ((srp->rq_result & SG_ML_RESULT_MSK) || other_err) {
				set_bit(SG_FFD_MASTER_ERR, sfp->ffd_bm);
				err = sg_rstate_chg(srp, sr_st,
						    SG_RS_RCV_DONE);
				poll_type = POLL_HUP;	/* "Hang-UP flag */
			} else if (sr_st != SG_RS_SHR_SWAP) {
				err = sg_rstate_chg(srp, sr_st,
						    SG_RS_SHR_SWAP);
			}
			if (sfp->share_sfp)
				kill_fasync(&sfp->share_sfp->async_qp,
					    SIGPOLL, poll_type);
		}
		break;
	case SG_SHR_SL_RQ:	/* cleanup both on slave completion */
		{
			struct sg_fd *ma_sfp = sfp->share_sfp;

			if (ma_sfp) {
				struct sg_request *ma_srp =
					sg_get_rsv_by_slave(ma_sfp, srp);

				if (ma_srp) {
					sg_place_at_idx_lck(ma_sfp, -1,
							    ma_srp, NULL);
					ma_srp->sh_var = SG_SHR_MA_NOT_SRQ;
				} else {
					SG_LOG(2, ma_sfp,
					       "%s: slave's master missing\n",
					       __func__);
				}
			}
		}
		srp->sh_var = SG_SHR_SL_NOT_SRQ;
		srp->sgatp = &srp->sgat_h;
		err = sg_rstate_chg(srp, sr_st, SG_RS_RCV_DONE);
		break;
	case SG_SHR_SL_NOT_SRQ:
	default:
		err = sg_rstate_chg(srp, sr_st, SG_RS_RCV_DONE);
		break;
	}
	return err;
}

static int
sg_v4_receive(struct sg_fd *sfp, struct sg_request *srp, void __user *p,
	      struct sg_io_v4 *h4p)
{
	int err, err2;
	u32 rq_result = srp->rq_result;

	SG_LOG(3, sfp, "%s:%s%s\n", __func__,
	       (p ? " user ptr given," : ""), (h4p ? " h4p given" : ""));
	err = sg_rec_v3v4_state(sfp, srp, true);
	h4p->guard = 'Q';
	h4p->protocol = 0;
	h4p->subprotocol = 0;
	h4p->device_status = rq_result & 0xff;
	h4p->driver_status = driver_byte(rq_result);
	h4p->transport_status = host_byte(rq_result);
	h4p->response_len = srp->sense_len;
	h4p->info = srp->rq_info;
	h4p->flags = srp->rq_flags;
	h4p->duration = srp->duration;
	switch (srp->s_hdr4.dir) {
	case SG_DXFER_FROM_DEV:
		h4p->din_xfer_len = srp->sgatp->dlen;
		break;
	case SG_DXFER_TO_DEV:
		h4p->dout_xfer_len = srp->sgatp->dlen;
		break;
	default:
		break;
	}
	h4p->din_resid = srp->in_resid;
	h4p->dout_resid = srp->s_hdr4.out_resid;
	h4p->usr_ptr = srp->s_hdr4.usr_ptr;
	h4p->response = (uintptr_t)srp->s_hdr4.sbp;
	if (test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm))
		h4p->generated_tag = srp->tag;
	else
		h4p->request_extra = srp->pack_id;
	if (p) {
		if (unlikely(copy_to_user(p, h4p, SZ_SG_IO_V4)))
			err = err ? err : -EFAULT;
	}
	err2 = sg_v3v4_complete(sfp, srp, err < 0);
	if (unlikely(err2))
		err = err ? err : err2;
	sg_finish_scsi_blk_rq(srp);
	sg_deact_request(sfp, srp);
	return unlikely(err < 0) ? err : 0;
}

/*
 * Invoked when user calls ioctl(SG_IORECEIVE, SGV4_FLAG_MULTIPLE_REQS).
 * Returns negative on error including -ENODATA if there are no mrqs submitted
 * nor waiting. Otherwise it returns the number of elements written to
 * rsp_arr, which may be 0 if mrqs submitted but none waiting
 */
static int
sg_mrq_iorec_complets(struct sg_fd *sfp, bool non_block, int max_mrqs,
		      struct sg_io_v4 *rsp_arr)
{
	int k;
	int res = 0;
	struct sg_request *srp;

	SG_LOG(3, sfp, "%s: max_mrqs=%d\n", __func__, max_mrqs);
	for (k = 0; k < max_mrqs; ++k) {
		if (!sg_mrq_get_ready_srp(sfp, &srp))
			break;
		if (unlikely(!srp))
			return -EPROTO;
		if (IS_ERR(srp))
			return k ? k : PTR_ERR(srp);
		res = sg_v4_receive(sfp, srp, NULL, rsp_arr + k);
		if (unlikely(res))
			return res;
		rsp_arr[k].info |= SG_INFO_MRQ_FINI;
	}
	if (non_block)
		return k;

	for ( ; k < max_mrqs; ++k) {
		res = sg_wait_mrq_event(sfp, &srp);
		if (unlikely(res))
			return res;	/* signal --> -ERESTARTSYS */
		if (unlikely(!srp))
			return -EPROTO;
		if (IS_ERR(srp))
			return k ? k : PTR_ERR(srp);
		res = sg_v4_receive(sfp, srp, NULL, rsp_arr + k);
		if (unlikely(res))
			return res;
		rsp_arr[k].info |= SG_INFO_MRQ_FINI;
	}
	return k;
}

/*
 * Expected race as multiple concurrent calls with the same pack_id/tag can
 * occur. Only one should succeed per request (more may succeed but will get
 * different requests).
 */
static int
sg_mrq_ioreceive(struct sg_fd *sfp, struct sg_io_v4 *cop, void __user *p,
		 bool non_block)
{
	int res = 0;
	u32 len, n;
	struct sg_io_v4 *rsp_v4_arr;
	void __user *pp;

	SG_LOG(3, sfp, "%s: non_block=%d\n", __func__, !!non_block);
	n = cop->din_xfer_len;
	if (unlikely(n > SG_MAX_MULTI_REQ_SZ))
		return -E2BIG;
	if (unlikely(!cop->din_xferp || n < SZ_SG_IO_V4 || (n % SZ_SG_IO_V4)))
		return -ERANGE;
	n /= SZ_SG_IO_V4;
	len = n * SZ_SG_IO_V4;
	SG_LOG(3, sfp, "%s: %s, num_reqs=%u\n", __func__,
	       (non_block ? "IMMED" : "blocking"), n);
	rsp_v4_arr = kcalloc(n, SZ_SG_IO_V4, GFP_KERNEL);
	if (unlikely(!rsp_v4_arr))
		return -ENOMEM;

	sg_sgv4_out_zero(cop);
	cop->din_resid = n;
	res = sg_mrq_iorec_complets(sfp, non_block, n, rsp_v4_arr);
	if (unlikely(res < 0))
		goto fini;
	cop->din_resid -= res;
	cop->info = res;	/* number received */
	if (unlikely(copy_to_user(p, cop, sizeof(*cop))))
		return -EFAULT;
	res = 0;
	pp = uptr64(cop->din_xferp);
	if (likely(pp)) {
		if (unlikely(copy_to_user(pp, rsp_v4_arr, len)))
			res = -EFAULT;
	} else {
		SG_LOG(1, sfp, "%s: cop->din_xferp==NULL ?_?\n", __func__);
	}
fini:
	kfree(rsp_v4_arr);
	return res;
}

static int
sg_wait_id_event(struct sg_fd *sfp, struct sg_request **srpp, int id,
		 bool is_tag)
{
	if (test_bit(SG_FFD_EXCL_WAITQ, sfp->ffd_bm))
		return __wait_event_interruptible_exclusive
				(sfp->cmpl_wait,
				 sg_get_ready_srp(sfp, srpp, id, is_tag));
	return __wait_event_interruptible
			(sfp->cmpl_wait,
			 sg_get_ready_srp(sfp, srpp, id, is_tag));
}

/*
 * Called when ioctl(SG_IORECEIVE) received. Expects a v4 interface object.
 * Checks if O_NONBLOCK file flag given, if not checks given 'flags' field
 * to see if SGV4_FLAG_IMMED is set. Either of these implies non blocking.
 * When non-blocking and there is no request waiting, yields EAGAIN;
 * otherwise it waits (i.e. it "blocks").
 */
static int
sg_ctl_ioreceive(struct sg_fd *sfp, void __user *p)
{
	bool non_block = SG_IS_O_NONBLOCK(sfp);
	bool use_tag = false;
	int res, id;
	int pack_id = SG_PACK_ID_WILDCARD;
	int tag = SG_TAG_WILDCARD;
	struct sg_io_v4 h4;
	struct sg_io_v4 *h4p = &h4;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_request *srp;

	res = sg_allow_if_err_recovery(sdp, non_block);
	if (unlikely(res))
		return res;
	/* Get first three 32 bit integers: guard, proto+subproto */
	if (unlikely(copy_from_user(h4p, p, SZ_SG_IO_V4)))
		return -EFAULT;
	/* for v4: protocol=0 --> SCSI;  subprotocol=0 --> SPC++ */
	if (unlikely(h4p->guard != 'Q' || h4p->protocol != 0 ||
		     h4p->subprotocol != 0))
		return -EPERM;
	if (h4p->flags & SG_FL_IMMED_NO_WAITQ)
		non_block = true;	/* set by either this or O_NONBLOCK */
	SG_LOG(3, sfp, "%s: non_block(+IMMED)=%d\n", __func__, non_block);
	if (h4p->flags & SGV4_FLAG_MULTIPLE_REQS)
		return sg_mrq_ioreceive(sfp, h4p, p, non_block);
	/* read in part of v3 or v4 header for pack_id or tag based find */
	if (test_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm)) {
		use_tag = test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm);
		if (use_tag)
			tag = h4p->request_tag;	/* top 32 bits ignored */
		else
			pack_id = h4p->request_extra;
	}
	id = use_tag ? tag : pack_id;
	srp = sg_find_srp_by_id(sfp, id, use_tag);
	if (!srp) {     /* nothing available so wait on packet or */
		if (unlikely(SG_IS_DETACHING(sdp)))
			return -ENODEV;
		if (non_block)
			return -EAGAIN;
		res = sg_wait_id_event(sfp, &srp, id, use_tag);
		if (unlikely(SG_IS_DETACHING(sdp)))
			return -ENODEV;
		if (unlikely(res))
			return res;	/* signal --> -ERESTARTSYS */
	}	/* now srp should be valid */
	return sg_v4_receive(sfp, srp, p, h4p);
}

/*
 * Called when ioctl(SG_IORECEIVE_V3) received. Expects a v3 interface.
 * Checks if O_NONBLOCK file flag given, if not checks given flags field
 * to see if SGV4_FLAG_IMMED is set. Either of these implies non blocking.
 * When non-blocking and there is no request waiting, yields EAGAIN;
 * otherwise it waits.
 */
static int
sg_ctl_ioreceive_v3(struct sg_fd *sfp, void __user *p)
{
	bool non_block = SG_IS_O_NONBLOCK(sfp);
	int res;
	int pack_id = SG_PACK_ID_WILDCARD;
	struct sg_io_hdr h3;
	struct sg_io_hdr *h3p = &h3;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_request *srp;

	res = sg_allow_if_err_recovery(sdp, non_block);
	if (unlikely(res))
		return res;
	/* Get first three 32 bit integers: guard, proto+subproto */
	if (unlikely(copy_from_user(h3p, p, SZ_SG_IO_HDR)))
		return -EFAULT;
	/* for v3: interface_id=='S' (in a 32 bit int) */
	if (unlikely(h3p->interface_id != 'S'))
		return -EPERM;
	if (h3p->flags & SG_FL_IMMED_NO_WAITQ)
		non_block = true;	/* set by either this or O_NONBLOCK */
	SG_LOG(3, sfp, "%s: non_block(+IMMED)=%d\n", __func__, non_block);
	if (unlikely(h3p->flags & SGV4_FLAG_MULTIPLE_REQS))
		return -EINVAL;

	if (test_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm))
		pack_id = h3p->pack_id;

	srp = sg_find_srp_by_id(sfp, pack_id, false);
	if (!srp) {     /* nothing available so wait on packet or */
		if (unlikely(SG_IS_DETACHING(sdp)))
			return -ENODEV;
		if (non_block)
			return -EAGAIN;
		res = sg_wait_id_event(sfp, &srp, pack_id, false);
		if (unlikely(SG_IS_DETACHING(sdp)))
			return -ENODEV;
		if (unlikely(res))
			return res;	/* signal --> -ERESTARTSYS */
	}	/* now srp should be valid */
	return sg_v3_receive(sfp, srp, p);
}

static int
sg_rd_v1v2(void __user *buf, int count, struct sg_fd *sfp,
	   struct sg_request *srp)
{
	int res = 0;
	u32 rq_result = srp->rq_result;
	struct sg_header *h2p;
	struct sg_slice_hdr3 *sh3p;
	struct sg_header a_v2hdr;

	h2p = &a_v2hdr;
	memset(h2p, 0, SZ_SG_HEADER);
	sh3p = &srp->s_hdr3;
	h2p->reply_len = (int)sh3p->timeout;
	h2p->pack_len = h2p->reply_len; /* old, strange behaviour */
	h2p->pack_id = sh3p->pack_id;
	h2p->twelve_byte = (srp->cmd_opcode >= 0xc0 && sh3p->cmd_len == 12);
	h2p->target_status = status_byte(rq_result);
	h2p->host_status = host_byte(rq_result);
	h2p->driver_status = driver_byte(rq_result);
	if ((CHECK_CONDITION & status_byte(rq_result)) ||
	    (DRIVER_SENSE & driver_byte(rq_result))) {
		if (srp->sense_bp) {
			u8 *sbp = srp->sense_bp;

			srp->sense_bp = NULL;
			memcpy(h2p->sense_buffer, sbp,
			       sizeof(h2p->sense_buffer));
			mempool_free(sbp, sg_sense_pool);
		}
	}
	switch (unlikely(host_byte(rq_result))) {
	/*
	 * This foolowing setting of 'result' is for backward compatibility
	 * and is best ignored by the user who should use target, host and
	 * driver status.
	 */
	case DID_OK:
	case DID_PASSTHROUGH:
	case DID_SOFT_ERROR:
		h2p->result = 0;
		break;
	case DID_NO_CONNECT:
	case DID_BUS_BUSY:
	case DID_TIME_OUT:
		h2p->result = EBUSY;
		break;
	case DID_BAD_TARGET:
	case DID_ABORT:
	case DID_PARITY:
	case DID_RESET:
	case DID_BAD_INTR:
		h2p->result = EIO;
		break;
	case DID_ERROR:
		h2p->result = (status_byte(rq_result) == GOOD) ? 0 : EIO;
		break;
	default:
		h2p->result = EIO;
		break;
	}

	/* Now copy the result back to the user buffer.  */
	if (count >= SZ_SG_HEADER) {
		if (unlikely(copy_to_user(buf, h2p, SZ_SG_HEADER))) {
			res = -EFAULT;
			goto fini;
		}
		buf += SZ_SG_HEADER;
		if (count > h2p->reply_len)
			count = h2p->reply_len;
		if (count > SZ_SG_HEADER) {
			res = sg_rd_append(srp, buf, count - SZ_SG_HEADER);
			if (unlikely(res))
				goto fini;
		}
	} else
		res = (h2p->result == 0) ? 0 : -EIO;
fini:
	atomic_set(&srp->rq_st, SG_RS_RCV_DONE);
	sg_finish_scsi_blk_rq(srp);
	sg_deact_request(sfp, srp);
	return res;
}

/*
 * This is the read(2) system call entry point (see sg_fops) for this driver.
 * Accepts v1, v2 or v3 type headers (not v4). Returns count or negated
 * errno; if count is 0 then v3: returns -EINVAL; v1+v2: 0 when no other
 * error detected or -EIO.
 */
static ssize_t
sg_read(struct file *filp, char __user *p, size_t count, loff_t *ppos)
{
	bool could_be_v3;
	bool non_block = !!(filp->f_flags & O_NONBLOCK);
	int want_id = SG_PACK_ID_WILDCARD;
	int hlen, ret;
	struct sg_device *sdp = NULL;
	struct sg_fd *sfp;
	struct sg_request *srp = NULL;
	struct sg_header *h2p = NULL;
	struct sg_io_hdr a_sg_io_hdr;

	/*
	 * This could cause a response to be stranded. Close the associated
	 * file descriptor to free up any resources being held.
	 */
	ret = sg_check_file_access(filp, __func__);
	if (unlikely(ret))
		return ret;

	sfp = filp->private_data;
	sdp = sfp->parentdp;
	SG_LOG(3, sfp, "%s: read() count=%d\n", __func__, (int)count);
	ret = sg_allow_if_err_recovery(sdp, non_block);
	if (unlikely(ret))
		return ret;

	could_be_v3 = (count >= SZ_SG_IO_HDR);
	hlen = could_be_v3 ? SZ_SG_IO_HDR : SZ_SG_HEADER;
	h2p = (struct sg_header *)&a_sg_io_hdr;

	if (test_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm) && (int)count >= hlen) {
		/*
		 * Even though this is a user space read() system call, this
		 * code is cheating to fetch the pack_id.
		 * Only need first three 32 bit ints to determine interface.
		 */
		if (unlikely(copy_from_user(h2p, p, 3 * sizeof(int))))
			return -EFAULT;
		if (h2p->reply_len < 0 && could_be_v3) {
			struct sg_io_hdr *v3_hdr = (struct sg_io_hdr *)h2p;

			if (likely(v3_hdr->interface_id == 'S')) {
				struct sg_io_hdr __user *h3_up;

				h3_up = (struct sg_io_hdr __user *)p;
				ret = get_user(want_id, &h3_up->pack_id);
				if (unlikely(ret))
					return ret;
				if (!non_block) {
					int flgs;

					ret = get_user(flgs, &h3_up->flags);
					if (unlikely(ret))
						return ret;
					if (flgs & SG_FL_IMMED_NO_WAITQ)
						non_block = true;
				}
			} else if (v3_hdr->interface_id == 'Q') {
				pr_info_once("sg: %s: v4 interface%s here\n",
					     __func__, " disallowed");
				return -EPERM;
			} else {
				return -EPERM;
			}
		} else { /* for v1+v2 interfaces, this is the 3rd integer */
			want_id = h2p->pack_id;
		}
	}
	srp = sg_find_srp_by_id(sfp, want_id, false);
	if (!srp) {	/* nothing available so wait on packet to arrive or */
		if (unlikely(SG_IS_DETACHING(sdp)))
			return -ENODEV;
		if (non_block) /* O_NONBLOCK or v3::flags & SGV4_FLAG_IMMED */
			return -EAGAIN;
		ret = sg_wait_id_event(sfp, &srp, want_id, false);
		if (unlikely(SG_IS_DETACHING(sdp)))
			return -ENODEV;
		if (unlikely(ret))  /* -ERESTARTSYS as signal hit process */
			return ret;
		/* otherwise srp should be valid */
	}
	if (srp->s_hdr3.interface_id == '\0') {
		ret = sg_rd_v1v2(p, (int)count, sfp, srp);
	} else {
		if (unlikely((int)count < SZ_SG_IO_HDR))
			return -EINVAL;
		ret = sg_v3_receive(sfp, srp, p);
	}
#if IS_ENABLED(SG_LOG_ACTIVE)
	if (unlikely(ret < 0))
		SG_LOG(1, sfp, "%s: negated errno: %d\n", __func__, ret);
#endif
	return unlikely(ret < 0) ? ret : (int)count;
}

/*
 * Completes a v3 request/command. Called from sg_read {v2 or v3},
 * ioctl(SG_IO) {for v3}, or from ioctl(SG_IORECEIVE) when its
 * completing a v3 request/command.
 */
static int
sg_v3_receive(struct sg_fd *sfp, struct sg_request *srp, void __user *p)
{
	int err, err2;
	int rq_result = srp->rq_result;
	struct sg_io_hdr hdr3;
	struct sg_io_hdr *hp = &hdr3;

	SG_LOG(3, sfp, "%s: sh_var: %s srp=0x%pK\n", __func__,
	       sg_shr_str(srp->sh_var, false), srp);
	err = sg_rec_v3v4_state(sfp, srp, false);
	memset(hp, 0, sizeof(*hp));
	memcpy(hp, &srp->s_hdr3, sizeof(srp->s_hdr3));
	hp->sb_len_wr = srp->sense_len;
	hp->info = srp->rq_info;
	hp->resid = srp->in_resid;
	hp->duration = srp->duration;
	hp->status = rq_result & 0xff;
	hp->masked_status = status_byte(rq_result);
	hp->msg_status = msg_byte(rq_result);
	hp->host_status = host_byte(rq_result);
	hp->driver_status = driver_byte(rq_result);
	/* copy_*_user() [without leading underscores] checks access */
	if (unlikely(copy_to_user(p, hp, SZ_SG_IO_HDR)))
		err = err ? err : -EFAULT;
	err2 = sg_v3v4_complete(sfp, srp, err < 0);
	if (unlikely(err2))
		err = err ? err : err2;
	sg_finish_scsi_blk_rq(srp);
	sg_deact_request(sfp, srp);
	return unlikely(err) ? err : 0;
}

static int
max_sectors_bytes(struct request_queue *q)
{
	unsigned int max_sectors = queue_max_sectors(q);

	max_sectors = min_t(unsigned int, max_sectors, INT_MAX >> 9);
	return max_sectors << 9;
}

/*
 * Calculates sg_device::max_sgat_elems and sg_device::max_sgat_sz. It uses
 * the device's request queue. If q not available sets max_sgat_elems to 1
 * and max_sgat_sz to PAGE_SIZE. If potential max_sgat_sz is greater than
 * 2^30 scales down the implied max_segment_size so the product of the
 * max_segment_size and max_sgat_elems is less than or equal to 2^30 .
 */
static void
sg_calc_sgat_param(struct sg_device *sdp)
{
	int sz;
	u64 m;
	struct scsi_device *sdev = sdp->device;
	struct request_queue *q = sdev ? sdev->request_queue : NULL;

	clear_bit(SG_FDEV_LOG_SENSE, sdp->fdev_bm);
	if (!q) {
		sdp->max_sgat_elems = 1;
		sdp->max_sgat_sz = PAGE_SIZE;
		return;
	}
	sdp->max_sgat_elems = queue_max_segments(q);
	m = (u64)queue_max_segment_size(q) * queue_max_segments(q);
	if (m < PAGE_SIZE) {
		sdp->max_sgat_elems = 1;
		sdp->max_sgat_sz = PAGE_SIZE;
		return;
	}
	sz = (int)min_t(u64, m, 1 << 30);
	if (sz == (1 << 30))	/* round down so: sz = elems * elem_sz */
		sz = ((1 << 30) / sdp->max_sgat_elems) * sdp->max_sgat_elems;
	sdp->max_sgat_sz = sz;
}

/*
 * Only valid for shared file descriptors, else -EINVAL. Should only be
 * called after a master side request has successfully completed so that
 * there is valid data in reserve buffer. If fini1_again0 is true then
 * master is taken out of the state waiting for a slave request and the
 * master is put in the inactive state. If fini1_again0 is false (0) then
 * the master (assuming it is inactive) is put in a state waiting for
 * a slave request. This function is called when the write mask is set on
 * ioctl(SG_SET_GET_EXTENDED(SG_CTL_FLAGM_MASTER_FINI)).
 */
static int
sg_change_after_master_rq(struct sg_fd *sfp, bool fini1_again0)
{
	int res = 0;
	enum sg_rq_state sr_st;
	struct sg_request *ma_rsv_srp = NULL;

	rcu_read_lock();
	if (sfp->shr_fd == SG_SHARE_FD_MASTER) {	/* from master */
		ma_rsv_srp = sfp->rsv_arr[0].rsv_srp;
	} else if (unlikely(sfp->shr_fd == SG_SHARE_FD_UNUSED)) {
		res = -EINVAL;
	} else {					/* from slave */
		if (likely(sfp->share_sfp))
			ma_rsv_srp = sfp->share_sfp->rsv_arr[0].rsv_srp;
		else
			res = -EINVAL;
	}
	if (res == 0 && ma_rsv_srp) {
		spin_lock(&ma_rsv_srp->req_lck);
		sr_st = atomic_read(&ma_rsv_srp->rq_st);
		if (fini1_again0) {
			switch (sr_st) {
			case SG_RS_SHR_SWAP:
				ma_rsv_srp->sh_var = SG_SHR_MA_NOT_SRQ;
				ma_rsv_srp = NULL;
				res = sg_rstate_chg(ma_rsv_srp, sr_st,
						    SG_RS_INACTIVE);
				break;
			case SG_RS_SHR_SLAVE:	/* too late, slave rq active */
			case SG_RS_BUSY:
				res = -EAGAIN;
				break;
			default:	/* master in SG_RS_SHR_SWAIT is bad */
				res = -EINVAL;
				break;
			}
		} else if (sr_st == SG_RS_INACTIVE) {
			ma_rsv_srp->sh_var = SG_SHR_MA_RQ;
			res = sg_rstate_chg(ma_rsv_srp, sr_st, SG_RS_SHR_SWAP);
		} else {
			res = -EBUSY;	/* master busy doing something else */
		}
		spin_unlock(&ma_rsv_srp->req_lck);
	}
	rcu_read_unlock();
	if (unlikely(res)) {
		SG_LOG(1, sfp, "%s: err=%d\n", __func__, -res);
	} else {
		SG_LOG(6, sfp, "%s: okay, fini1_again0=%d\n", __func__,
		       fini1_again0);
	}
	return res;
}

static void
sg_unshare_fds(struct sg_fd *ma_sfp, struct sg_fd *sl_sfp)
{
	if (ma_sfp) {
		int k;

		ma_sfp->share_sfp = NULL;
		for (k = 0; k < SG_MAX_RSV_REQS; ++k)
			ma_sfp->rsv_arr[k].slave_srp = NULL;
		ma_sfp->shr_fd = SG_SHARE_FD_UNUSED;
	}
	if (sl_sfp) {
		sl_sfp->share_sfp = NULL;
		sl_sfp->shr_fd = SG_SHARE_FD_UNUSED;
	}
}

/* Placed here with other share/unshare processing functions */
static void
sg_remove_sfp_share(struct sg_fd *sfp)
{
	bool o_sfp_is_master = false;
	__maybe_unused int res = 0;
	int k;
	enum sg_rq_state sr_st;
	struct sg_fd *o_sfp = sfp->share_sfp;
	struct sg_rsv_slave *rsp = NULL;
	struct sg_request *rsv_srp = NULL;

	SG_LOG(3, sfp, "%s: sfp=0x%pK, o_sfp=0x%pK%s\n", __func__, sfp, o_sfp,
	       ((sfp->shr_fd == SG_SHARE_FD_MASTER) ? " master" : ""));
	if (!o_sfp)
		return;
	if (sfp->shr_fd == SG_SHARE_FD_MASTER) {	/* close on master */
		rsp = sfp->rsv_arr;
	} else if (sfp->shr_fd == SG_SHARE_FD_UNUSED) {
		goto fini;
	} else {					/* close on slave */
		o_sfp_is_master = true;
		rsp = o_sfp->rsv_arr;
	}
	spin_lock(&o_sfp->rq_list_lock);
	sg_unshare_fds(o_sfp, NULL);
	spin_unlock(&o_sfp->rq_list_lock);
	for (k = 0; k < SG_MAX_RSV_REQS; ++k, ++rsp) {
		rsv_srp = rsp->rsv_srp;
		if (!rsv_srp)
			continue;
		spin_lock(&rsv_srp->req_lck);
		if (o_sfp_is_master) {
			if (rsv_srp->sh_var == SG_SHR_MA_RQ) {
				sr_st = atomic_read(&rsv_srp->rq_st);
				switch (sr_st) {
				case SG_RS_SHR_SLAVE:
				case SG_RS_SHR_SWAP:
					res = sg_rstate_chg(rsv_srp, sr_st,
							    SG_RS_INACTIVE);
					break;
				case SG_RS_BUSY:
					res = -EBUSY;
				default:
					break;
				}
				rsv_srp->sh_var = SG_SHR_NONE;
			}
		} else {
			rsv_srp->sh_var = SG_SHR_NONE;
		}
		spin_unlock(&rsv_srp->req_lck);
	}
fini:
#if IS_ENABLED(SG_LOG_ACTIVE)
	if (unlikely(res))
		SG_LOG(1, sfp, "%s: internal err=%d\n", __func__, -res);
#else
	return;		/* to please moronic sanity checkers */
#endif
}

/*
 * Active when writing 1 to ioctl(SG_SET_GET_EXTENDED(CTL_FLAGS(UNSHARE))),
 * writing 0 has no effect. Undoes the configuration that has done by
 * ioctl(SG_SET_GET_EXTENDED(SHARE_FD)).
 */
static void
sg_chk_unshare(struct sg_fd *sfp, bool unshare_val)
{
	bool retry;
	int retry_count = 0;
	unsigned long iflags;
	struct sg_request *ma_rsv_srp;
	struct sg_fd *ma_sfp;
	struct sg_fd *sl_sfp;
	struct sg_fd *o_sfp = sfp->share_sfp;/* other side of existing share */

	if (sfp->shr_fd == SG_SHARE_FD_UNUSED || !o_sfp)
		return;	/* no share to undo */
	if (!unshare_val)
		return;
again:
	retry = false;
	spin_lock_irqsave(&sfp->rq_list_lock, iflags);
	if (sfp->shr_fd == SG_SHARE_FD_MASTER) {  /* called on master fd */
		ma_sfp = sfp;
		sl_sfp = o_sfp;
		ma_rsv_srp = ma_sfp->rsv_arr[0].rsv_srp;
		if (ma_rsv_srp && ma_rsv_srp->sh_var != SG_SHR_MA_RQ) {
			if (!spin_trylock(&sl_sfp->rq_list_lock)) {
				if (++retry_count > SG_ADD_RQ_MAX_RETRIES)
					SG_LOG(1, sfp,
					       "%s: cannot get slave lock\n",
					       __func__);
				else
					retry = true;
				goto fini;
			}
			sg_unshare_fds(ma_sfp, sl_sfp);
			spin_unlock(&sl_sfp->rq_list_lock);
		}
	} else {			/* called on slave fd */
		ma_sfp = o_sfp;
		sl_sfp = sfp;
		if (!spin_trylock(&ma_sfp->rq_list_lock)) {
			if (++retry_count > SG_ADD_RQ_MAX_RETRIES)
				SG_LOG(1, sfp, "%s: cannot get master lock\n",
				       __func__);
			else
				retry = true;
			goto fini;
		}
		ma_rsv_srp = ma_sfp->rsv_arr[0].rsv_srp;
		if (ma_rsv_srp->sh_var != SG_SHR_MA_RQ)
			sg_unshare_fds(ma_sfp, sl_sfp);
		spin_unlock(&ma_sfp->rq_list_lock);
	}
fini:
	spin_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	if (unlikely(retry)) {
		cpu_relax();
		goto again;
	}
}

/*
 * Returns duration since srp->start_ns (using boot time as an epoch). Unit
 * is nanoseconds when time_in_ns==true; else it is in milliseconds.
 * For backward compatibility the duration is placed in a 32 bit unsigned
 * integer. This limits the maximum nanosecond duration that can be
 * represented (without wrapping) to about 4.3 seconds. If that is exceeded
 * return equivalent of 3.999.. secs as it is more eye catching than the real
 * number. Negative durations should not be possible but if they occur set
 * duration to an unlikely 2 nanosec. Stalls in a request setup will have
 * ts0==U64_MAX and will return 1 for an unlikely 1 nanosecond duration.
 */
static u32
sg_calc_rq_dur(const struct sg_request *srp, bool time_in_ns)
{
	ktime_t ts0 = srp->start_ns;
	ktime_t now_ts;
	s64 diff;

	if (ts0 == 0)	/* only when SG_FFD_NO_DURATION is set */
		return 0;
	if (unlikely(ts0 == U64_MAX))	/* _prior_ to issuing req */
		return time_in_ns ? 1 : 999999999;
	now_ts = ktime_get_boottime_ns();
	if (unlikely(ts0 > now_ts))
		return time_in_ns ? 2 : 999999998;
	if (time_in_ns)
		diff = ktime_to_ns(ktime_sub(now_ts, ts0));
	else	/* unlikely req duration will exceed 2**32 milliseconds */
		diff = ktime_ms_delta(now_ts, ts0);
	return (diff > (s64)U32_MAX) ? 3999999999U : (u32)diff;
}

/*
 * Return of U32_MAX means srp is inactive or in slave waiting state. *is_durp
 * is unused as an input but if is_dur is non-NULL, it is set on output when
 * duration calculation has completed, clear (or false) if it is on-going.
 */
static u32
sg_get_dur(struct sg_request *srp, const enum sg_rq_state *sr_stp,
	   bool time_in_ns, bool *is_durp)
{
	bool is_dur = false;
	u32 res = U32_MAX;

	switch (sr_stp ? *sr_stp : atomic_read(&srp->rq_st)) {
	case SG_RS_INFLIGHT:
	case SG_RS_BUSY:
		res = sg_calc_rq_dur(srp, time_in_ns);
		break;
	case SG_RS_AWAIT_RCV:
	case SG_RS_RCV_DONE:
	case SG_RS_SHR_SWAP:
	case SG_RS_SHR_SLAVE:
		res = srp->duration;
		is_dur = true;	/* completion has occurred, timing finished */
		break;
	case SG_RS_INACTIVE:
	case SG_RS_SHR_SWAIT:
	default:
		break;
	}
	if (is_durp)
		*is_durp = is_dur;
	return res;
}

static void
sg_fill_request_element(struct sg_fd *sfp, struct sg_request *srp,
			struct sg_req_info *rip)
		__must_hold(&sfp->rq_list_lock)
{
	spin_lock(&srp->req_lck);
	rip->duration = sg_get_dur(srp, NULL, test_bit(SG_FFD_TIME_IN_NS,
						       sfp->ffd_bm), NULL);
	if (rip->duration == U32_MAX)
		rip->duration = 0;
	rip->orphan = test_bit(SG_FRQ_IS_ORPHAN, srp->frq_bm);
	rip->sg_io_owned = SG_SYNC_INVOC(srp);
	rip->problem = !!(srp->rq_result & SG_ML_RESULT_MSK);
	rip->pack_id = test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm) ?
				srp->tag : srp->pack_id;
	rip->usr_ptr = test_bit(SG_FRQ_IS_V4I, srp->frq_bm) ?
			uptr64(srp->s_hdr4.usr_ptr) : srp->s_hdr3.usr_ptr;
	spin_unlock(&srp->req_lck);
}

static inline bool
sg_rq_landed(struct sg_device *sdp, struct sg_request *srp)
{
	return atomic_read(&srp->rq_st) != SG_RS_INFLIGHT ||
	       unlikely(SG_IS_DETACHING(sdp));
}

/* This is a blocking wait then complete for a specific srp. */
static int
sg_wait_event_srp(struct sg_fd *sfp, void __user *p, struct sg_io_v4 *h4p,
		  struct sg_request *srp)
{
	int res;
	struct sg_device *sdp = sfp->parentdp;

	SG_LOG(3, sfp, "%s: about to wait_event...()\n", __func__);
	/* N.B. The SG_FFD_EXCL_WAITQ flag is ignored here. */
	res = __wait_event_interruptible(sfp->cmpl_wait,
					 sg_rq_landed(sdp, srp));
	if (unlikely(res)) { /* -ERESTARTSYS because signal hit thread */
		set_bit(SG_FRQ_IS_ORPHAN, srp->frq_bm);
		/* orphans harvested when sfp->keep_orphan is false */
		atomic_set(&srp->rq_st, SG_RS_INFLIGHT);
		SG_LOG(1, sfp, "%s:  wait_event_interruptible gave %d\n",
		       __func__, res);
		return res;
	}
	if (unlikely(SG_IS_DETACHING(sdp))) {
		atomic_set(&srp->rq_st, SG_RS_INACTIVE);
		return -ENODEV;
	}
	res = sg_rstate_chg(srp, SG_RS_AWAIT_RCV, SG_RS_BUSY);
	if (unlikely(res))
		return res;
	if (test_bit(SG_FRQ_IS_V4I, srp->frq_bm))
		res = sg_v4_receive(sfp, srp, p, h4p);
	else
		res = sg_v3_receive(sfp, srp, p);
	return (res < 0) ? res : 0;
}

/*
 * Handles ioctl(SG_IO) for blocking (sync) usage of v3 or v4 interface.
 * Returns 0 on success else a negated errno.
 */
static int
sg_ctl_sg_io(struct sg_device *sdp, struct sg_fd *sfp, void __user *p)
{
	int res;
	struct sg_request *srp = NULL;
	u8 hu8arr[SZ_SG_IO_V4];
	struct sg_io_hdr *h3p = (struct sg_io_hdr *)hu8arr;
	struct sg_io_v4 *h4p = (struct sg_io_v4 *)hu8arr;

	SG_LOG(3, sfp, "%s:  SG_IO%s\n", __func__,
	       (SG_IS_O_NONBLOCK(sfp) ? " O_NONBLOCK ignored" : ""));
	res = sg_allow_if_err_recovery(sdp, false);
	if (unlikely(res))
		return res;
	if (unlikely(copy_from_user(h3p, p, SZ_SG_IO_HDR)))
		return -EFAULT;
	if (h3p->interface_id == 'Q') {
		/* copy in rest of sg_io_v4 object */
		if (unlikely(copy_from_user(hu8arr + SZ_SG_IO_HDR,
					    ((u8 __user *)p) + SZ_SG_IO_HDR,
					    SZ_SG_IO_V4 - SZ_SG_IO_HDR)))
			return -EFAULT;
		res = sg_v4_submit(sfp, p, h4p, true, &srp);
	} else if (likely(h3p->interface_id == 'S')) {
		res = sg_v3_submit(sfp, h3p, true, &srp);
	} else {
		pr_info_once("sg: %s: v3 or v4 interface only here\n",
			     __func__);
		return -EPERM;
	}
	if (unlikely(res < 0))
		return res;
	if (!srp)	/* mrq case: already processed all responses */
		return res;
	res = sg_wait_event_srp(sfp, p, h4p, srp);
#if IS_ENABLED(SG_LOG_ACTIVE)
	if (unlikely(res))
		SG_LOG(1, sfp, "%s: %s=0x%pK  state: %s, share: %s\n",
		       __func__, "unexpected srp", srp,
		       sg_rq_st_str(atomic_read(&srp->rq_st), false),
		       sg_shr_str(srp->sh_var, false));
#endif
	return res;
}

/*
 * When use_tag is true then id is a tag, else it is a pack_id. Returns
 * valid srp if match, else returns NULL.
 */
static struct sg_request *
sg_match_request(struct sg_fd *sfp, bool use_tag, int id)
		__must_hold(&sfp->rq_list_lock)
{
	bool found = false;
	struct sg_request *srp;

	if (list_empty(&sfp->rq_list))
		return NULL;
	if (id == SG_PACK_ID_WILDCARD)	/* both wildcards same value: -1 */
		return list_first_entry_or_null(&sfp->rq_list,
						struct sg_request, rq_entry);
	if (use_tag) {
		list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
			if (id == srp->tag) {
				found = true;
				break;
			}
		}
	} else {
		list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
			if (id == srp->pack_id) {
				found = true;
				break;
			}
		}
	}
	return found ? srp : NULL;
}

/*
 * Looks for first request following 'after_rp' (or the start if after_rp is
 * NULL) whose pack_id_of_mrq matches the given pack_id. The chosen srp must
 * have the SG_FRQ_MULTI_REQ flag set. If after_rp is non-NULL and it is not
 * found, then the search restarts from the beginning of the list. If no
 * match is found then NULL is returned.
 */
static struct sg_request *
sg_match_first_mrq_after(struct sg_fd *sfp, int pack_id,
			 struct sg_request *after_rp)
		__must_hold(&sfp->rq_list_lock)
{
	bool found = false;
	bool look_for_after = after_rp ? true : false;
	int id;
	struct sg_request *srp;

	if (list_empty(&sfp->rq_list))
		return NULL;
once_more:
	list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
		if (look_for_after) {
			if (after_rp == srp)
				look_for_after = false;
			continue;
		}
		if (!test_bit(SG_FRQ_MULTI_REQ, srp->frq_bm))
			continue;
		id = atomic_read(&srp->s_hdr4.pack_id_of_mrq);
		if (id == 0)	/* mrq_pack_ids cannot be zero */
			continue;
		if (pack_id == SG_PACK_ID_WILDCARD || pack_id == id) {
			found = true;
			break;
		}
	}
	if (look_for_after) {	/* after_rp may now be on free list */
		look_for_after = false;
		goto once_more;
	}
	return found ? srp : NULL;
}

static int
sg_abort_req(struct sg_fd *sfp, struct sg_request *srp)
		__must_hold(&srp->req_lck)
{
	int res = 0;
	enum sg_rq_state rq_st;

	if (test_and_set_bit(SG_FRQ_ABORTING, srp->frq_bm)) {
		SG_LOG(1, sfp, "%s: already aborting req pack_id/tag=%d/%d\n",
		       __func__, srp->pack_id, srp->tag);
		goto fini;	/* skip quietly if already aborted */
	}
	rq_st = atomic_read(&srp->rq_st);
	SG_LOG(3, sfp, "%s: req pack_id/tag=%d/%d, status=%s\n", __func__,
	       srp->pack_id, srp->tag, sg_rq_st_str(rq_st, false));
	switch (rq_st) {
	case SG_RS_BUSY:
		clear_bit(SG_FRQ_ABORTING, srp->frq_bm);
		res = -EBUSY;	/* should not occur often */
		break;
	case SG_RS_INACTIVE:	/* perhaps done already */
		clear_bit(SG_FRQ_ABORTING, srp->frq_bm);
		break;
	case SG_RS_AWAIT_RCV:	/* user should still do completion */
	case SG_RS_RCV_DONE:
	case SG_RS_SHR_SWAP:
	case SG_RS_SHR_SLAVE:
		clear_bit(SG_FRQ_ABORTING, srp->frq_bm);
		break;		/* nothing to do here, return 0 */
	case SG_RS_INFLIGHT:	/* only attempt abort if inflight */
		srp->rq_result |= (DRIVER_SOFT << 24);
		if (likely(srp->rq)) {	/* assume callback will occur */
			SG_LOG(5, sfp, "%s: -->blk_abort_request srp=0x%pK\n",
			       __func__, srp);
			blk_abort_request(srp->rq);
		}
		break;
	case SG_RS_SHR_SWAIT:
		srp->rq_result |= (DRIVER_SOFT << 24);
		if (sg_rstate_chg(srp, SG_RS_SHR_SWAIT, SG_RS_AWAIT_RCV))
			pr_warn("%s: unable to set rq_st?\n", __func__);
		/* Wake any sg_read()/ioctl(SG_IORECEIVE) awaiting this req */
		wake_up_interruptible(&sfp->cmpl_wait);
		kill_fasync(&sfp->async_qp, SIGPOLL, POLL_IN);
		break;
	default:
		clear_bit(SG_FRQ_ABORTING, srp->frq_bm);
		break;
	}
fini:
	return res;
}

static int
sg_mrq_abort_inflight(struct sg_fd *sfp, int pack_id)
		__must_hold(sfp->rq_list_lock)
{
	bool got_ebusy = false;
	int res = 0;
	struct sg_request *srp;
	struct sg_request *prev_srp;

	for (prev_srp = NULL; true; prev_srp = srp) {
		srp = sg_match_first_mrq_after(sfp, pack_id, prev_srp);
		if (!srp)
			break;
		spin_lock(&srp->req_lck);
		res = sg_abort_req(sfp, srp);
		spin_unlock(&srp->req_lck);
		if (res == -EBUSY)	/* check rest of active list */
			got_ebusy = true;
		else if (res)
			break;
	}
	if (res)
		return res;
	return got_ebusy ? -EBUSY : 0;
}

/*
 * Implements ioctl(SG_IOABORT) when SGV4_FLAG_MULTIPLE_REQS set. pack_id is
 * non-zero and is from the request_extra field. dev_scope is set when
 * SGV4_FLAG_DEV_SCOPE is given; in that case there is one level of recursion
 * if there is no match or clash with given sfp. Will abort the first
 * mrq that matches then exit. Can only do mrq abort if the mrq submission
 * used a non-zero ctl_obj.request_extra (pack_id).
 */
static int
sg_mrq_abort(struct sg_fd *sfp, int pack_id, bool dev_scope)
		__must_hold(sfp->f_mutex)
{
	int existing_id;
	int res = 0;
	struct sg_device *sdp;
	struct sg_fd *o_sfp;
	struct sg_fd *s_sfp;

	if (pack_id != SG_PACK_ID_WILDCARD)
		SG_LOG(3, sfp, "%s: pack_id=%d, dev_scope=%s\n", __func__,
		       pack_id, (dev_scope ? "true" : "false"));
	existing_id = atomic_read(&sfp->mrq_id_abort);
	if (existing_id == 0) {
		if (dev_scope)
			goto check_whole_dev;
		SG_LOG(1, sfp, "%s: sfp->mrq_id_abort is 0, nothing to do\n",
		       __func__);
		return -EADDRNOTAVAIL;
	}
	if (pack_id == SG_PACK_ID_WILDCARD) {
		pack_id = existing_id;
		SG_LOG(3, sfp, "%s: wildcard becomes pack_id=%d\n", __func__,
		       pack_id);
	} else if (pack_id != existing_id) {
		if (dev_scope)
			goto check_whole_dev;
		SG_LOG(1, sfp, "%s: want id=%d, got sfp->mrq_id_abort=%d\n",
		       __func__, pack_id, existing_id);
		return -EADDRINUSE;
	}
	if (test_and_set_bit(SG_FFD_MRQ_ABORT, sfp->ffd_bm))
		SG_LOG(2, sfp, "%s: repeated SG_IOABORT on mrq_id=%d\n",
		       __func__, pack_id);

	/* now look for inflight requests matching that mrq pack_id */
	spin_lock(&sfp->rq_list_lock);
	res = sg_mrq_abort_inflight(sfp, pack_id);
	if (res == -EBUSY) {
		res = sg_mrq_abort_inflight(sfp, pack_id);
		if (res)
			goto fini;
	}
	s_sfp = sfp->share_sfp;
	if (s_sfp) {	/* SGV4_FLAG_DO_ON_OTHER may have been used */
		spin_unlock(&sfp->rq_list_lock);
		sfp = s_sfp;	/* if share, check other fd */
		spin_lock(&sfp->rq_list_lock);
		if (!sfp->share_sfp)
			goto fini;
		/* tough luck if other fd used same mrq pack_id */
		res = sg_mrq_abort_inflight(sfp, pack_id);
		if (res == -EBUSY)
			res = sg_mrq_abort_inflight(sfp, pack_id);
	}
fini:
	spin_unlock(&sfp->rq_list_lock);
	return res;

check_whole_dev:
	res = -ENODATA;
	sdp = sfp->parentdp;
	read_lock(&sdp->sfd_lock);
	list_for_each_entry(o_sfp, &sdp->sfds, sfd_entry) {
		if (o_sfp == sfp)
			continue;       /* already checked */
		spin_lock(&o_sfp->rq_list_lock);
		/* recurse, dev_scope==false is stopping condition */
		res = sg_mrq_abort(o_sfp, pack_id, false);
		spin_unlock(&o_sfp->rq_list_lock);
		if (res == 0)
			break;
	}
	read_unlock(&sdp->sfd_lock);
	return res;
}

/*
 * Tries to abort an inflight request/command. First it checks the current fd
 * for a match on pack_id or tag. If there is a match, aborts that match.
 * Otherwise, if SGV4_FLAG_DEV_SCOPE is set, the rest of the file descriptors
 * belonging to the current device are similarly checked. If there is no match
 * then -ENODATA is returned. If the pack_id or tag matches a deferred slave
 * request (in slave wait state) then a DRIVER_SOFT driver_status is set and
 * any user thread waiting for it is woken up.
 */
static int
sg_ctl_abort(struct sg_device *sdp, struct sg_fd *sfp, void __user *p)
		__must_hold(sfp->f_mutex)
{
	bool use_tag, res;
	int pack_id, id, dev_scope;
	unsigned long iflags;
	struct sg_fd *o_sfp;
	struct sg_request *srp;
	struct sg_io_v4 io_v4;
	struct sg_io_v4 *h4p = &io_v4;

	if (unlikely(copy_from_user(h4p, p, SZ_SG_IO_V4)))
		return -EFAULT;
	if (unlikely(h4p->guard != 'Q' || h4p->protocol != 0 ||
		     h4p->subprotocol != 0))
		return -EPERM;
	dev_scope = !!(h4p->flags & SGV4_FLAG_DEV_SCOPE);
	pack_id = h4p->request_extra;
	if (h4p->flags & SGV4_FLAG_MULTIPLE_REQS) {
		if (pack_id == 0)
			return -ENOSTR;
		return sg_mrq_abort(sfp, pack_id, dev_scope);
	}
	use_tag = test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm);
	id = use_tag ? (int)h4p->request_tag : pack_id;
	SG_LOG(3, sfp, "%s: %s_id=%d, dev_scope=%s\n", __func__,
	       (use_tag ? "tag" : "pack"), id, (dev_scope ? "true" : "false"));

	spin_lock(&sfp->rq_list_lock);
	srp = sg_match_request(sfp, use_tag, id);
	if (srp) {
		spin_lock_irqsave(&srp->req_lck, iflags);
		spin_unlock(&sfp->rq_list_lock);
	} else if (dev_scope) {
		spin_unlock(&sfp->rq_list_lock);
		read_lock(&sdp->sfd_lock);
		list_for_each_entry(o_sfp, &sdp->sfds, sfd_entry) {
			if (o_sfp == sfp)
				continue;	/* already checked */
			spin_lock(&o_sfp->rq_list_lock);
			srp = sg_match_request(o_sfp, use_tag, id);
			if (srp) {
				spin_lock_irqsave(&srp->req_lck, iflags);
				spin_unlock(&o_sfp->rq_list_lock);
				sfp = o_sfp;
				break;
			}
			spin_unlock(&o_sfp->rq_list_lock);
		}
		read_unlock(&sdp->sfd_lock);
		if (unlikely(!srp))
			return -ENODATA;
		spin_lock_irqsave(&srp->req_lck, iflags);
	} else {
		spin_unlock(&sfp->rq_list_lock);
		return -ENODATA;
	}

	/* should only be holding srp->req_lck spinlock here */
	res = sg_abort_req(sfp, srp);
	spin_unlock_irqrestore(&srp->req_lck, iflags);
	return res;
}

static int
sg_idr_max_id(int id, void *p, void *data)
		__must_hold(&sg_index_lock)
{
	int *k = data;

	if (*k < id)
		*k = id;
	return 0;
}

/*
 * Scans sg driver object tree looking for search_for. Returns valid pointer
 * if found; returns negated errno twisted by ERR_PTR(); or return NULL if
 * not found (and no error).
 */
static struct sg_fd *
sg_find_sfp_by_fd(const struct file *search_for, int search_fd,
		  struct sg_fd *from_sfp, bool from_is_master)
{
	bool found = false;
	int k, num_d;
	int res = 0;
	unsigned long iflags;
	struct sg_fd *sfp;
	struct sg_device *sdp;

	num_d = -1;
	read_lock_irqsave(&sg_index_lock, iflags);
	idr_for_each(&sg_index_idr, sg_idr_max_id, &num_d);
	++num_d;
	for (k = 0; k < num_d; ++k) {
		sdp = idr_find(&sg_index_idr, k);
		if (unlikely(!sdp || SG_IS_DETACHING(sdp)))
			continue;
		read_lock(&sdp->sfd_lock);
		list_for_each_entry(sfp, &sdp->sfds, sfd_entry) {
			if (test_bit(SG_FFD_RELEASE, sfp->ffd_bm))
				continue;
			spin_lock(&sfp->rq_list_lock);
			if (search_for != sfp->filp) {
				spin_unlock(&sfp->rq_list_lock);
				continue;       /* not this one */
			}
			if (unlikely(sfp->shr_fd != SG_SHARE_FD_UNUSED)) {
				res = -EADDRNOTAVAIL;/* already sharing */
			} else if (unlikely(SG_HAVE_EXCLUDE(sdp))) {
				res = -EPERM;
			} else {
				res = 0;
				found = true;
				if (spin_trylock(&from_sfp->rq_list_lock)) {
					from_sfp->share_sfp = sfp;
					from_sfp->shr_fd = from_is_master ?
						SG_SHARE_FD_MASTER : search_fd;
					spin_unlock(&from_sfp->rq_list_lock);
					sfp->share_sfp = from_sfp;
					sfp->shr_fd = from_is_master ?
						search_fd : SG_SHARE_FD_MASTER;
				} else { /* ?? borrow EPROBE_DEFER + encore */
					res = -EPROBE_DEFER;
				}
			}
			spin_unlock(&sfp->rq_list_lock);
			if (res || found)
				break;
		}       /* end of loop of all fd_s in current device */
		read_unlock(&sdp->sfd_lock);
		if (res || found)
			break;
	}       /* end of loop of all sg devices */
	read_unlock_irqrestore(&sg_index_lock, iflags);
	if (unlikely(res < 0))
		return ERR_PTR(res);
	return found ? sfp : NULL;
}

/*
 * After checking the proposed master-slave relationship is unique and valid,
 * sets up pointers between master and slave sg_fd objects. Returns 0 on
 * success or negated errno value. From ioctl(EXTENDED(SG_SEIM_SHARE_FD)).
 */
static int
sg_fd_share(struct sg_fd *sl_sfp, int m_fd)
{
	bool found = false;
	int res = 0;
	int retry_count = 0;
	struct file *fp;
	struct sg_fd *ma_sfp;

	SG_LOG(3, sl_sfp, "%s:  SHARE: master_fd: %d\n", __func__, m_fd);
	if (unlikely(!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO)))
		return -EACCES;
	if (unlikely(m_fd < 0))
		return -EBADF;
	if (unlikely(sl_sfp->shr_fd != SG_SHARE_FD_UNUSED))
		return -EADDRINUSE;  /* don't allow chain of shares */
	/* Alternate approach: fcheck_files(current->files, m_fd) */
	fp = fget(m_fd);
	if (unlikely(!fp))
		return -ENOENT;
	if (unlikely(sl_sfp->filp == fp)) {/* share with self is confusing */
		res = -ELOOP;
		goto fini;
	}
	SG_LOG(6, sl_sfp, "%s: master fd okay, scan for filp=0x%pK\n",
	       __func__, fp);
again:
	ma_sfp = sg_find_sfp_by_fd(fp, m_fd, sl_sfp, false);
	if (IS_ERR(ma_sfp)) {
		res = PTR_ERR(ma_sfp);
		if (res == -EPROBE_DEFER) {
			if (unlikely(++retry_count > SG_ADD_RQ_MAX_RETRIES)) {
				res = -EBUSY;
			} else {
				res = 0;
				cpu_relax();
				goto again;
			}
		}
	} else {
		found = !!ma_sfp;
	}
fini:
	/* paired with fp=fget(m_fd) above */
	fput(fp);
	if (unlikely(res))
		return res;
	return found ? 0 : -ENOTSOCK; /* ENOTSOCK for fd exists but not sg */
}

/*
 * After checking the proposed master-slave relationship is unique and
 * valid, sets up pointers between master and slave sg_fd objects. Return
 * 0 on success or negated errno value.
 */
static int
sg_fd_reshare(struct sg_fd *ma_sfp, int new_sl_fd)
{
	bool found = false;
	int res = 0;
	int retry_count = 0;
	unsigned long iflags;
	struct file *fp;
	struct sg_fd *sl_sfp = ma_sfp->share_sfp;

	SG_LOG(3, sl_sfp, "%s:  new_slave_fd: %d\n", __func__, new_sl_fd);
	if (unlikely(!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO)))
		return -EACCES;
	if (unlikely(new_sl_fd < 0))
		return -EBADF;
	if (unlikely(ma_sfp->shr_fd != SG_SHARE_FD_MASTER || !sl_sfp))
		return -EINVAL;
	if (sg_any_rsv_in_use(ma_sfp))
		return -EBUSY;

	/* Alternate approach: fcheck_files(current->files, m_fd) */
	fp = fget(new_sl_fd);
	if (unlikely(!fp))
		return -ENOENT;
	if (unlikely(ma_sfp->filp == fp)) {/* share with self is confusing */
		res = -ELOOP;
		goto fini;
	}
	SG_LOG(6, sl_sfp, "%s: slave fd ok, scan for filp=0x%pK\n", __func__,
	       fp);
	/* cleanup up old slave side */
	spin_lock_irqsave(&sl_sfp->rq_list_lock, iflags);
	sl_sfp->share_sfp = NULL;
	sl_sfp->shr_fd = SG_SHARE_FD_UNUSED;
	spin_unlock_irqrestore(&sl_sfp->rq_list_lock, iflags);

again:
	sl_sfp = sg_find_sfp_by_fd(fp, new_sl_fd, ma_sfp, true);
	if (IS_ERR(sl_sfp)) {
		res = PTR_ERR(sl_sfp);
		if (res == -EPROBE_DEFER) {
			if (unlikely(++retry_count > SG_ADD_RQ_MAX_RETRIES)) {
				res = -EBUSY;
			} else {
				res = 0;
				cpu_relax();
				goto again;
			}
		}
	} else {
		found = !!sl_sfp;
	}
fini:
	/* paired with fp=fget(new_sl_fd) above */
	fput(fp);
	if (unlikely(res))
		return res;
	return found ? 0 : -ENOTSOCK; /* ENOTSOCK for fd exists but not sg */
}

/*
 * First normalize want_rsv_sz to be >= sfp->sgat_elem_sz and
 * <= max_segment_size. Exit if that is the same as old size; otherwise
 * create a new candidate request of the new size. Then decide whether to
 * re-use an existing free list request (least buflen >= required size) or
 * use the new candidate. If new one used, leave old one but it is no longer
 * the reserved request. Returns 0 on success, else a negated errno value.
 */
static int
sg_set_reserved_sz(struct sg_fd *sfp, int want_rsv_sz)
		__must_hold(&sfp->f_mutex)
{
	bool use_new_srp = false;
	int res = 0;
	int k, new_sz, blen;
	unsigned long iflags;
	struct sg_request *o_srp;       /* prior reserve sg_request */
	struct sg_request *n_srp;       /* new sg_request, may be used */
	struct sg_request *t_srp;       /* other fl entries */
	struct sg_device *sdp = sfp->parentdp;
	struct sg_rsv_slave *rsp = sfp->rsv_arr;

	if (unlikely(sfp->shr_fd != SG_SHARE_FD_UNUSED))
		return -EBUSY;	/* this fd can't be either side of share */
	new_sz = min_t(int, want_rsv_sz, sdp->max_sgat_sz);
	new_sz = max_t(int, new_sz, sfp->sgat_elem_sz);
	SG_LOG(3, sfp, "%s: was=%d, ask=%d, new=%d (sgat_elem_sz=%d)\n",
	       __func__, rsp->rsv_srp ? rsp->rsv_srp->sgatp->buflen : -1,
	       want_rsv_sz, new_sz, sfp->sgat_elem_sz);

	rsp += SG_MAX_RSV_REQS - 1;
	for (k = SG_MAX_RSV_REQS; k >= 0; --k, --rsp) {
		rcu_read_lock();
		o_srp = rsp->rsv_srp;
		if (unlikely(!o_srp)) {
			rcu_read_unlock();
			continue;
		}
		blen = o_srp->sgatp->buflen;
		rcu_read_unlock();
		if (blen == new_sz)
			continue;
		n_srp = sg_mk_srp_sgat(sfp, true /* can take time */, new_sz);
		if (IS_ERR(n_srp))
			return PTR_ERR(n_srp);
		/* new sg_request object, sized correctly is now available */
		spin_lock_irqsave(&sfp->rq_list_lock, iflags);
		o_srp = rsp->rsv_srp;
		if (unlikely(!o_srp)) {
			res = -EPROTO;
			goto wr_unlock;
		}
		if (unlikely(SG_RS_ACTIVE(o_srp) ||
			     test_bit(SG_FFD_MMAP_CALLED, sfp->ffd_bm))) {
			res = -EBUSY;
			goto wr_unlock;
		}
		use_new_srp = true;
		list_for_each_entry(t_srp, &sfp->rq_fl, fl_entry) {
			if (t_srp != o_srp && new_sz <= t_srp->sgatp->buflen &&
			    !SG_RS_ACTIVE(t_srp)) {
				/* good candidate on free list, use */
				use_new_srp = false;
				rsp->rsv_srp = t_srp;
				break;
			}
		}
		if (use_new_srp) {
			rsp->rsv_srp = n_srp;
			/* add to front of free list */
			list_add_rcu(&n_srp->fl_entry, &sfp->rq_fl);
			SG_LOG(6, sfp, "%s: new rsv srp=0x%pK ++\n", __func__,
			       n_srp);
		}
wr_unlock:
		spin_unlock_irqrestore(&sfp->rq_list_lock, iflags);
		if (!use_new_srp) {
			sg_remove_sgat(n_srp);
			kfree(n_srp);	/* nothing has seen n_srp, so safe */
		}
	}
	return res;
}

static bool
sg_any_persistent_orphans(struct sg_fd *sfp)
{
	bool res = false;
	struct sg_request *srp;

	if (!test_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm))
		return false;
	rcu_read_lock();
	list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
		if (test_bit(SG_FRQ_IS_ORPHAN, srp->frq_bm)) {
			res = true;
			break;
		}
	}
	rcu_read_unlock();
	return res;
}

/*
 * Will clear_first if size already over half of available buffer.
 *
 * N.B. This function is a useful debug aid to be called inline with its
 * output going to /sys/kernel/debug/scsi_generic/snapped for later
 * examination. Best to call it with no locks held and that implies that
 * the driver state may change while it is processing. Interpret the
 * result with this in mind.
 */
static void
sg_take_snap(struct sg_fd *sfp, bool clear_first)
{
	struct sg_device *sdp = sfp->parentdp;
	struct timespec64 ts64;
	char b[64];

	if (!sdp)
		return;
	ktime_get_real_ts64(&ts64);
	/* prefer local time but sys_tz.tz_minuteswest is always 0 */
	snprintf(b, sizeof(b), "UTC time: %.2u:%.2u:%.2u:%.6lu",
		 (unsigned int)(ts64.tv_sec / 3600) % 24,
		 (unsigned int)(ts64.tv_sec / 60) % 60,
		 (unsigned int)ts64.tv_sec % 60,
		 ts64.tv_nsec / 1000);
	mutex_lock(&snapped_mutex);
	if (!snapped_buf) {
		snapped_buf = kzalloc(SG_SNAP_BUFF_SZ,
				      GFP_KERNEL | __GFP_NOWARN);
		if (!snapped_buf)
			goto fini;
	} else if (clear_first) {
		memset(snapped_buf, 0, SG_PROC_DEBUG_SZ);
	}
#if IS_ENABLED(SG_PROC_OR_DEBUG_FS)
	if (true) {	/* for some declarations */
		int n, prevlen, bp_len;
		char *bp;

		prevlen = strlen(snapped_buf);
		if (prevlen > SG_PROC_DEBUG_SZ / 2)
			prevlen = 0;
		bp_len = SG_PROC_DEBUG_SZ - prevlen;
		bp = snapped_buf + prevlen;
		n = scnprintf(bp, bp_len, "%s\n", b);
		bp += n;
		bp_len -= n;
		if (bp_len < 2)
			goto fini;
		read_lock(&sdp->sfd_lock);
		n = sg_proc_debug_sdev(sdp, bp, bp_len, NULL, false);
		if (n >= bp_len - 1) {
			if (bp[bp_len - 2] != '\n')
				bp[bp_len - 2] = '\n';
		}
		read_unlock(&sdp->sfd_lock);
	}
#endif
fini:
	mutex_unlock(&snapped_mutex);
}

/*
 * Processing of ioctl(SG_SET_GET_EXTENDED(SG_SEIM_CTL_FLAGS)) which is a set
 * of boolean flags. Access abbreviations: [rw], read-write; [ro], read-only;
 * [wo], write-only; [raw], read after write; [rbw], read before write.
 */
static void
sg_extended_bool_flags(struct sg_fd *sfp, struct sg_extended_info *seip)
{
	bool flg = false;
	u32 c_flgs_wm = seip->ctl_flags_wr_mask;
	u32 c_flgs_rm = seip->ctl_flags_rd_mask;
	u32 *c_flgsp = &seip->ctl_flags;
	struct sg_fd *ma_sfp = sfp->share_sfp;
	struct sg_device *sdp = sfp->parentdp;

	/* TIME_IN_NS boolean, [raw] time in nanoseconds (def: millisecs) */
	if (c_flgs_wm & SG_CTL_FLAGM_TIME_IN_NS)
		assign_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_TIME_IN_NS));
	if (c_flgs_rm & SG_CTL_FLAGM_TIME_IN_NS) {
		if (test_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm))
			*c_flgsp |= SG_CTL_FLAGM_TIME_IN_NS;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_TIME_IN_NS;
	}
	/* TAG_FOR_PACK_ID boolean, [raw] search by tag or pack_id (def) */
	if (c_flgs_wm & SG_CTL_FLAGM_TAG_FOR_PACK_ID)
		assign_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_TAG_FOR_PACK_ID));
	if (c_flgs_rm & SG_CTL_FLAGM_TAG_FOR_PACK_ID) {
		if (test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm))
			*c_flgsp |= SG_CTL_FLAGM_TAG_FOR_PACK_ID;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_TAG_FOR_PACK_ID;
	}
	/* ORPHANS boolean, [ro] does this fd have any orphan requests? */
	if (c_flgs_rm & SG_CTL_FLAGM_ORPHANS) {
		if (sg_any_persistent_orphans(sfp))
			*c_flgsp |= SG_CTL_FLAGM_ORPHANS;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_ORPHANS;
	}
	/* OTHER_OPENS boolean, [ro] any other sg open fds on this dev? */
	if (c_flgs_rm & SG_CTL_FLAGM_OTHER_OPENS) {
		if (atomic_read(&sdp->open_cnt) > 1)
			*c_flgsp |= SG_CTL_FLAGM_OTHER_OPENS;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_OTHER_OPENS;
	}
	/* Q_TAIL boolean, [raw] 1: queue at tail; 0: head (def: depends) */
	if (c_flgs_wm & SG_CTL_FLAGM_Q_TAIL)
		assign_bit(SG_FFD_Q_AT_TAIL, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_Q_TAIL));
	if (c_flgs_rm & SG_CTL_FLAGM_Q_TAIL) {
		if (test_bit(SG_FFD_Q_AT_TAIL, sfp->ffd_bm))
			*c_flgsp |= SG_CTL_FLAGM_Q_TAIL;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_Q_TAIL;
	}
	/*
	 * UNSHARE boolean: when reading yields zero. When writing true,
	 * unshares this fd from a previously established fd share. If
	 * a shared commands is inflight, waits a little while for it
	 * to finish.
	 */
	if (c_flgs_wm & SG_CTL_FLAGM_UNSHARE)
		sg_chk_unshare(sfp, !!(*c_flgsp & SG_CTL_FLAGM_UNSHARE));
	if (c_flgs_rm & SG_CTL_FLAGM_UNSHARE)
		*c_flgsp &= ~SG_CTL_FLAGM_UNSHARE;	/* clear bit */
	/* IS_SHARE boolean: [ro] true if fd may be master or slave share */
	if (c_flgs_rm & SG_CTL_FLAGM_IS_SHARE) {
		if (sfp->shr_fd == SG_SHARE_FD_UNUSED)
			*c_flgsp &= ~SG_CTL_FLAGM_IS_SHARE;
		else
			*c_flgsp |= SG_CTL_FLAGM_IS_SHARE;
	}
	/* IS_MASTER boolean: [ro] true if this fd may be a master share */
	if (c_flgs_rm & SG_CTL_FLAGM_IS_MASTER) {
		if (sfp->shr_fd == SG_SHARE_FD_MASTER)
			*c_flgsp |= SG_CTL_FLAGM_IS_MASTER;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_IS_MASTER;
	}
	/*
	 * MASTER_FINI boolean, [rbw] should be called by slave; when
	 * reading: master is finished, awaiting action by slave;
	 * when written: 1 --> slave doesn't want to continue
	 */
	if (c_flgs_rm & SG_CTL_FLAGM_MASTER_FINI) {
		struct sg_rsv_slave *rsp = ma_sfp->rsv_arr;

		if (ma_sfp && rsp->rsv_srp) {
			struct sg_request *res_srp = rsp->rsv_srp;

			if (atomic_read(&res_srp->rq_st) == SG_RS_SHR_SWAP)
				*c_flgsp |= SG_CTL_FLAGM_MASTER_FINI;
			else
				*c_flgsp &= ~SG_CTL_FLAGM_MASTER_FINI;
		} else {
			*c_flgsp &= ~SG_CTL_FLAGM_MASTER_FINI;
		}
	}
	if (c_flgs_wm & SG_CTL_FLAGM_MASTER_FINI) {
		bool ma_fini_wm = !!(*c_flgsp & SG_CTL_FLAGM_MASTER_FINI);

		sg_change_after_master_rq(sfp, ma_fini_wm);
	}
	/* MASTER_ERR boolean, [ro] share: master finished with error */
	if (c_flgs_rm & SG_CTL_FLAGM_MASTER_ERR) {
		if (ma_sfp && test_bit(SG_FFD_MASTER_ERR, ma_sfp->ffd_bm))
			*c_flgsp |= SG_CTL_FLAGM_MASTER_ERR;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_MASTER_ERR;
	}
	/* NO_DURATION boolean, [rbw] */
	if (c_flgs_rm & SG_CTL_FLAGM_NO_DURATION)
		flg = test_bit(SG_FFD_NO_DURATION, sfp->ffd_bm);
	if (c_flgs_wm & SG_CTL_FLAGM_NO_DURATION)
		assign_bit(SG_FFD_NO_DURATION, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_NO_DURATION));
	if (c_flgs_rm & SG_CTL_FLAGM_NO_DURATION) {
		if (flg)
			*c_flgsp |= SG_CTL_FLAGM_NO_DURATION;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_NO_DURATION;
	}
	/* MORE_ASYNC boolean, [rbw] */
	if (c_flgs_rm & SG_CTL_FLAGM_MORE_ASYNC)
		flg = test_bit(SG_FFD_MORE_ASYNC, sfp->ffd_bm);
	if (c_flgs_wm & SG_CTL_FLAGM_MORE_ASYNC)
		assign_bit(SG_FFD_MORE_ASYNC, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_MORE_ASYNC));
	if (c_flgs_rm & SG_CTL_FLAGM_MORE_ASYNC) {
		if (flg)
			*c_flgsp |= SG_CTL_FLAGM_MORE_ASYNC;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_MORE_ASYNC;
	}
	/* EXCL_WAITQ boolean, [rbw] */
	if (c_flgs_rm & SG_CTL_FLAGM_EXCL_WAITQ)
		flg = test_bit(SG_FFD_EXCL_WAITQ, sfp->ffd_bm);
	if (c_flgs_wm & SG_CTL_FLAGM_EXCL_WAITQ)
		assign_bit(SG_FFD_EXCL_WAITQ, sfp->ffd_bm,
			   !!(*c_flgsp & SG_CTL_FLAGM_EXCL_WAITQ));
	if (c_flgs_rm & SG_CTL_FLAGM_EXCL_WAITQ) {
		if (flg)
			*c_flgsp |= SG_CTL_FLAGM_EXCL_WAITQ;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_EXCL_WAITQ;
	}
	/* SNAP_DEV boolean, [rbw] */
	if (c_flgs_rm & SG_CTL_FLAGM_SNAP_DEV) {
		mutex_lock(&snapped_mutex);
		flg = (snapped_buf && strlen(snapped_buf) > 0);
		mutex_unlock(&snapped_mutex);
	}
	if (c_flgs_wm & SG_CTL_FLAGM_SNAP_DEV)
		sg_take_snap(sfp, !!(*c_flgsp & SG_CTL_FLAGM_SNAP_DEV));
	if (c_flgs_rm & SG_CTL_FLAGM_SNAP_DEV) {
		if (flg)
			*c_flgsp |= SG_CTL_FLAGM_SNAP_DEV;
		else
			*c_flgsp &= ~SG_CTL_FLAGM_SNAP_DEV;
	}
}

static void
sg_extended_read_value(struct sg_fd *sfp, struct sg_extended_info *seip)
{
	u32 uv;
	unsigned long iflags;
	struct sg_fd *a_sfp;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_request *srp;

	switch (seip->read_value) {
	case SG_SEIRV_INT_MASK:
		seip->read_value = SG_SEIM_ALL_BITS;
		break;
	case SG_SEIRV_BOOL_MASK:
		seip->read_value = SG_CTL_FLAGM_ALL_BITS;
		break;
	case SG_SEIRV_VERS_NUM:
		seip->read_value = sg_version_num;
		break;
	case SG_SEIRV_FL_RQS:
		uv = 0;
		rcu_read_lock();
		list_for_each_entry_rcu(srp, &sfp->rq_fl, fl_entry)
			++uv;
		rcu_read_unlock();
		seip->read_value = uv;
		break;
	case SG_SEIRV_DEV_FL_RQS:
		uv = 0;
		read_lock_irqsave(&sdp->sfd_lock, iflags);
		list_for_each_entry(a_sfp, &sdp->sfds, sfd_entry) {
			rcu_read_lock();
			list_for_each_entry_rcu(srp, &a_sfp->rq_fl, fl_entry)
				++uv;
			rcu_read_unlock();
		}
		read_unlock_irqrestore(&sdp->sfd_lock, iflags);
		seip->read_value = uv;
		break;
	case SG_SEIRV_SUBMITTED:  /* counts all non-blocking on active list */
		seip->read_value = (u32)atomic_read(&sfp->submitted);
		break;
	case SG_SEIRV_DEV_SUBMITTED: /* sum(submitted) on all fd's siblings */
		uv = 0;
		read_lock_irqsave(&sdp->sfd_lock, iflags);
		list_for_each_entry(a_sfp, &sdp->sfds, sfd_entry)
			uv += (u32)atomic_read(&a_sfp->submitted);
		read_unlock_irqrestore(&sdp->sfd_lock, iflags);
		seip->read_value = uv;
		break;
	case SG_SEIRV_MAX_RSV_REQS:
		seip->read_value = SG_MAX_RSV_REQS;
		break;
	default:
		SG_LOG(6, sfp, "%s: can't decode %d --> read_value\n",
		       __func__, seip->read_value);
		seip->read_value = 0;
		break;
	}
}

/* Called when processing ioctl(SG_SET_GET_EXTENDED) */
static int
sg_ctl_extended(struct sg_fd *sfp, void __user *p)
{
	int result = 0;
	int ret = 0;
	int n, j, s_wr_mask, s_rd_mask;
	u32 uv, or_masks;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_extended_info *seip;
	struct sg_extended_info sei;

	seip = &sei;
	if (unlikely(copy_from_user(seip, p, SZ_SG_EXTENDED_INFO)))
		return -EFAULT;
	s_wr_mask = seip->sei_wr_mask;
	s_rd_mask = seip->sei_rd_mask;
	or_masks = s_wr_mask | s_rd_mask;
	if (unlikely(or_masks == 0)) {
		SG_LOG(2, sfp, "%s: both masks 0, do nothing\n", __func__);
		return 0;
	}
	SG_LOG(3, sfp, "%s: wr_mask=0x%x rd_mask=0x%x\n", __func__, s_wr_mask,
	       s_rd_mask);
	/* tot_fd_thresh (u32), [raw] [sum of active cmd dlen_s] */
	if (or_masks & SG_SEIM_TOT_FD_THRESH) {
		if (s_wr_mask & SG_SEIM_TOT_FD_THRESH) {
			uv = seip->tot_fd_thresh;
			if (uv > 0 && uv < PAGE_SIZE)
				uv = PAGE_SIZE;
			sfp->tot_fd_thresh = uv;
		}
		if (s_rd_mask & SG_SEIM_TOT_FD_THRESH)
			seip->tot_fd_thresh = sfp->tot_fd_thresh;
	}
	/* check all boolean flags for either wr or rd mask set in or_mask */
	if (or_masks & SG_SEIM_CTL_FLAGS)
		sg_extended_bool_flags(sfp, seip);
	/* yields minor_index (type: u32) [ro] */
	if (or_masks & SG_SEIM_MINOR_INDEX) {
		if (s_wr_mask & SG_SEIM_MINOR_INDEX) {
			SG_LOG(2, sfp, "%s: writing to minor_index ignored\n",
			       __func__);
		}
		if (s_rd_mask & SG_SEIM_MINOR_INDEX)
			seip->minor_index = sdp->index;
	}
	if ((s_rd_mask & SG_SEIM_READ_VAL) && (s_wr_mask & SG_SEIM_READ_VAL))
		sg_extended_read_value(sfp, seip);
	/* slave side gives fd of master to share with [rbw] */
	if (or_masks & SG_SEIM_SHARE_FD) {
		mutex_lock(&sfp->f_mutex);
		n = sfp->shr_fd;        /* hold prior shr_fd for later read */
		if (s_wr_mask & SG_SEIM_SHARE_FD) {
			result = sg_fd_share(sfp, (int)seip->share_fd);
			if (ret == 0 && unlikely(result))
				ret = result;
		}
		if (s_rd_mask & SG_SEIM_SHARE_FD)
			seip->share_fd = (unsigned int)n;
		mutex_unlock(&sfp->f_mutex);
	}
	/* master side is given shr_fd of new slave [rbw] */
	if (or_masks & SG_SEIM_CHG_SHARE_FD) {
		mutex_lock(&sfp->f_mutex);
		n = sfp->shr_fd;        /* hold prior sl_fd for later read */
		if (s_wr_mask & SG_SEIM_CHG_SHARE_FD) {
			result = sg_fd_reshare(sfp, (int)seip->share_fd);
			if (ret == 0 && unlikely(result))
				ret = result;
		}
		if (s_rd_mask & SG_SEIM_CHG_SHARE_FD)
			seip->share_fd = (unsigned int)n;
		mutex_unlock(&sfp->f_mutex);
	}
	/* override scatter gather element size [rbw] (def: SG_SCATTER_SZ) */
	if (or_masks & SG_SEIM_SGAT_ELEM_SZ) {
		n = sfp->sgat_elem_sz;
		if (s_wr_mask & SG_SEIM_SGAT_ELEM_SZ) {
			j = (int)seip->sgat_elem_sz;
			if (j != (1 << ilog2(j)) || j < (int)PAGE_SIZE) {
				SG_LOG(1, sfp, "%s: %s not power of 2, %s\n",
				       __func__, "sgat element size",
				       "or less than PAGE_SIZE");
				ret = -EINVAL;
			} else {
				sfp->sgat_elem_sz = j;
			}
		}
		if (s_rd_mask & SG_SEIM_SGAT_ELEM_SZ)
			seip->sgat_elem_sz = n; /* prior value if rw */
	}
	/* reserved_sz [raw], since may be reduced by other limits */
	if (s_wr_mask & SG_SEIM_RESERVED_SIZE) {
		mutex_lock(&sfp->f_mutex);
		result = sg_set_reserved_sz(sfp, (int)seip->reserved_sz);
		if (ret == 0 && unlikely(result))
			ret = result;
		mutex_unlock(&sfp->f_mutex);
	}
	if (s_rd_mask & SG_SEIM_RESERVED_SIZE) {
		struct sg_rsv_slave *rsp = sfp->rsv_arr;

		seip->reserved_sz = (u32)min_t(int,
					       rsp->rsv_srp->sgatp->buflen,
					       sdp->max_sgat_sz);
	}
	/* copy to user space if int or boolean read mask non-zero */
	if (s_rd_mask || seip->ctl_flags_rd_mask) {
		if (copy_to_user(p, seip, SZ_SG_EXTENDED_INFO))
			ret = ret ? ret : -EFAULT;
	}
	return ret;
}

/*
 * For backward compatibility, output SG_MAX_QUEUE sg_req_info objects. First
 * fetch from the active list then, if there is still room, from the free
 * list. Some of the trailing elements may be empty which is indicated by all
 * fields being zero. Any requests beyond SG_MAX_QUEUE are ignored.
 */
static int
sg_ctl_req_tbl(struct sg_fd *sfp, void __user *p)
{
	int k, len, val, n;
	int result = 0;
	struct sg_request *srp;
	struct sg_req_info *rinfop;
	struct sg_req_info *rip;

	SG_LOG(3, sfp, "%s:    SG_GET_REQUEST_TABLE\n", __func__);
	k = SG_MAX_QUEUE;
	len = SZ_SG_REQ_INFO * k;
	rinfop = kcalloc(k, SZ_SG_REQ_INFO, GFP_KERNEL);
	if (unlikely(!rinfop))
		return -ENOMEM;
	val = 0;
	rcu_read_lock();
	list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry)
		++val;

	if (val > 0) {
		int max_num = min_t(int, k, val);

		n = 0;
		list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
			if (n >= max_num)
				break;
			rip = &rinfop[n];
			sg_fill_request_element(sfp, srp, rip);
			n++;
		}
		list_for_each_entry_rcu(srp, &sfp->rq_fl, fl_entry) {
			if (n >= max_num)
				break;
			rip = &rinfop[n];
			sg_fill_request_element(sfp, srp, rip);
			n++;
		}
	}
	rcu_read_unlock();
	if (unlikely(copy_to_user(p, rinfop, len)))
		result = -EFAULT;
	kfree(rinfop);
	return result;
}

static int
sg_ctl_scsi_id(struct scsi_device *sdev, struct sg_fd *sfp, void __user *p)
{
	struct sg_scsi_id ss_id;
	struct scsi_lun lun8b;

	SG_LOG(3, sfp, "%s:    SG_GET_SCSI_ID\n", __func__);
	ss_id.host_no = sdev->host->host_no;
	ss_id.channel = sdev->channel;
	ss_id.scsi_id = sdev->id;
	ss_id.lun = sdev->lun;
	ss_id.scsi_type = sdev->type;
	ss_id.h_cmd_per_lun = sdev->host->cmd_per_lun;
	ss_id.d_queue_depth = sdev->queue_depth;
	int_to_scsilun(sdev->lun, &lun8b);
	/* ss_id.scsi_lun is in an anonymous union with 'int unused[2]' */
	memcpy(ss_id.scsi_lun, lun8b.scsi_lun, 8);
	if (copy_to_user(p, &ss_id, sizeof(struct sg_scsi_id)))
		return -EFAULT;
	return 0;
}

static long
sg_ioctl(struct file *filp, unsigned int cmd_in, unsigned long arg)
{
	bool read_only = O_RDWR != (filp->f_flags & O_ACCMODE);
	int val;
	int res = 0;
	void __user *p = uptr64(arg);
	int __user *ip = p;
	struct sg_device *sdp;
	struct sg_fd *sfp;
	struct sg_request *srp;
	struct scsi_device *sdev;
	__maybe_unused const char *pmlp = ", pass to mid-level";

	sfp = filp->private_data;
	sdp = sfp->parentdp;
	SG_LOG(6, sfp, "%s: cmd=0x%x, O_NONBLOCK=%d%s\n", __func__, cmd_in,
	       !!(filp->f_flags & O_NONBLOCK),
	       (cmd_in == SG_GET_NUM_WAITING ? ", SG_GET_NUM_WAITING" : ""));
	if (unlikely(!sdp))
		return -ENXIO;
	if (unlikely(SG_IS_DETACHING(sdp)))
		return -ENODEV;
	sdev = sdp->device;

	switch (cmd_in) {
	case SG_GET_NUM_WAITING:
		/* Want as fast as possible, with a useful result */
		return put_user(atomic_read(&sfp->waiting), ip);
	case SG_IO:
		return sg_ctl_sg_io(sdp, sfp, p);
	case SG_IOSUBMIT:
		SG_LOG(3, sfp, "%s:    SG_IOSUBMIT\n", __func__);
		return sg_ctl_iosubmit(sfp, p);
	case SG_IOSUBMIT_V3:
		SG_LOG(3, sfp, "%s:    SG_IOSUBMIT_V3\n", __func__);
		return sg_ctl_iosubmit_v3(sfp, p);
	case SG_IORECEIVE:
		SG_LOG(3, sfp, "%s:    SG_IORECEIVE\n", __func__);
		return sg_ctl_ioreceive(sfp, p);
	case SG_IORECEIVE_V3:
		SG_LOG(3, sfp, "%s:    SG_IORECEIVE_V3\n", __func__);
		return sg_ctl_ioreceive_v3(sfp, p);
	case SG_IOABORT:
		SG_LOG(3, sfp, "%s:    SG_IOABORT\n", __func__);
		if (unlikely(read_only))
			return -EPERM;
		if (!mutex_trylock(&sfp->f_mutex))
			return -EAGAIN;
		res = sg_ctl_abort(sdp, sfp, p);
		mutex_unlock(&sfp->f_mutex);
		return res;
	case SG_SET_GET_EXTENDED:
		SG_LOG(3, sfp, "%s:    SG_SET_GET_EXTENDED\n", __func__);
		return sg_ctl_extended(sfp, p);
	case SG_GET_SCSI_ID:
		return sg_ctl_scsi_id(sdev, sfp, p);
	case SG_SET_FORCE_PACK_ID:
		SG_LOG(3, sfp, "%s:    SG_SET_FORCE_PACK_ID\n", __func__);
		res = get_user(val, ip);
		if (unlikely(res))
			return res;
		assign_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm, !!val);
		return 0;
	case SG_GET_PACK_ID:    /* or tag of oldest "read"-able, -1 if none */
		rcu_read_lock();
		val = -1;
		if (test_bit(SG_FFD_PREFER_TAG, sfp->ffd_bm)) {
			list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
				if (atomic_read(&(srp)->rq_st) ==
				    SG_RS_AWAIT_RCV && !SG_SYNC_INVOC(srp)) {
					val = srp->tag;
					break;
				}
			}
		} else {
			list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
				if (atomic_read(&(srp)->rq_st) ==
				    SG_RS_AWAIT_RCV && !SG_SYNC_INVOC(srp)) {
					val = srp->pack_id;
					break;
				}
			}
		}
		rcu_read_unlock();
		SG_LOG(3, sfp, "%s:    SG_GET_PACK_ID=%d\n", __func__, val);
		return put_user(val, ip);
	case SG_GET_SG_TABLESIZE:
		SG_LOG(3, sfp, "%s:    SG_GET_SG_TABLESIZE=%d\n", __func__,
		       sdp->max_sgat_elems);
		return put_user(sdp->max_sgat_elems, ip);
	case SG_SET_RESERVED_SIZE:
		mutex_lock(&sfp->f_mutex);
		res = get_user(val, ip);
		if (likely(!res)) {
			if (likely(val >= 0 && val <= (1024 * 1024 * 1024))) {
				res = sg_set_reserved_sz(sfp, val);
			} else {
				SG_LOG(3, sfp, "%s: invalid size\n", __func__);
				res = -EINVAL;
			}
		}
		mutex_unlock(&sfp->f_mutex);
		return res;
	case SG_GET_RESERVED_SIZE:
		mutex_lock(&sfp->f_mutex);
		{
			struct sg_rsv_slave *rsp = sfp->rsv_arr;

			val = min_t(int, rsp->rsv_srp->sgatp->buflen,
				    sdp->max_sgat_sz);
			res = put_user(val, ip);
		}
		mutex_unlock(&sfp->f_mutex);
		SG_LOG(3, sfp, "%s:    SG_GET_RESERVED_SIZE=%d\n", __func__,
		       val);
		return res;
	case SG_SET_COMMAND_Q:	/* set by driver whenever v3 or v4 req seen */
		SG_LOG(3, sfp, "%s:    SG_SET_COMMAND_Q\n", __func__);
		res = get_user(val, ip);
		if (unlikely(res))
			return res;
		assign_bit(SG_FFD_NO_CMD_Q, sfp->ffd_bm, !val);
		return 0;
	case SG_GET_COMMAND_Q:
		SG_LOG(3, sfp, "%s:    SG_GET_COMMAND_Q\n", __func__);
		return put_user(!test_bit(SG_FFD_NO_CMD_Q, sfp->ffd_bm), ip);
	case SG_SET_KEEP_ORPHAN:
		SG_LOG(3, sfp, "%s:    SG_SET_KEEP_ORPHAN\n", __func__);
		res = get_user(val, ip);
		if (unlikely(res))
			return res;
		assign_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm, !!val);
		return 0;
	case SG_GET_KEEP_ORPHAN:
		SG_LOG(3, sfp, "%s:    SG_GET_KEEP_ORPHAN\n", __func__);
		return put_user(test_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm),
				ip);
	case SG_GET_VERSION_NUM:
		SG_LOG(3, sfp, "%s:    SG_GET_VERSION_NUM\n", __func__);
		return put_user(sg_version_num, ip);
	case SG_GET_REQUEST_TABLE:
		return sg_ctl_req_tbl(sfp, p);
	case SG_SCSI_RESET:
		SG_LOG(3, sfp, "%s:    SG_SCSI_RESET\n", __func__);
		break;
	case SG_SET_TIMEOUT:
		SG_LOG(3, sfp, "%s:    SG_SET_TIMEOUT\n", __func__);
		res = get_user(val, ip);
		if (unlikely(res))
			return res;
		if (unlikely(val < 0))
			return -EIO;
		if (val >= mult_frac((s64)INT_MAX, USER_HZ, HZ))
			val = min_t(s64, mult_frac((s64)INT_MAX, USER_HZ, HZ),
				    INT_MAX);
		sfp->timeout_user = val;
		sfp->timeout = mult_frac(val, HZ, USER_HZ);
		return 0;
	case SG_GET_TIMEOUT:    /* N.B. User receives timeout as return value */
				/* strange ..., for backward compatibility */
		SG_LOG(3, sfp, "%s:    SG_GET_TIMEOUT\n", __func__);
		return sfp->timeout_user;
	case SG_SET_FORCE_LOW_DMA:
		/*
		 * N.B. This ioctl never worked properly, but failed to
		 * return an error value. So returning '0' to keep
		 * compatibility with legacy applications.
		 */
		SG_LOG(3, sfp, "%s:    SG_SET_FORCE_LOW_DMA\n", __func__);
		return 0;
	case SG_GET_LOW_DMA:
		SG_LOG(3, sfp, "%s:    SG_GET_LOW_DMA\n", __func__);
		return put_user((int)sdev->host->unchecked_isa_dma, ip);
	case SG_NEXT_CMD_LEN:	/* active only in v2 interface */
		SG_LOG(3, sfp, "%s:    SG_NEXT_CMD_LEN\n", __func__);
		res = get_user(val, ip);
		if (unlikely(res))
			return res;
		if (unlikely(val > SG_MAX_CDB_SIZE))
			return -ENOMEM;
		mutex_lock(&sfp->f_mutex);
		sfp->next_cmd_len = max_t(int, val, 0);
		mutex_unlock(&sfp->f_mutex);
		return 0;
	case SG_GET_ACCESS_COUNT:
		SG_LOG(3, sfp, "%s:    SG_GET_ACCESS_COUNT\n", __func__);
		/* faked - we don't have a real access count anymore */
		val = (sdev ? 1 : 0);
		return put_user(val, ip);
	case SG_EMULATED_HOST:
		SG_LOG(3, sfp, "%s:    SG_EMULATED_HOST\n", __func__);
		if (unlikely(SG_IS_DETACHING(sdp)))
			return -ENODEV;
		return put_user(sdev->host->hostt->emulated, ip);
	case SCSI_IOCTL_SEND_COMMAND:
		SG_LOG(3, sfp, "%s:    SCSI_IOCTL_SEND_COMMAND\n", __func__);
		return sg_scsi_ioctl(sdev->request_queue, NULL, filp->f_mode,
				     p);
	case SG_SET_DEBUG:
		SG_LOG(3, sfp, "%s:    SG_SET_DEBUG\n", __func__);
		res = get_user(val, ip);
		if (unlikely(res))
			return res;
		assign_bit(SG_FDEV_LOG_SENSE, sdp->fdev_bm, !!val);
		return 0;
	case BLKSECTGET:
		SG_LOG(3, sfp, "%s:    BLKSECTGET\n", __func__);
		return put_user(max_sectors_bytes(sdev->request_queue), ip);
	case BLKTRACESETUP:
		SG_LOG(3, sfp, "%s:    BLKTRACESETUP\n", __func__);
		return blk_trace_setup(sdev->request_queue,
				       sdp->disk->disk_name,
				       MKDEV(SCSI_GENERIC_MAJOR, sdp->index),
				       NULL, p);
	case BLKTRACESTART:
		SG_LOG(3, sfp, "%s:    BLKTRACESTART\n", __func__);
		return blk_trace_startstop(sdev->request_queue, 1);
	case BLKTRACESTOP:
		SG_LOG(3, sfp, "%s:    BLKTRACESTOP\n", __func__);
		return blk_trace_startstop(sdev->request_queue, 0);
	case BLKTRACETEARDOWN:
		SG_LOG(3, sfp, "%s:    BLKTRACETEARDOWN\n", __func__);
		return blk_trace_remove(sdev->request_queue);
	case SCSI_IOCTL_GET_IDLUN:
		SG_LOG(3, sfp, "%s:    SCSI_IOCTL_GET_IDLUN %s\n", __func__,
		       pmlp);
		break;
	case SCSI_IOCTL_GET_BUS_NUMBER:
		SG_LOG(3, sfp, "%s:    SCSI_IOCTL_GET_BUS_NUMBER%s\n",
		       __func__, pmlp);
		break;
	case SCSI_IOCTL_PROBE_HOST:
		SG_LOG(3, sfp, "%s:    SCSI_IOCTL_PROBE_HOST%s",
		       __func__, pmlp);
		break;
	case SG_GET_TRANSFORM:
		SG_LOG(3, sfp, "%s:    SG_GET_TRANSFORM%s\n", __func__, pmlp);
		break;
	case SG_SET_TRANSFORM:
		SG_LOG(3, sfp, "%s:    SG_SET_TRANSFORM%s\n", __func__, pmlp);
		break;
	default:
		SG_LOG(3, sfp, "%s:    unrecognized ioctl [0x%x]%s\n",
		       __func__, cmd_in, pmlp);
		if (read_only)
			return -EPERM;	/* don't know, so take safer approach */
		break;
	}
	/* ioctl that reach here are forwarded to the mid-level */
	if (likely(scsi_block_when_processing_errors(sdp->device)))
		return scsi_ioctl(sdev, cmd_in, p);
	return -ENXIO;
}

#if IS_ENABLED(CONFIG_COMPAT)
static long
sg_compat_ioctl(struct file *filp, unsigned int cmd_in, unsigned long arg)
{
	struct sg_device *sdp;
	struct sg_fd *sfp;
	struct scsi_device *sdev;

	sfp = filp->private_data;
	SG_LOG(3, sfp, "%s: cmd=0x%x\n", __func__, (int)cmd_in);
	sdp = sfp->parentdp;
	if (unlikely(!sdp))
		return -ENXIO;

	sdev = sdp->device;
	if (sdev->host->hostt->compat_ioctl) {
		int ret;

		ret = sdev->host->hostt->compat_ioctl(sdev, cmd_in,
						      uptr64(arg));
		return ret;
	}
	return -ENOIOCTLCMD;
}
#endif

/*
 * Implements the poll(2) system call for this driver. Returns various EPOLL*
 * flags OR-ed together.
 */
static __poll_t
sg_poll(struct file *filp, poll_table *wait)
{
	__poll_t p_res = 0;
	struct sg_fd *sfp = filp->private_data;

	poll_wait(filp, &sfp->cmpl_wait, wait);
	if (atomic_read(&sfp->waiting) > 0)
		p_res = EPOLLIN | EPOLLRDNORM;

	if (unlikely(SG_IS_DETACHING(sfp->parentdp)))
		p_res |= EPOLLHUP;
	else if (likely(!test_bit(SG_FFD_NO_CMD_Q, sfp->ffd_bm)))
		p_res |= EPOLLOUT | EPOLLWRNORM;
	else if (atomic_read(&sfp->submitted) == 0)
		p_res |= EPOLLOUT | EPOLLWRNORM;
	SG_LOG(3, sfp, "%s: p_res=0x%x\n", __func__, (__force u32)p_res);
	return p_res;
}

static int
sg_fasync(int fd, struct file *filp, int mode)
{
	struct sg_fd *sfp = filp->private_data;

	SG_LOG(3, sfp, "%s: mode(%s)\n", __func__, (mode ? "add" : "remove"));
	return fasync_helper(fd, filp, mode, &sfp->async_qp);
}

/* Note: the error return: VM_FAULT_SIGBUS causes a "bus error" */
static vm_fault_t
sg_vma_fault(struct vm_fault *vmf)
{
	int k, length;
	unsigned long offset, len, sa, iflags;
	struct vm_area_struct *vma = vmf->vma;
	struct sg_scatter_hold *rsv_schp;
	struct sg_request *srp;
	struct sg_device *sdp;
	struct sg_fd *sfp;
	const char *nbp = "==NULL, bad";

	if (unlikely(!vma)) {
		pr_warn("%s: vma%s\n", __func__, nbp);
		goto out_err;
	}
	sfp = vma->vm_private_data;
	if (unlikely(!sfp)) {
		pr_warn("%s: sfp%s\n", __func__, nbp);
		goto out_err;
	}
	sdp = sfp->parentdp;
	if (sdp && unlikely(SG_IS_DETACHING(sdp))) {
		SG_LOG(1, sfp, "%s: device detaching\n", __func__);
		goto out_err;
	}
	srp = sfp->rsv_arr[0].rsv_srp;
	if (unlikely(!srp)) {
		SG_LOG(1, sfp, "%s: srp%s\n", __func__, nbp);
		goto out_err;
	}
	spin_lock_irqsave(&srp->req_lck, iflags);
	rsv_schp = srp->sgatp;
	offset = vmf->pgoff << PAGE_SHIFT;
	if (unlikely(offset >= (unsigned int)rsv_schp->buflen)) {
		SG_LOG(1, sfp, "%s: offset[%lu] >= rsv.buflen\n", __func__,
		       offset);
		goto out_err_unlock;
	}
	sa = vma->vm_start;
	SG_LOG(3, sfp, "%s: vm_start=0x%lx, off=%lu\n", __func__, sa, offset);
	length = 1 << (PAGE_SHIFT + rsv_schp->page_order);
	for (k = 0; k < rsv_schp->num_sgat && sa < vma->vm_end; ++k) {
		len = vma->vm_end - sa;
		len = min_t(int, len, (int)length);
		if (offset < len) {
			struct page *page;
			struct page *pp;

			pp = rsv_schp->pages[k];
			spin_unlock_irqrestore(&srp->req_lck, iflags);
			page = nth_page(pp, offset >> PAGE_SHIFT);
			get_page(page); /* increment page count */
			vmf->page = page;
			return 0; /* success */
		}
		sa += len;
		offset -= len;
	}
out_err_unlock:
	spin_unlock_irqrestore(&srp->req_lck, iflags);
out_err:
	return VM_FAULT_SIGBUS;
}

static const struct vm_operations_struct sg_mmap_vm_ops = {
	.fault = sg_vma_fault,
};

/* Entry point for mmap(2) system call */
static int
sg_mmap(struct file *filp, struct vm_area_struct *vma)
{
	int k, length;
	int ret = 0;
	unsigned long req_sz, len, sa, iflags;
	struct sg_scatter_hold *rsv_schp;
	struct sg_fd *sfp;
	struct sg_request *srp;

	if (unlikely(!filp || !vma))
		return -ENXIO;
	sfp = filp->private_data;
	if (unlikely(!sfp)) {
		pr_warn("sg: %s: sfp is NULL\n", __func__);
		return -ENXIO;
	}
	req_sz = vma->vm_end - vma->vm_start;
	SG_LOG(3, sfp, "%s: vm_start=%pK, len=%d\n", __func__,
	       (void *)vma->vm_start, (int)req_sz);
	if (unlikely(vma->vm_pgoff))
		return -EINVAL; /* only an offset of 0 accepted */
	/* Check reserve request is inactive and has large enough buffer */
	mutex_lock(&sfp->f_mutex);
	srp = sfp->rsv_arr[0].rsv_srp;
	spin_lock_irqsave(&srp->req_lck, iflags);
	if (SG_RS_ACTIVE(srp)) {
		ret = -EBUSY;
		goto out;
	}
	rsv_schp = srp->sgatp;
	if (unlikely(req_sz > (unsigned long)rsv_schp->buflen)) {
		ret = -ENOMEM;
		goto out;
	}
	sa = vma->vm_start;
	length = 1 << (PAGE_SHIFT + rsv_schp->page_order);
	for (k = 0; k < rsv_schp->num_sgat && sa < vma->vm_end; ++k) {
		len = vma->vm_end - sa;
		len = min_t(unsigned long, len, (unsigned long)length);
		sa += len;
	}

	__set_bit(SG_FFD_MMAP_CALLED, sfp->ffd_bm);
	vma->vm_flags |= VM_IO | VM_DONTEXPAND | VM_DONTDUMP;
	vma->vm_private_data = sfp;
	vma->vm_ops = &sg_mmap_vm_ops;
out:
	spin_unlock_irqrestore(&srp->req_lck, iflags);
	mutex_unlock(&sfp->f_mutex);
	return ret;
}

/*
 * This user context function is called from sg_rq_end_io() when a slave was
 * in SWAIT state and sg_rq_end_io() has just been called on the corresponding
 * master request.
 */
static void
sg_uc_rq_end_io_swait(struct work_struct *work)
{
	unsigned long iflags;
	enum sg_rq_state rqq_st;
	struct sg_fd *slave_sfp;
	struct sg_request *ma_srp = container_of(work, struct sg_request,
						 ew_orph.work);
	struct sg_fd *sfp = ma_srp->parentfp;
	struct sg_request *slrp;

	if (unlikely(!sfp)) {
		WARN_ONCE(1, "%s: sfp unexpectedly NULL\n", __func__);
		return;
	}
	slrp = sg_get_slave_by_rsv_lck(sfp, ma_srp);
	if (!slrp) {
		SG_LOG(3, sfp, "%s: couldn't find matching slave srp\n",
		       __func__);
		goto fini;
	}
	SG_LOG(3, sfp, "%s: ma_srp=0x%pK, rq_st: %s, share: %s\n", __func__,
	       ma_srp, sg_rq_st_str(atomic_read(&ma_srp->rq_st), false),
	       sg_shr_str(ma_srp->sh_var, false));
	slave_sfp = slrp->parentfp;

	spin_lock_irqsave(&slrp->req_lck, iflags);
	rqq_st = atomic_read(&slrp->rq_st);
	if (slave_sfp && rqq_st == SG_RS_SHR_SWAIT) {
		if (!(ma_srp->rq_result & SG_ML_RESULT_MSK)) {
			/* master is error/sense free */
			sg_rstate_chg(slrp, rqq_st, SG_RS_INFLIGHT);
			spin_unlock_irqrestore(&slrp->req_lck, iflags);
			sg_execute_cmd(slave_sfp, slrp);
			goto fini;
		}
		/* end slave cmd with ::driver_status=DRIVER_SOFT */
		SG_LOG(1, sfp, "%s: master rq_result=0x%x\n", __func__,
		       ma_srp->rq_result);
		sg_rstate_chg(slrp, rqq_st, SG_RS_AWAIT_RCV);
		spin_unlock_irqrestore(&slrp->req_lck, iflags);
		slrp->rq_result |= (DRIVER_SOFT << 24);
		if (slave_sfp) {
			wake_up_interruptible(&slave_sfp->cmpl_wait);
			kill_fasync(&slave_sfp->async_qp, SIGPOLL, POLL_IN);
		}
	} else {
		SG_LOG(1, sfp, "%s: bad slave state: %s\n", __func__,
		       sg_rq_st_str(rqq_st, false));
		spin_unlock_irqrestore(&slrp->req_lck, iflags);
	}
fini:
	kref_put(&sfp->f_ref, sg_remove_sfp);
}

/*
 * This user context function is called from sg_rq_end_io() when an orphaned
 * request needs to be cleaned up (e.g. when control C is typed while an
 * ioctl(SG_IO) is active).
 */
static void
sg_uc_rq_end_io_orphaned(struct work_struct *work)
{
	struct sg_request *srp = container_of(work, struct sg_request,
					      ew_orph.work);
	struct sg_fd *sfp;

	sfp = srp->parentfp;
	if (unlikely(!sfp)) {
		WARN_ONCE(1, "%s: sfp unexpectedly NULL\n", __func__);
		return;
	}
	SG_LOG(3, sfp, "%s: srp=0x%pK\n", __func__, srp);
	if (test_bit(SG_FRQ_DEACT_ORPHAN, srp->frq_bm)) {
		sg_finish_scsi_blk_rq(srp);	/* clean up orphan case */
		sg_deact_request(sfp, srp);
	}
	kref_put(&sfp->f_ref, sg_remove_sfp);
}

static void
sg_check_sense(struct sg_device *sdp, struct sg_request *srp, int sense_len)
{
	int driver_stat;
	u32 rq_res = srp->rq_result;
	struct scsi_request *scsi_rp = scsi_req(srp->rq);
	u8 *sbp = scsi_rp ? scsi_rp->sense : NULL;

	if (unlikely(!sbp))
		return;
	driver_stat = driver_byte(rq_res);
	if (driver_stat & DRIVER_SENSE) {
		struct scsi_sense_hdr ssh;

		if (scsi_normalize_sense(sbp, sense_len, &ssh)) {
			if (!scsi_sense_is_deferred(&ssh)) {
				if (ssh.sense_key == UNIT_ATTENTION) {
					if (sdp->device->removable)
						sdp->device->changed = 1;
				}
			}
		}
	}
	if (test_bit(SG_FDEV_LOG_SENSE, sdp->fdev_bm) > 0) {
		int scsi_stat = rq_res & 0xff;

		if (scsi_stat == SAM_STAT_CHECK_CONDITION ||
		    scsi_stat == SAM_STAT_COMMAND_TERMINATED)
			__scsi_print_sense(sdp->device, __func__, sbp,
					   sense_len);
	}
}

/*
 * This "bottom half" (soft interrupt) handler is called by the mid-level
 * when a request has completed or failed. This callback is registered in a
 * blk_execute_rq_nowait() call in the sg_common_write(). For ioctl(SG_IO)
 * (sync) usage, sg_ctl_sg_io() waits to be woken up by this callback.
 */
static void
sg_rq_end_io(struct request *rq, blk_status_t status)
{
	bool need_put = true;
	enum sg_rq_state rqq_state = SG_RS_AWAIT_RCV;
	int a_resid, slen;
	struct sg_request *srp = rq->end_io_data;
	struct scsi_request *scsi_rp = scsi_req(rq);
	struct sg_device *sdp;
	struct sg_fd *sfp;

	if (unlikely(!scsi_rp)) {
		WARN_ONCE(1, "%s: scsi_req(rq) unexpectedly NULL\n", __func__);
		return;
	}
	if (unlikely(!srp)) {
		WARN_ONCE(1, "%s: srp unexpectedly NULL\n", __func__);
		return;
	}
	/* Expect 0 --> 1 transition, otherwise processed elsewhere */
	if (unlikely(test_and_set_bit(SG_FRQ_BLK_PUT_REQ, srp->frq_bm))) {
		pr_info("%s: srp=%pK already completed\n", __func__, srp);
		return;
	}
	if (WARN_ON(atomic_read(&srp->rq_st) != SG_RS_INFLIGHT)) {
		pr_warn("%s: bad rq_st=%d\n", __func__,
			atomic_read(&srp->rq_st));
		goto early_err;
	}
	sfp = srp->parentfp;
	if (unlikely(!sfp)) {
		WARN_ONCE(1, "%s: sfp unexpectedly NULL", __func__);
		goto early_err;
	}
	sdp = sfp->parentdp;
	if (unlikely(SG_IS_DETACHING(sdp)))
		pr_info("%s: device detaching\n", __func__);

	srp->rq_result = scsi_rp->result;
	slen = min_t(int, scsi_rp->sense_len, SCSI_SENSE_BUFFERSIZE);
	a_resid = scsi_rp->resid_len;

	if (unlikely(a_resid)) {
		if (test_bit(SG_FRQ_IS_V4I, srp->frq_bm)) {
			if (rq_data_dir(rq) == READ)
				srp->in_resid = a_resid;
			else
				srp->s_hdr4.out_resid = a_resid;
		} else {
			srp->in_resid = a_resid;
		}
	}
	if (likely(srp->rq_result == 0) &&
	    unlikely(test_bit(SG_FRQ_ABORTING, srp->frq_bm)))
		srp->rq_result |= (DRIVER_HARD << 24);

	SG_LOG(6, sfp, "%s: pack/tag_id=%d/%d, cmd=0x%x, res=0x%x\n", __func__,
	       srp->pack_id, srp->tag, srp->cmd_opcode, srp->rq_result);
	if (srp->start_ns > 0)	/* zero only when SG_FFD_NO_DURATION is set */
		srp->duration = sg_calc_rq_dur(srp, test_bit(SG_FFD_TIME_IN_NS,
							     sfp->ffd_bm));
	if (unlikely((srp->rq_result & SG_ML_RESULT_MSK) && slen > 0))
		sg_check_sense(sdp, srp, slen);
	if (unlikely(slen > 0)) {
		if (scsi_rp->sense) {
			srp->sense_bp =
				mempool_alloc(sg_sense_pool,
					      GFP_ATOMIC   /* <-- leave */);
			if (likely(srp->sense_bp)) {
				memcpy(srp->sense_bp, scsi_rp->sense, slen);
				if (slen < SCSI_SENSE_BUFFERSIZE)
				memset(srp->sense_bp + slen, 0,
				       SCSI_SENSE_BUFFERSIZE - slen);
			} else {
				slen = 0;
				pr_warn("%s: sense but can't alloc buffer\n",
					__func__);
			}
		} else {
			slen = 0;
			pr_warn("%s: sense_len>0 but sense==NULL\n", __func__);
		}
	}
	srp->sense_len = slen;
	if (unlikely(test_bit(SG_FRQ_IS_ORPHAN, srp->frq_bm))) {
		spin_lock(&srp->req_lck);
		if (test_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm)) {
			clear_bit(SG_FRQ_SYNC_INVOC, srp->frq_bm);
		} else {
			rqq_state = SG_RS_BUSY;
			set_bit(SG_FRQ_DEACT_ORPHAN, srp->frq_bm);
		}
		spin_unlock(&srp->req_lck);
	}
	if (!SG_SYNC_INVOC(srp))
		atomic_inc(&sfp->waiting);
	if (unlikely(sg_rstate_chg(srp, SG_RS_INFLIGHT, rqq_state)))
		pr_warn("%s: can't set rq_st\n", __func__);
	/*
	 * Free the mid-level resources apart from the bio (if any). The bio's
	 * blk_rq_unmap_user() can be called later from user context.
	 */
	srp->rq = NULL;
	scsi_req_free_cmd(scsi_rp);
	blk_put_request(rq);

	if (unlikely(rqq_state != SG_RS_AWAIT_RCV)) {
		/* clean up orphaned request that aren't being kept */
		INIT_WORK(&srp->ew_orph.work, sg_uc_rq_end_io_orphaned);
		schedule_work(&srp->ew_orph.work);
		return;
	}
	if (test_bit(SG_FRQ_WAKE_MASTER, srp->frq_bm) && sfp->share_sfp) {
		struct sg_fd *ma_sfp;

		ma_sfp = (srp->sh_var == SG_SHR_SL_NOT_SRQ ||
			  srp->sh_var == SG_SHR_SL_RQ) ? sfp->share_sfp : sfp;
		atomic_inc(&ma_sfp->svb_cnt);
		wake_up_interruptible(&ma_sfp->cmpl_wait);
	} else {
		/* Wake any sg_read()/ioctl(SG_IORECEIVE) awaiting this req */
		if (!(srp->rq_flags & SGV4_FLAG_NO_WAITQ))
			wake_up_interruptible(&sfp->cmpl_wait);
		kill_fasync(&sfp->async_qp, SIGPOLL, POLL_IN);
	}
	if (srp->sh_var == SG_SHR_MA_RQ) {
		struct sg_request *slrp = sg_get_slave_by_rsv(sfp, srp);

		if (slrp &&	/* schedule waiting slave rq */
		    atomic_read(&slrp->rq_st) == SG_RS_SHR_SWAIT) {
			INIT_WORK(&srp->ew_orph.work,
				  sg_uc_rq_end_io_swait);
			schedule_work(&srp->ew_orph.work);
			need_put = false;
		}
	}
	if (need_put)
		kref_put(&sfp->f_ref, sg_remove_sfp);
	return;

early_err:
	srp->rq = NULL;
	if (likely(scsi_rp))
		scsi_req_free_cmd(scsi_rp);
	blk_put_request(rq);
}

static const struct file_operations sg_fops = {
	.owner = THIS_MODULE,
	.read = sg_read,
	.write = sg_write,
	.poll = sg_poll,
	.unlocked_ioctl = sg_ioctl,
#if IS_ENABLED(CONFIG_COMPAT)
	.compat_ioctl = sg_compat_ioctl,
#endif
	.open = sg_open,
	.mmap = sg_mmap,
	.release = sg_release,
	.fasync = sg_fasync,
	.llseek = no_llseek,
};

static struct class *sg_sysfs_class;

static bool sg_sysfs_valid;

/* Returns valid pointer to sg_device or negated errno twisted by ERR_PTR */
static struct sg_device *
sg_alloc(struct gendisk *disk, struct scsi_device *scsidp)
{
	struct sg_device *sdp;
	int error;
	u32 k;
	unsigned long iflags;

	sdp = kzalloc(sizeof(struct sg_device), GFP_KERNEL);
	if (unlikely(!sdp))
		return ERR_PTR(-ENOMEM);

	idr_preload(GFP_KERNEL);
	write_lock_irqsave(&sg_index_lock, iflags);

	error = idr_alloc(&sg_index_idr, sdp, 0, SG_MAX_DEVS, GFP_NOWAIT);
	if (unlikely(error < 0)) {
		if (error == -ENOSPC) {
			sdev_printk(KERN_WARNING, scsidp,
				    "Unable to attach sg device type=%d, minor number exceeds %d\n",
				    scsidp->type, SG_MAX_DEVS - 1);
			error = -ENODEV;
		} else {
			sdev_printk(KERN_WARNING, scsidp,
				"%s: idr allocation sg_device failure: %d\n",
				    __func__, error);
		}
		goto out_unlock;
	}
	k = error;

	SCSI_LOG_TIMEOUT(3, sdev_printk(KERN_INFO, scsidp,
			 "%s: dev=%d, sdp=0x%pK ++\n", __func__, k, sdp));
	sprintf(disk->disk_name, "sg%d", k);
	disk->first_minor = k;
	sdp->disk = disk;
	sdp->device = scsidp;
	mutex_init(&sdp->open_rel_lock);
	INIT_LIST_HEAD(&sdp->sfds);
	init_waitqueue_head(&sdp->open_wait);
	atomic_set(&sdp->open_cnt, 0);
	rwlock_init(&sdp->sfd_lock);
	sg_calc_sgat_param(sdp);
	sdp->index = k;
	kref_init(&sdp->d_ref);
	error = 0;

out_unlock:
	write_unlock_irqrestore(&sg_index_lock, iflags);
	idr_preload_end();

	if (unlikely(error)) {
		kfree(sdp);
		return ERR_PTR(error);
	}
	return sdp;
}

static int
sg_add_device(struct device *cl_dev, struct class_interface *cl_intf)
{
	struct scsi_device *scsidp = to_scsi_device(cl_dev->parent);
	struct gendisk *disk;
	struct sg_device *sdp = NULL;
	struct cdev *cdev = NULL;
	int error;
	unsigned long iflags;

	disk = alloc_disk(1);
	if (unlikely(!disk)) {
		pr_warn("%s: alloc_disk failed\n", __func__);
		return -ENOMEM;
	}
	disk->major = SCSI_GENERIC_MAJOR;

	error = -ENOMEM;
	cdev = cdev_alloc();
	if (unlikely(!cdev)) {
		pr_warn("%s: cdev_alloc failed\n", __func__);
		goto out;
	}
	cdev->owner = THIS_MODULE;
	cdev->ops = &sg_fops;

	sdp = sg_alloc(disk, scsidp);
	if (IS_ERR(sdp)) {
		pr_warn("%s: sg_alloc failed\n", __func__);
		error = PTR_ERR(sdp);
		goto out;
	}

	error = cdev_add(cdev, MKDEV(SCSI_GENERIC_MAJOR, sdp->index), 1);
	if (unlikely(error))
		goto cdev_add_err;

	sdp->cdev = cdev;
	if (likely(sg_sysfs_valid)) {
		struct device *sg_class_member;

		sg_class_member = device_create(sg_sysfs_class, cl_dev->parent,
						MKDEV(SCSI_GENERIC_MAJOR,
						      sdp->index),
						sdp, "%s", disk->disk_name);
		if (IS_ERR(sg_class_member)) {
			pr_err("%s: device_create failed\n", __func__);
			error = PTR_ERR(sg_class_member);
			goto cdev_add_err;
		}
		error = sysfs_create_link(&scsidp->sdev_gendev.kobj,
					  &sg_class_member->kobj, "generic");
		if (unlikely(error))
			pr_err("%s: unable to make symlink 'generic' back "
			       "to sg%d\n", __func__, sdp->index);
	} else
		pr_warn("%s: sg_sys Invalid\n", __func__);

	sdev_printk(KERN_NOTICE, scsidp, "Attached scsi generic sg%d "
		    "type %d\n", sdp->index, scsidp->type);

	dev_set_drvdata(cl_dev, sdp);
	return 0;

cdev_add_err:
	write_lock_irqsave(&sg_index_lock, iflags);
	idr_remove(&sg_index_idr, sdp->index);
	write_unlock_irqrestore(&sg_index_lock, iflags);
	kfree(sdp);

out:
	put_disk(disk);
	if (cdev)
		cdev_del(cdev);
	return error;
}

static void
sg_device_destroy(struct kref *kref)
{
	struct sg_device *sdp = container_of(kref, struct sg_device, d_ref);
	unsigned long iflags;

	/* CAUTION!  Note that the device can still be found via idr_find()
	 * even though the refcount is 0.  Therefore, do idr_remove() BEFORE
	 * any other cleanup.
	 */

	write_lock_irqsave(&sg_index_lock, iflags);
	idr_remove(&sg_index_idr, sdp->index);
	write_unlock_irqrestore(&sg_index_lock, iflags);

	sdev_printk(KERN_INFO, sdp->device, "[tid=%d] %s: sdp=0x%pK --\n",
		    (current ? current->pid : -1), __func__, sdp);

	put_disk(sdp->disk);
	kfree(sdp);
}

static void
sg_remove_device(struct device *cl_dev, struct class_interface *cl_intf)
{
	struct scsi_device *scsidp = to_scsi_device(cl_dev->parent);
	struct sg_device *sdp = dev_get_drvdata(cl_dev);
	unsigned long iflags;
	struct sg_fd *sfp;

	if (unlikely(!sdp))
		return;
	/* set this flag as soon as possible as it could be a surprise */
	if (test_and_set_bit(SG_FDEV_DETACHING, sdp->fdev_bm))
		return; /* only want to do following once per device */

	SCSI_LOG_TIMEOUT(3, sdev_printk(KERN_INFO, sdp->device,
					"%s: 0x%pK\n", __func__, sdp));

	read_lock_irqsave(&sdp->sfd_lock, iflags);
	list_for_each_entry(sfp, &sdp->sfds, sfd_entry) {
		wake_up_interruptible_all(&sfp->cmpl_wait);
		kill_fasync(&sfp->async_qp, SIGPOLL, POLL_HUP);
	}
	wake_up_interruptible_all(&sdp->open_wait);
	read_unlock_irqrestore(&sdp->sfd_lock, iflags);

	sysfs_remove_link(&scsidp->sdev_gendev.kobj, "generic");
	device_destroy(sg_sysfs_class, MKDEV(SCSI_GENERIC_MAJOR, sdp->index));
	cdev_del(sdp->cdev);
	sdp->cdev = NULL;

	kref_put(&sdp->d_ref, sg_device_destroy);
}

static int __init
init_sg(void)
{
	int rc;

	/* check scatter_elem_sz module parameter, change if inappropriate */
	if (scatter_elem_sz < (int)PAGE_SIZE)
		scatter_elem_sz = PAGE_SIZE;
	else if (scatter_elem_sz != (1 << ilog2(scatter_elem_sz)))
		scatter_elem_sz = 1 << ilog2(scatter_elem_sz);
	/* scatter_elem_sz rounded down to power of 2, or PAGE_SIZE */
	if (def_reserved_size >= 0)
		sg_big_buff = def_reserved_size;
	else
		def_reserved_size = sg_big_buff;

	rc = register_chrdev_region(MKDEV(SCSI_GENERIC_MAJOR, 0),
				    SG_MAX_DEVS, "sg");
	if (unlikely(rc))
		return rc;

	sg_sense_cache = kmem_cache_create("sg_sense", SCSI_SENSE_BUFFERSIZE,
					   0, 0, NULL);
	if (!sg_sense_cache) {
		pr_err("sg: can't init sense cache\n");
		rc = -ENOMEM;
		goto err_out_unreg;
	}
	sg_sense_pool = mempool_create_slab_pool(SG_MEMPOOL_MIN_NR,
						 sg_sense_cache);
	if (!sg_sense_pool) {
		pr_err("sg: can't init sense pool\n");
		rc = -ENOMEM;
		goto err_out_cache;
	}

	pr_info("Registered %s[char major=0x%x], version: %s, date: %s\n",
		"sg device ", SCSI_GENERIC_MAJOR, SG_VERSION_STR,
		sg_version_date);
	sg_sysfs_class = class_create(THIS_MODULE, "scsi_generic");
	if (IS_ERR(sg_sysfs_class)) {
		rc = PTR_ERR(sg_sysfs_class);
		goto err_out_pool;
	}
	sg_sysfs_valid = true;
	rc = scsi_register_interface(&sg_interface);
	if (likely(rc == 0)) {
		sg_proc_init();
		sg_dfs_init();
		return 0;
	}
	class_destroy(sg_sysfs_class);

err_out_pool:
	mempool_destroy(sg_sense_pool);
err_out_cache:
	kmem_cache_destroy(sg_sense_cache);
err_out_unreg:
	unregister_chrdev_region(MKDEV(SCSI_GENERIC_MAJOR, 0), SG_MAX_DEVS);
	return rc;
}

static void __exit
exit_sg(void)
{
	sg_dfs_exit();
	if (IS_ENABLED(CONFIG_SCSI_PROC_FS))
		remove_proc_subtree("scsi/sg", NULL);
	kfree(snapped_buf);
	scsi_unregister_interface(&sg_interface);
	mempool_destroy(sg_sense_pool);
	kmem_cache_destroy(sg_sense_cache);
	class_destroy(sg_sysfs_class);
	sg_sysfs_valid = false;
	unregister_chrdev_region(MKDEV(SCSI_GENERIC_MAJOR, 0),
				 SG_MAX_DEVS);
	idr_destroy(&sg_index_idr);
}

/*
 * Setup to move data between kernel buffers managed by this driver and a SCSI
 * device. Note that there is no corresponding 'unmap' call as is required by
 * blk_rq_map_user() . blk_rq has already been told direction of transfer.
 */
static int
sg_rq_map_kern(struct sg_request *srp, struct request_queue *q,
	       struct request *rq)
{
	int k, pg_sz, dlen, ln;
	int res = 0;
	struct sg_scatter_hold *schp = srp->sgatp;

	dlen = schp->dlen;
	pg_sz = 1 << (PAGE_SHIFT + schp->page_order);
	SG_LOG(4, srp->parentfp, "%s: dlen=%d, pg_sz=%d\n", __func__, dlen,
	       pg_sz);
	for (k = 0; k < schp->num_sgat && dlen > 0; ++k, dlen -= ln) {
		ln = min_t(int, dlen, pg_sz);
		res = blk_rq_map_kern(q, rq, page_address(schp->pages[k]),
				      ln, GFP_ATOMIC);
		if (unlikely(res))
			break;
	}
	return res;
}

static inline bool
sg_chk_dio_allowed(struct sg_device *sdp, struct sg_request *srp,
		   int iov_count, int dir)
{
	if (unlikely(sg_allow_dio && (srp->rq_flags & SG_FLAG_DIRECT_IO))) {
		if (dir != SG_DXFER_UNKNOWN && !iov_count) {
			if (!sdp->device->host->unchecked_isa_dma)
				return true;
		}
	}
	return false;
}

static void
sg_set_map_data(const struct sg_scatter_hold *schp, bool up_valid,
		struct rq_map_data *mdp)
{
	memset(mdp, 0, sizeof(*mdp));
	mdp->pages = schp->pages;
	mdp->page_order = schp->page_order;
	mdp->nr_entries = schp->num_sgat;
	mdp->offset = 0;
	mdp->null_mapped = !up_valid;
}

static int
sg_start_req(struct sg_request *srp, struct sg_comm_wr_t *cwrp, int dxfer_dir)
{
	bool no_xfer, us_xfer;
	int res = 0;
	int dxfer_len = 0;
	int r0w = READ;
	unsigned int iov_count = 0;
	void __user *up;
	struct request *rq;
	struct scsi_request *scsi_rp;
	struct sg_fd *sfp = cwrp->sfp;
	struct sg_device *sdp;
	struct sg_scatter_hold *req_schp;
	struct request_queue *q;
	struct rq_map_data *md = (void *)srp; /* want any non-NULL value */
	u8 *long_cmdp = NULL;
	__maybe_unused const char *cp = "";
	struct rq_map_data map_data;

	sdp = sfp->parentdp;
	if (cwrp->cmd_len > BLK_MAX_CDB) {	/* for longer SCSI cdb_s */
		long_cmdp = kzalloc(cwrp->cmd_len, GFP_KERNEL);
		if (unlikely(!long_cmdp)) {
			res = -ENOMEM;
			goto err_out;
		}
		SG_LOG(5, sfp, "%s: long_cmdp=0x%pK ++\n", __func__, long_cmdp);
	}
	if (test_bit(SG_FRQ_IS_V4I, srp->frq_bm)) {
		struct sg_io_v4 *h4p = cwrp->h4p;

		if (dxfer_dir == SG_DXFER_TO_DEV) {
			r0w = WRITE;
			up = uptr64(h4p->dout_xferp);
			dxfer_len = (int)h4p->dout_xfer_len;
			iov_count = h4p->dout_iovec_count;
		} else if (dxfer_dir == SG_DXFER_FROM_DEV) {
			up = uptr64(h4p->din_xferp);
			dxfer_len = (int)h4p->din_xfer_len;
			iov_count = h4p->din_iovec_count;
		} else {
			up = NULL;
		}
	} else {
		struct sg_slice_hdr3 *sh3p = &srp->s_hdr3;

		up = sh3p->dxferp;
		dxfer_len = (int)sh3p->dxfer_len;
		iov_count = sh3p->iovec_count;
		r0w = dxfer_dir == SG_DXFER_TO_DEV ? WRITE : READ;
	}
	SG_LOG(4, sfp, "%s: dxfer_len=%d, data-%s\n", __func__, dxfer_len,
	       (r0w ? "OUT" : "IN"));
	q = sdp->device->request_queue;

	/*
	 * For backward compatibility default to using blocking variant even
	 * when in non-blocking (async) mode. If the SG_CTL_FLAGM_MORE_ASYNC
	 * boolean set on this file descriptor, returns -EAGAIN if
	 * blk_get_request(BLK_MQ_REQ_NOWAIT) yields EAGAIN (aka EWOULDBLOCK).
	 */
	rq = blk_get_request(q, (r0w ? REQ_OP_SCSI_OUT : REQ_OP_SCSI_IN),
			     (test_bit(SG_FFD_MORE_ASYNC, sfp->ffd_bm) ?
						BLK_MQ_REQ_NOWAIT : 0));
	if (IS_ERR(rq)) {
		kfree(long_cmdp);
		res = PTR_ERR(rq);
		goto err_out;
	}
	/* current sg_request protected by SG_RS_BUSY state */
	scsi_rp = scsi_req(rq);
	srp->rq = rq;

	if (cwrp->cmd_len > BLK_MAX_CDB)
		scsi_rp->cmd = long_cmdp;
	if (cwrp->u_cmdp)
		res = sg_fetch_cmnd(sfp, cwrp->u_cmdp, cwrp->cmd_len,
				    scsi_rp->cmd);
	else if (likely(cwrp->cmdp))
		memcpy(scsi_rp->cmd, cwrp->cmdp, cwrp->cmd_len);
	else
		res = -EPROTO;
	if (unlikely(res)) {
		kfree(long_cmdp);
		goto err_out;
	}
	scsi_rp->cmd_len = cwrp->cmd_len;
	srp->cmd_opcode = scsi_rp->cmd[0];
	no_xfer = dxfer_len <= 0 || dxfer_dir == SG_DXFER_NONE;
	us_xfer = !(srp->rq_flags & SG_FLAG_NO_DXFER);
	__assign_bit(SG_FRQ_NO_US_XFER, srp->frq_bm, no_xfer || !us_xfer);
	rq->end_io_data = srp;
	scsi_rp->retries = SG_DEFAULT_RETRIES;
	req_schp = srp->sgatp;

	if (no_xfer) {
		SG_LOG(4, sfp, "%s: no data xfer [0x%pK]\n", __func__, srp);
		goto fini;	/* path of reqs with no din nor dout */
	} else if (sg_chk_dio_allowed(sdp, srp, iov_count, dxfer_dir) &&
		   blk_rq_aligned(q, (unsigned long)up, dxfer_len)) {
		set_bit(SG_FRQ_DIO_IN_USE, srp->frq_bm);
		srp->rq_info |= SG_INFO_DIRECT_IO;
		md = NULL;
		if (IS_ENABLED(CONFIG_SCSI_PROC_FS))
			cp = "direct_io, ";
	} else {	/* normal IO and failed conditions for dio path */
		md = &map_data;
	}

	if (likely(md)) {	/* normal, "indirect" IO */
		if (unlikely(srp->rq_flags & SG_FLAG_MMAP_IO)) {
			/* mmap IO must use and fit in reserve request */
			bool reserve0 = (sfp->rsv_arr[0].rsv_srp == srp);

			if (unlikely(!reserve0 ||
				     dxfer_len > req_schp->buflen))
				res = reserve0 ? -ENOMEM : -EBUSY;
		} else if (req_schp->buflen == 0) {
			int up_sz = max_t(int, dxfer_len, sfp->sgat_elem_sz);

			res = sg_mk_sgat(srp, sfp, up_sz);
		}
		if (unlikely(res))
			goto fini;

		sg_set_map_data(req_schp, !!up, md);
		md->from_user = (dxfer_dir == SG_DXFER_TO_FROM_DEV);
	}

	if (unlikely(iov_count)) {
		struct iovec *iov = NULL;
		struct iov_iter i;

		res = import_iovec(r0w, up, iov_count, 0, &iov, &i);
		if (unlikely(res < 0))
			goto fini;

		iov_iter_truncate(&i, dxfer_len);
		if (unlikely(!iov_iter_count(&i))) {
			kfree(iov);
			res = -EINVAL;
			goto fini;
		}

		if (us_xfer)
			res = blk_rq_map_user_iov(q, rq, md, &i, GFP_ATOMIC);
		kfree(iov);
		if (IS_ENABLED(CONFIG_SCSI_PROC_FS))
			cp = "iov_count > 0";
	} else if (us_xfer) { /* setup for transfer data to/from user space */
		res = blk_rq_map_user(q, rq, md, up, dxfer_len, GFP_ATOMIC);
#if IS_ENABLED(SG_LOG_ACTIVE)
		if (unlikely(res))
			SG_LOG(1, sfp, "%s: blk_rq_map_user() res=%d\n",
			       __func__, res);
#endif
	} else if (!test_bit(SG_FRQ_DEFER_XFR, srp->frq_bm)) {
		/* transfer data to/from kernel buffers */
		res = sg_rq_map_kern(srp, q, rq);
	}
fini:
	if (likely(res == 0)) {
		res = sg_rstate_chg(srp, SG_RS_BUSY, SG_RS_INFLIGHT);
		if (likely(res == 0))
			srp->bio = rq->bio;
	}
	if (unlikely(res && rq)) {		/* failure, free up resources */
		scsi_req_free_cmd(scsi_rp);
		if (likely(!test_and_set_bit(SG_FRQ_BLK_PUT_REQ,
					     srp->frq_bm))) {
			srp->rq = NULL;
			blk_put_request(rq);
		}
	}
err_out:
	SG_LOG((res ? 1 : 4), sfp, "%s: %s %s res=%d [0x%pK]\n", __func__,
	       sg_shr_str(srp->sh_var, false), cp, res, srp);
	return res;
}

/*
 * Clean up mid-level and block layer resources of finished request. Sometimes
 * blk_rq_unmap_user() returns -4 (-EINTR) and this is why: "If we're in a
 * workqueue, the request is orphaned, so don't copy into a random user
 * address space, just free and return -EINTR so user space doesn't expect
 * any data." [block/bio.c]
 */
static void
sg_finish_scsi_blk_rq(struct sg_request *srp)
{
	int ret;
	struct sg_fd *sfp = srp->parentfp;
	char b[32];

	SG_LOG(4, sfp, "%s: srp=0x%pK%s\n", __func__, srp,
	       sg_get_rsv_str_lck(srp, " ", "", sizeof(b), b));
	if (!SG_SYNC_INVOC(srp)) {
		atomic_dec(&sfp->submitted);
		atomic_dec(&sfp->waiting);
	}
	if (likely(srp->bio)) {
		bool us_xfer = !test_bit(SG_FRQ_NO_US_XFER, srp->frq_bm);

		if (us_xfer) {
			ret = blk_rq_unmap_user(srp->bio);
			if (unlikely(ret)) { /* -EINTR (-4) can be ignored */
				SG_LOG(6, sfp,
				       "%s: blk_rq_unmap_user() --> %d\n",
				       __func__, ret);
			}
		}
		srp->bio = NULL;
	}
	/* In worst case READ data returned to user space by this point */

	/* Expect blk_put_request(rq) already called in sg_rq_end_io() */
	if (unlikely(!test_and_set_bit(SG_FRQ_BLK_PUT_REQ, srp->frq_bm))) {
		struct request *rq = srp->rq;

		if (rq) {	/* blk_get_request() may have failed */
			if (scsi_req(rq))
				scsi_req_free_cmd(scsi_req(rq));
			srp->rq = NULL;
			blk_put_request(rq);
		}
	}
}

static int
sg_mk_sgat(struct sg_request *srp, struct sg_fd *sfp, int minlen)
{
	int j, k, rem_sz, order, align_sz;
	int m_size = minlen;
	int rup_sz = 0;
	int mx_sgat_elems = sfp->parentdp->max_sgat_elems;
	u32 elem_sz;
	const size_t struct_page_sz = sizeof(struct page *);
	gfp_t mask_ap = GFP_ATOMIC | __GFP_COMP | __GFP_NOWARN | __GFP_ZERO;
	gfp_t mask_kz = GFP_ATOMIC | __GFP_NOWARN;
	struct sg_device *sdp = sfp->parentdp;
	struct sg_scatter_hold *schp = srp->sgatp;

	if (unlikely(m_size < 0))
		return -EFAULT;
	if (unlikely(m_size == 0))
		++m_size;	/* don't remember why */
	/* round request up to next highest SG_DEF_SECTOR_SZ byte boundary */
	align_sz = ALIGN(m_size, SG_DEF_SECTOR_SZ);

	schp->pages = kcalloc(mx_sgat_elems, struct_page_sz, mask_kz);
	SG_LOG(4, sfp, "%s: minlen=%d, align_sz=%d [sz=%zu, 0x%pK ++]\n",
	       __func__, minlen, align_sz, mx_sgat_elems * struct_page_sz,
	       schp->pages);
	if (unlikely(!schp->pages))
		return -ENOMEM;

	elem_sz = sfp->sgat_elem_sz;    /* power of 2 and >= PAGE_SIZE */
	if (sdp && unlikely(sdp->device->host->unchecked_isa_dma))
		mask_ap |= GFP_DMA;
	/* PAGE_SIZE == (1 << PAGE_SHIFT) == (2 ** PAGE_SHIFT) */
	order = get_order(elem_sz); /* --> ilog2(1+((elem_sz-1)/PAGE_SIZE)) */
again:
	rup_sz = 1 << (PAGE_SHIFT + order);	/* round up size */

	for (k = 0, rem_sz = align_sz; rem_sz > 0 && k < mx_sgat_elems;
	     ++k, rem_sz -= rup_sz) {
		schp->pages[k] = alloc_pages(mask_ap, order);
		if (unlikely(!schp->pages[k]))
			goto err_out;
		SG_LOG(5, sfp, "%s: k=%d, rup_sz=%d [0x%pK ++]\n", __func__, k,
		       rup_sz, schp->pages[k]);
	}
	schp->page_order = order;
	schp->num_sgat = k;
	SG_LOG(5, sfp, "%s: num_sgat=%d, order=%d\n", __func__, k, order);
	if (unlikely(rem_sz > 0))       /* must have failed */
		return -ENOMEM;
	schp->buflen = align_sz;
	if (sfp->tot_fd_thresh > 0)
		atomic_add(align_sz, &sfp->sum_fd_dlens);
	return 0;
err_out:
	for (j = 0; j < k; ++j)
		__free_pages(schp->pages[j], order);

	if (--order >= 0)
		goto again;
	kfree(schp->pages);
	return -ENOMEM;
}

static void
sg_remove_sgat_helper(struct sg_fd *sfp, struct sg_scatter_hold *schp)
{
	int k;
	void *p;

	if (!schp->pages)
		return;
	for (k = 0; k < schp->num_sgat; ++k) {
		p = schp->pages[k];
		SG_LOG(5, sfp, "%s: pg[%d]=0x%pK --\n", __func__, k, p);
		if (unlikely(!p))
			continue;
		__free_pages(p, schp->page_order);
	}
	SG_LOG(5, sfp, "%s: pg_order=%u, free pgs=0x%pK --\n", __func__,
	       schp->page_order, schp->pages);
	kfree(schp->pages);
}

/* Remove the data (possibly a sgat list) held by srp, not srp itself */
static void
sg_remove_sgat(struct sg_request *srp)
{
	struct sg_scatter_hold *schp = &srp->sgat_h; /* care: remove own data */
	struct sg_fd *sfp = srp->parentfp;
	char b[48];

	SG_LOG(4, sfp, "%s: num_sgat=%d%s\n", __func__, schp->num_sgat,
	       sg_get_rsv_str_lck(srp, " [", "]", sizeof(b), b));
	if (!test_bit(SG_FRQ_DIO_IN_USE, srp->frq_bm))
		sg_remove_sgat_helper(sfp, schp);

	if (sfp->tot_fd_thresh > 0) {
		/* this is a subtraction, error if it goes negative */
		if (atomic_add_negative(-schp->buflen, &sfp->sum_fd_dlens)) {
			SG_LOG(2, sfp, "%s: logic error: this dlen > %s\n",
			       __func__, "sum_fd_dlens");
			atomic_set(&sfp->sum_fd_dlens, 0);
		}
	}
	memset(schp, 0, sizeof(*schp));         /* zeros buflen and dlen */
}

/*
 * For sg v1 and v2 interface: with a command yielding a data-in buffer, after
 * it has arrived in kernel memory, this function copies it to the user space,
 * appended to given struct sg_header object.
 */
static int
sg_rd_append(struct sg_request *srp, void __user *outp, int num_xfer)
{
	int k, num, res;
	struct page *pgp;
	struct sg_scatter_hold *schp = srp->sgatp;

	SG_LOG(4, srp->parentfp, "%s: num_xfer=%d\n", __func__, num_xfer);
	if (unlikely(!outp || num_xfer <= 0))
		return (num_xfer == 0 && outp) ? 0 : -EINVAL;

	num = 1 << (PAGE_SHIFT + schp->page_order);
	for (k = 0, res = 0; k < schp->num_sgat; ++k) {
		pgp = schp->pages[k];
		if (unlikely(!pgp)) {
			res = -ENXIO;
			break;
		}
		if (num > num_xfer) {
			if (unlikely(copy_to_user(outp, page_address(pgp),
						  num_xfer)))
				res = -EFAULT;
			break;
		} else {
			if (unlikely(copy_to_user(outp, page_address(pgp),
						  num))) {
				res = -EFAULT;
				break;
			}
			num_xfer -= num;
			if (num_xfer <= 0)
				break;
			outp += num;
		}
	}
	return res;
}

/*
 * If there are multiple requests outstanding, the speed of this function is
 * important. 'id' is pack_id when is_tag=false, otherwise it is a tag. Both
 * SG_PACK_ID_WILDCARD and SG_TAG_WILDCARD are -1 and that case is typically
 * the fast path. This function is only used in the non-blocking cases.
 * Returns pointer to (first) matching sg_request or NULL. If found,
 * sg_request state is moved from SG_RS_AWAIT_RCV to SG_RS_BUSY.
 */
static struct sg_request *
sg_find_srp_by_id(struct sg_fd *sfp, int id, bool is_tag)
{
	__maybe_unused bool is_bad_st = false;
	__maybe_unused enum sg_rq_state bad_sr_st = SG_RS_INACTIVE;
	bool search_for_1 = (id != SG_TAG_WILDCARD);
	enum sg_rq_state sr_st;
	int res;
	struct sg_request *srp = NULL;

	if (atomic_read(&sfp->waiting) < 1)
		return NULL;
	rcu_read_lock();
	if (unlikely(search_for_1)) {
		list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
			if (SG_SYNC_INVOC(srp))
				continue;
			if (unlikely(is_tag)) {
				if (srp->tag != id)
					continue;
			} else {
				if (srp->pack_id != id)
					continue;
			}
			sr_st = atomic_read(&srp->rq_st);
			switch (sr_st) {
			case SG_RS_AWAIT_RCV:
				res = sg_rstate_chg(srp, sr_st, SG_RS_BUSY);
				if (likely(res == 0))
					goto good;
				/* else another caller got it, move on */
				if (IS_ENABLED(CONFIG_SCSI_PROC_FS)) {
					is_bad_st = true;
					bad_sr_st = atomic_read(&srp->rq_st);
				}
				break;
			case SG_RS_SHR_SLAVE:
				goto good;
			case SG_RS_INFLIGHT:
			case SG_RS_SHR_SWAIT:	/* awaiting master's finish */
				break;
			default:
				if (IS_ENABLED(CONFIG_SCSI_PROC_FS)) {
					is_bad_st = true;
					bad_sr_st = sr_st;
				}
				break;
			}
			break;	/* problem if same id on > 1 requests */
		}
	} else {        /* search for any request is more likely */
		list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
			if (SG_SYNC_INVOC(srp))
				continue;
			sr_st = atomic_read(&srp->rq_st);
			if (sr_st == SG_RS_AWAIT_RCV) {
				if (likely(sg_rstate_chg(srp, sr_st,
							 SG_RS_BUSY) == 0))
					goto good;
			}
		}
	}
	/* here if one of above loops does _not_ find a match */
	rcu_read_unlock();
#if IS_ENABLED(SG_LOG_ACTIVE)
	if (search_for_1) {
		const char *cptp = is_tag ? "tag=" : "pack_id=";

		if (unlikely(is_bad_st))
			SG_LOG(1, sfp, "%s: %s%d wrong state: %s\n", __func__,
			       cptp, id, sg_rq_st_str(bad_sr_st, true));
		else
			SG_LOG(6, sfp, "%s: %s%d not awaiting read\n",
			       __func__, cptp, id);
	}
#endif
	return NULL;
good:
	rcu_read_unlock();
	if (search_for_1) {
		SG_LOG(6, sfp, "%s: %s%d found [srp=0x%pK]\n", __func__,
		       (is_tag ? "tag=" : "pack_id="), id, srp);
	}
	return srp;
}

/*
 * Returns true if a request is ready and its srp is written to *srpp . If
 * nothing can be found (because nothing is currently submitted) then true
 * is returned and ERR_PTR(-ENODATA) --> *srpp . If nothing is found but
 * sfp has requests submitted, returns false and NULL --> *srpp . If device
 * is detaching, returns true and NULL --> *srpp .
 */
static bool
sg_mrq_get_ready_srp(struct sg_fd *sfp, struct sg_request **srpp)
{
	enum sg_rq_state sr_st;
	struct sg_request *srp;

	if (unlikely(SG_IS_DETACHING(sfp->parentdp))) {
		*srpp = NULL;
		return true;
	}
	rcu_read_lock();
	if (atomic_read(&sfp->submitted) < 1) {
		rcu_read_unlock();
		*srpp = ERR_PTR(-ENODATA);
		return true;
	}
	if (atomic_read(&sfp->waiting) < 1)
		goto fini;

	list_for_each_entry_rcu(srp, &sfp->rq_list, rq_entry) {
		if (SG_SYNC_INVOC(srp))
			continue;
		sr_st = atomic_read(&srp->rq_st);
		if (sr_st == SG_RS_AWAIT_RCV) {
			if (likely(sg_rstate_chg(srp, sr_st, SG_RS_BUSY) ==
				   0)) {
				rcu_read_unlock();
				*srpp = srp;
				return true;
			}
		}
	}
fini:
	rcu_read_unlock();
	*srpp = NULL;
	return false;
}

/*
 * Makes a new sg_request object. If 'first' is set then use GFP_KERNEL which
 * may take time but has improved chance of success, otherwise use GFP_ATOMIC.
 * Note that basic initialization is done but srp is not added to either sfp
 * list. On error returns twisted negated errno value (not NULL).
 */
static struct sg_request *
sg_mk_srp(struct sg_fd *sfp, bool first)
{
	struct sg_request *srp;
	gfp_t gfp = __GFP_NOWARN;

	if (first)      /* prepared to wait if none already outstanding */
		srp = kzalloc(sizeof(*srp), gfp | GFP_KERNEL);
	else
		srp = kzalloc(sizeof(*srp), gfp | GFP_ATOMIC);
	if (likely(srp)) {
		spin_lock_init(&srp->req_lck);
		atomic_set(&srp->rq_st, SG_RS_INACTIVE);
		srp->sh_var =  SG_SHR_NONE;
		srp->parentfp = sfp;
		srp->tag = SG_TAG_WILDCARD;
		srp->sgatp = &srp->sgat_h; /* only slave share changes sgatp */
		return srp;
	} else {
		return ERR_PTR(-ENOMEM);
	}
}

static struct sg_request *
sg_mk_srp_sgat(struct sg_fd *sfp, bool first, int db_len)
{
	int res;
	struct sg_request *n_srp = sg_mk_srp(sfp, first);

	if (IS_ERR(n_srp))
		return n_srp;
	if (db_len > 0) {
		res = sg_mk_sgat(n_srp, sfp, db_len);
		if (unlikely(res)) {
			kfree(n_srp);
			return ERR_PTR(res);
		}
	}
	return n_srp;
}

/*
 * Irrespective of the given reserve request size, the minimum size requested
 * will be PAGE_SIZE (often 4096 bytes). Returns a pointer to reserve object or
 * a negated errno value twisted by ERR_PTR() macro. The actual number of bytes
 * allocated (maybe less than buflen) is in srp->sgatp->buflen . Note that this
 * function is only called in contexts where locking is not required.
 */
static struct sg_request *
sg_build_reserve(struct sg_fd *sfp, int buflen)
{
	bool go_out = false;
	int res, idx;
	struct sg_request *srp;
	struct sg_rsv_slave *rsp;

	SG_LOG(3, sfp, "%s: buflen=%d\n", __func__, buflen);
	idx = sg_get_idx_new_lck(sfp);
	if (idx < 0) {
		SG_LOG(1, sfp, "%s: sg_get_idx_new_lck() failed\n", __func__);
		return ERR_PTR(-EFBIG);
	}
	rsp = sfp->rsv_arr + idx;
	srp = sg_mk_srp(sfp, list_empty(&sfp->rq_fl));
	if (IS_ERR(srp)) {
		rsp->rsv_srp = NULL;
		sg_place_at_idx_lck(sfp, idx, NULL, NULL);
		return srp;
	}
	rsp->rsv_srp = srp;
	do {
		if (buflen < (int)PAGE_SIZE) {
			buflen = PAGE_SIZE;
			go_out = true;
		}
		res = sg_mk_sgat(srp, sfp, buflen);
		if (likely(res == 0)) {
			sg_place_at_idx_lck(sfp, idx, srp, NULL);
			SG_LOG(4, sfp,
			       "%s: rsv%d: final buflen=%d, srp=0x%pK ++\n",
			       __func__, idx, buflen, srp);
			return srp;
		}
		if (go_out) {
			sg_place_at_idx_lck(sfp, idx, NULL, NULL);
			return ERR_PTR(res);
		}
		/* failed so remove, halve buflen, try again */
		sg_remove_sgat(srp);
		buflen >>= 1;   /* divide by 2 */
	} while (true);
}

/*
 * Adds an active request (soon to carry a SCSI command) to the current file
 * descriptor by creating a new one or re-using a request from the free
 * list (fl). If successful returns a valid pointer to a sg_request object
 * which is in the SG_RS_BUSY state. On failure returns a negated errno value
 * twisted by ERR_PTR() macro. Note that once a file share is established,
 * the master side's reserve request can only be used in a request share.
 */
static struct sg_request *
sg_add_request(struct sg_comm_wr_t *cwrp, enum sg_shr_var sh_var, int dxfr_len)
{
	bool act_empty = false;
	bool allow_rsv = true;		/* see note above */
	bool mk_new_srp = true;
	bool new_rsv_srp = false;
	bool sl_req = false;
	bool sl_swait = false;
	int idx = 0;
	u32 sum_dlen;
	unsigned long iflags;
	enum sg_rq_state sr_st = SG_RS_INACTIVE;
	enum sg_rq_state ma_sr_st = SG_RS_INACTIVE;
	struct sg_fd *fp = cwrp->sfp;
	struct sg_request *r_srp = NULL; /* returned value won't be NULL */
	__maybe_unused struct sg_request *rsv_srp = NULL;
	struct sg_request *ma_rsv_srp = NULL;
	struct sg_fd *ma_sfp = NULL;
	__maybe_unused const char *cp = NULL;
	char b[48];

	spin_lock_irqsave(&fp->rq_list_lock, iflags);
	b[0] = '\0';

	switch (sh_var) {
	case SG_SHR_NONE:
	case SG_SHR_SL_NOT_SRQ:
		break;
	case SG_SHR_MA_RQ:
		idx = sg_get_idx_available(fp);
		if (idx < 0) {
			new_rsv_srp = true;
			cp = "m_rq";
			goto good_fini;
		}
		r_srp = fp->rsv_arr[idx].rsv_srp;
		sr_st = atomic_read(&r_srp->rq_st);
		if (sr_st == SG_RS_INACTIVE) {
			if (likely(sg_rstate_chg(r_srp, sr_st,
						 SG_RS_BUSY) == 0)) {
				sg_place_at_idx(fp, idx, r_srp,
						ERR_PTR(-ENOENT));
				mk_new_srp = false;
				cp = "m_rq";
				goto good_fini;
			}
		}
		/* Did not find the reserve request available */
		r_srp = ERR_PTR(-EBUSY);
		break;
	case SG_SHR_MA_NOT_SRQ:
		allow_rsv = false;
		break;
	case SG_SHR_SL_RQ:
		ma_sfp = fp->share_sfp;
		if (unlikely(!ma_sfp)) {
			r_srp = ERR_PTR(-EPROTO);
			break;
		}
		/*
		 * There may be contention with another potential slave trying
		 * to pair with this master. The loser will receive an
		 * EADDRINUSE errno. The winner advances master's rq_state:
		 *     SG_RS_SHR_SWAP --> SG_RS_SHR_SLAVE
		 */
		ma_rsv_srp = sg_get_rsv_by_slave(ma_sfp, ERR_PTR(-ENOENT));
		if (!ma_rsv_srp) {
			r_srp = ERR_PTR(-ENOSTR);
			break;
		}
		ma_sr_st = atomic_read(&ma_rsv_srp->rq_st);
		switch (ma_sr_st) {
		case SG_RS_AWAIT_RCV:
			if (ma_rsv_srp->rq_result & SG_ML_RESULT_MSK) {
				/* master done but error occurred */
				r_srp = ERR_PTR(-ENOSTR);
				break;
			}
			sl_req = true;
			break;
		case SG_RS_RCV_DONE:
			if (ma_rsv_srp->rq_result & SG_ML_RESULT_MSK) {
				/* master done but error occurred */
				r_srp = ERR_PTR(-ENOSTR);
				break;
			}
			/* fallthrough */
		case SG_RS_SHR_SWAP:
			sl_req = true;
			if (unlikely(sg_rstate_chg(ma_rsv_srp, ma_sr_st,
						   SG_RS_SHR_SLAVE) != 0))
				/* another potential slave req won race */
				r_srp = ERR_PTR(-EADDRINUSE);
			break;
		case  SG_RS_INFLIGHT:
			sl_req = true;
			/* could be multiple slave waiter requests */
			sl_swait = true;
			break;
		case SG_RS_INACTIVE:
			r_srp = ERR_PTR(-EADDRNOTAVAIL);
			break;
		case SG_RS_BUSY:
			r_srp = ERR_PTR(-EBUSY);
			break;
		case SG_RS_SHR_SLAVE:
		case SG_RS_SHR_SWAIT:
		default:
			r_srp = ERR_PTR(-EADDRINUSE);
			break;
		}
		break;
	}
	if (IS_ERR(r_srp)) {
#if IS_ENABLED(SG_LOG_ACTIVE)
		if (sh_var == SG_SHR_MA_RQ)
			snprintf(b, sizeof(b), "SG_SHR_MA_RQ --> sr_st=%s",
				 sg_rq_st_str(sr_st, false));
		else if (sh_var == SG_SHR_SL_RQ && ma_sfp)
			snprintf(b, sizeof(b), "SG_SHR_SL_RQ-->ma_sr_st=%s",
				 sg_rq_st_str(ma_sr_st, false));
		else
			snprintf(b, sizeof(b), "sh_var=%s",
				 sg_shr_str(sh_var, false));
#endif
		goto err_out;
	}
	cp = "";

	if (sl_req) {	/* slave dlen may be <= master's dlen */
		if (unlikely(dxfr_len > ma_rsv_srp->sgatp->dlen)) {
			SG_LOG(4, fp, "%s: slave dlen [%d] > master dlen\n",
			       __func__, dxfr_len);
			r_srp = ERR_PTR(-E2BIG);
			goto err_out;
		}
		dxfr_len = 0;	/* any srp for slave will do, pick smallest */
	}
	/*
	 * Check the free list (fl) for candidates. If allow_rsv==false then
	 * don't re-use the reserve request. Pick zero data length requests
	 * from the back of the fl, the rest from the front.
	 */
	if (list_empty(&fp->rq_fl)) {	/* N.B. mk_new_srp is true here */
		act_empty = true;
	} else if (dxfr_len < 1) {  /* 0 data length requests at back of fl */
		list_for_each_entry_reverse(r_srp, &fp->rq_fl, fl_entry) {
			if (!allow_rsv &&
			    test_bit(SG_FRQ_RESERVED, r_srp->frq_bm))
				continue;
			sr_st = atomic_read(&r_srp->rq_st);
			if (sr_st == SG_RS_INACTIVE) {
				if (likely(sg_rstate_chg(r_srp, sr_st,
							 SG_RS_BUSY) == 0)) {
					cp = "from back of fl";
					mk_new_srp = false;
					break;
				}
			}
		}
	} else { /*     find request with large enough dlen */
		list_for_each_entry_rcu(r_srp, &fp->rq_fl, fl_entry) {
			if (!allow_rsv &&
			    test_bit(SG_FRQ_RESERVED, r_srp->frq_bm))
				continue;
			sr_st = atomic_read(&r_srp->rq_st);
			if (sr_st == SG_RS_INACTIVE &&
			    r_srp->sgatp->buflen >= dxfr_len) {
				if (likely(sg_rstate_chg(r_srp, sr_st,
							 SG_RS_BUSY) == 0)) {
					cp = "from front of fl";
					mk_new_srp = false;
					break;
				} /* else other got it, move on */
			}
		}
	}

good_fini:
	if (mk_new_srp) {	/* Need new sg_request object */
		bool disallow_cmd_q = test_bit(SG_FFD_NO_CMD_Q, fp->ffd_bm);

		r_srp = NULL;
		if (disallow_cmd_q && !list_empty(&fp->rq_list)) {
			r_srp = ERR_PTR(-EDOM);
			SG_LOG(6, fp, "%s: trying 2nd req but cmd_q=false\n",
			       __func__);
		} else if (fp->tot_fd_thresh > 0) {
			sum_dlen = atomic_read(&fp->sum_fd_dlens) + dxfr_len;
			if (unlikely(sum_dlen > (u32)fp->tot_fd_thresh)) {
				r_srp = ERR_PTR(-E2BIG);
				SG_LOG(2, fp, "%s: sum_of_dlen(%u) > %s\n",
				       __func__, sum_dlen, "tot_fd_thresh");
			}
		}
		if (!IS_ERR(r_srp) && new_rsv_srp) {
			idx = sg_get_idx_new(fp);
			if (idx < 0) {
				idx = sg_get_idx_available(fp);
				if (idx < 0) {
					SG_LOG(1, fp,
					       "%s: no masters available\n",
					       __func__);
					r_srp = ERR_PTR(-EFBIG);
				}
			}
		}
		spin_unlock_irqrestore(&fp->rq_list_lock, iflags);
		if (IS_ERR(r_srp))        /* NULL is _not_ an ERR here */
			goto err_no_lock;
		/* releasing rq_list_lock because next line could take time */
		r_srp = sg_mk_srp_sgat(fp, act_empty, dxfr_len);
		if (IS_ERR(r_srp))
			goto err_no_lock;
		cp = "new";
		SG_LOG(4, fp, "%s: %smk_new_srp=0x%pK ++\n", __func__,
		       (new_rsv_srp ? "rsv " : ""), r_srp);
		if (new_rsv_srp) {
			__set_bit(SG_FRQ_RESERVED, r_srp->frq_bm);
			sg_place_at_idx_lck(fp, idx, r_srp, ERR_PTR(-ENOENT));
		}
		atomic_set(&r_srp->rq_st, SG_RS_BUSY);
	} else {	/* otherwise found srp is on fl, remove from fl */
		list_del_rcu(&r_srp->fl_entry);
	}
	/* following copes with unlikely case where frq_bm > one ulong */
	memcpy(r_srp->frq_bm, cwrp->frq_bm, sizeof(r_srp->frq_bm));
	r_srp->sgatp->dlen = dxfr_len;/* must be <= r_srp->sgat_h.buflen */
	r_srp->sh_var = sh_var;
	r_srp->cmd_opcode = 0xff;  /* set invalid opcode (VS), 0x0 is TUR */
	/* If setup stalls (e.g. blk_get_request()) debug shows 'elap=1 ns' */
	if (test_bit(SG_FFD_TIME_IN_NS, fp->ffd_bm))
		r_srp->start_ns = U64_MAX;
	if (mk_new_srp)
		spin_lock_irqsave(&fp->rq_list_lock, iflags);
	if (sl_req && ma_rsv_srp) {
		/* slave "shares" the master reserve request's data buffer */
		r_srp->sgatp = &ma_rsv_srp->sgat_h;
		__assign_bit(SG_FRQ_DEFER_XFR, r_srp->frq_bm, sl_swait);
		sg_place_at_idx(ma_sfp, -1, ma_rsv_srp, r_srp);
	}
	list_add_tail_rcu(&r_srp->rq_entry, &fp->rq_list);
err_out:
	spin_unlock_irqrestore(&fp->rq_list_lock, iflags);

err_no_lock:
#if IS_ENABLED(SG_LOG_ACTIVE)
	if (IS_ERR(r_srp) && b[0])
		SG_LOG(1, fp, "%s: bad %s\n", __func__, b);
	if (!IS_ERR(r_srp))
		SG_LOG(4, fp, "%s: %s %sr_srp=0x%pK\n", __func__, cp,
		       sg_get_rsv_str_lck(r_srp, "[", "] ", sizeof(b), b),
		       r_srp);
#endif
	return r_srp;
}

/*
 * Moves a completed sg_request object to the free list and sets it to
 * SG_RS_INACTIVE which makes it available for re-use. Requests with no data
 * associated are appended to the tail of the free list while other requests
 * are prepended to the head of the free list.
 */
static void
sg_deact_request(struct sg_fd *sfp, struct sg_request *srp)
{
	bool on_fl = false;
	int dlen, buflen;
	unsigned long iflags;
	enum sg_rq_state sr_st;
	u8 *sbp;
	struct sg_request *t_srp;
	struct sg_scatter_hold *schp;
	__maybe_unused const char *cp = "head";
	char b[32];

	if (WARN_ON(!sfp || !srp))
		return;
	schp = &srp->sgat_h;	/* make sure it is own data buffer */
	spin_lock_irqsave(&sfp->rq_list_lock, iflags);
	sbp = srp->sense_bp;
	srp->sense_bp = NULL;
	sr_st = atomic_read(&srp->rq_st);
	if (sr_st != SG_RS_SHR_SWAP) /* mark _BUSY then _INACTIVE at end */
		atomic_set(&srp->rq_st, SG_RS_BUSY);
	list_del_rcu(&srp->rq_entry);
	/* maybe orphaned req, thus never received */
	mempool_free(sbp, sg_sense_pool);
	/*
	 * N.B. sg_request object is not de-allocated (freed). The contents
	 * of the rq_list and rq_fl lists are de-allocated (freed) when
	 * the owning file descriptor is closed. The free list acts as a LIFO
	 * for same size (buflen) sg_request objects. This can improve the
	 * chance of a cache hit when the sg_request object is re-used.
	 */
	srp->in_resid = 0;
	srp->rq_info = 0;
	srp->sense_len = 0;
	dlen = schp->dlen;
	buflen = schp->buflen;
	/* want ascending free list by dlen, but dlen==0 at end */
	if (dlen > 0) {
		list_for_each_entry_rcu(t_srp, &sfp->rq_fl, fl_entry) {
			if (buflen <= t_srp->sgat_h.buflen ||
			    t_srp->sgat_h.buflen == 0) {
				/* add srp _before_ t_srp on fl */
				list_add_tail_rcu(&srp->fl_entry,
						  &t_srp->fl_entry);
				on_fl = true;
				cp = "ascending pos";
				break;
			}
		}
	}
	if (!on_fl) { /* either empty list, dlen=0 or buflen highest */
		list_add_tail_rcu(&srp->fl_entry, &sfp->rq_fl);
		cp = "tail";
	}
	SG_LOG(5, sfp, "%s: %ssrp=0x%pK move to fl %s\n", __func__,
	       sg_get_rsv_str(srp, "", " ", sizeof(b), b), srp, cp);
	spin_unlock_irqrestore(&sfp->rq_list_lock, iflags);
	if (sr_st != SG_RS_SHR_SWAP) {
		atomic_set(&srp->rq_st, SG_RS_INACTIVE);
		srp->tag = SG_TAG_WILDCARD;
	}
}

/* Returns pointer to sg_fd object or negated errno twisted by ERR_PTR */
static struct sg_fd *
sg_add_sfp(struct sg_device *sdp, struct file *filp)
{
	bool reduced = false;
	int rbuf_len;
	unsigned long iflags;
	long err;
	struct sg_fd *sfp;
	struct sg_request *srp;

	sfp = kzalloc(sizeof(*sfp), GFP_ATOMIC | __GFP_NOWARN);
	if (unlikely(!sfp))
		return ERR_PTR(-ENOMEM);
	init_waitqueue_head(&sfp->cmpl_wait);
	spin_lock_init(&sfp->rq_list_lock);
	INIT_LIST_HEAD(&sfp->rq_list);
	INIT_LIST_HEAD(&sfp->rq_fl);
	kref_init(&sfp->f_ref);
	mutex_init(&sfp->f_mutex);
	sfp->timeout = SG_DEFAULT_TIMEOUT;
	sfp->timeout_user = SG_DEFAULT_TIMEOUT_USER;
	sfp->filp = filp;
	/* other bits in sfp->ffd_bm[1] cleared by kzalloc() above */
	__assign_bit(SG_FFD_FORCE_PACKID, sfp->ffd_bm, SG_DEF_FORCE_PACK_ID);
	__assign_bit(SG_FFD_NO_CMD_Q, sfp->ffd_bm, !SG_DEF_COMMAND_Q);
	__assign_bit(SG_FFD_KEEP_ORPHAN, sfp->ffd_bm, SG_DEF_KEEP_ORPHAN);
	__assign_bit(SG_FFD_TIME_IN_NS, sfp->ffd_bm, SG_DEF_TIME_UNIT);
	__assign_bit(SG_FFD_Q_AT_TAIL, sfp->ffd_bm, SG_DEFAULT_Q_AT);
	sfp->tot_fd_thresh = SG_TOT_FD_THRESHOLD;
	atomic_set(&sfp->sum_fd_dlens, 0);
	atomic_set(&sfp->submitted, 0);
	atomic_set(&sfp->waiting, 0);
	atomic_set(&sfp->svb_cnt, 0);
	/*
	 * SG_SCATTER_SZ initializes scatter_elem_sz but different value may
	 * be given as driver/module parameter (e.g. 'scatter_elem_sz=8192').
	 * Any user provided number will be changed to be PAGE_SIZE as a
	 * minimum, otherwise it will be rounded down (if required) to a
	 * power of 2. So it will always be a power of 2.
	 */
	sfp->sgat_elem_sz = scatter_elem_sz;
	sfp->parentdp = sdp;
	if (unlikely(SG_IS_DETACHING(sdp))) {
		SG_LOG(1, sfp, "%s: detaching\n", __func__);
		kfree(sfp);
		return ERR_PTR(-ENODEV);
	}
	sg_unshare_fds(sfp, NULL);
	if (unlikely(sg_big_buff != def_reserved_size))
		sg_big_buff = def_reserved_size;

	rbuf_len = min_t(int, sg_big_buff, sdp->max_sgat_sz);
	if (rbuf_len > 0) {
		srp = sg_build_reserve(sfp, rbuf_len);
		if (IS_ERR(srp)) {
			kfree(sfp);
			err = PTR_ERR(srp);
			SG_LOG(1, sfp, "%s: build reserve err=%ld\n", __func__,
			       -err);
			return ERR_PTR(err);
		}
		if (srp->sgatp->buflen < rbuf_len) {
			reduced = true;
			SG_LOG(2, sfp,
			       "%s: reserve reduced from %d to buflen=%d\n",
			       __func__, rbuf_len, srp->sgatp->buflen);
		}
		__set_bit(SG_FRQ_RESERVED, srp->frq_bm);
		list_add_rcu(&srp->fl_entry, &sfp->rq_fl);
	}
	if (!reduced) {
		SG_LOG(4, sfp, "%s: built reserve buflen=%d\n", __func__,
		       rbuf_len);
	}
	write_lock_irqsave(&sdp->sfd_lock, iflags);
	list_add_tail(&sfp->sfd_entry, &sdp->sfds);
	kref_get(&sdp->d_ref);
	__module_get(THIS_MODULE);
	write_unlock_irqrestore(&sdp->sfd_lock, iflags);
	SG_LOG(3, sfp, "%s: success, sfp=0x%pK ++\n", __func__, sfp);
	return sfp;
}

/*
 * A successful call to sg_release() will result, at some later time, to this
 * "user context" function being invoked. All requests associated with this
 * file descriptor should be completed or cancelled when this function is
 * called (due to sfp->f_ref). Also the file descriptor itself has not been
 * accessible since it was list_del()-ed by the preceding sg_remove_sfp()
 * call. So no locking is required. sdp should never be NULL but to make
 * debugging more robust, this function will not blow up in that case.
 */
static void
sg_uc_remove_sfp(struct work_struct *work)
{
	struct sg_fd *sfp = container_of(work, struct sg_fd, ew_fd.work);
	struct sg_device *sdp;
	struct sg_request *srp;
	__maybe_unused const char *cp = " srp=0x";

	if (unlikely(!sfp)) {
		pr_warn("sg: %s: sfp is NULL\n", __func__);
		return;
	}
	sdp = sfp->parentdp;

	/* Cleanup any responses which were never received. */
	while (!list_empty(&sfp->rq_list)) {
		srp = list_last_entry(&sfp->rq_list, struct sg_request,
				      rq_entry);
		if (unlikely(!srp))
			continue;
		sg_finish_scsi_blk_rq(srp);
		list_del(&srp->rq_entry);
		if (srp->sgatp->buflen > 0)
			sg_remove_sgat(srp);
		if (srp->sense_bp) {
			mempool_free(srp->sense_bp, sg_sense_pool);
			srp->sense_bp = NULL;
		}
		SG_LOG(6, sfp, "%s:%s%pK --\n", __func__, cp, srp);
		kfree(srp);
	}
	while (!list_empty(&sfp->rq_fl)) {
		srp = list_last_entry(&sfp->rq_fl, struct sg_request,
				      fl_entry);
		if (unlikely(!srp))
			continue;
		list_del(&srp->fl_entry);
		if (srp->sgatp->buflen > 0)
			sg_remove_sgat(srp);
		if (srp->sense_bp) {
			mempool_free(srp->sense_bp, sg_sense_pool);
			srp->sense_bp = NULL;
		}
		SG_LOG(6, sfp, "%s: fl%s%pK --\n", __func__, cp, srp);
		kfree(srp);
	}
	SG_LOG(3, sfp, "%s: kfree: sfp=0x%pK --\n", __func__, sfp);
	kfree(sfp);

	if (likely(sdp)) {
		scsi_device_put(sdp->device);
		kref_put(&sdp->d_ref, sg_device_destroy);
	}
	module_put(THIS_MODULE);
}

static void
sg_remove_sfp(struct kref *kref)
{
	unsigned long iflags;
	struct sg_fd *sfp = container_of(kref, struct sg_fd, f_ref);
	struct sg_device *sdp = sfp->parentdp;
	struct sg_fd *o_sfp = (sfp->shr_fd != SG_SHARE_FD_UNUSED) ?
					sfp->share_sfp : NULL;

	if (o_sfp && !test_bit(SG_FFD_RELEASE, o_sfp->ffd_bm)) {
		mutex_lock(&o_sfp->f_mutex);
		sg_remove_sfp_share(sfp);
		mutex_unlock(&o_sfp->f_mutex);
	}
	write_lock_irqsave(&sdp->sfd_lock, iflags);
	list_del(&sfp->sfd_entry);
	write_unlock_irqrestore(&sdp->sfd_lock, iflags);

	INIT_WORK(&sfp->ew_fd.work, sg_uc_remove_sfp);
	schedule_work(&sfp->ew_fd.work);
}

static struct sg_device *
sg_lookup_dev(int dev)
	__must_hold(&sg_index_lock)
{
	return idr_find(&sg_index_idr, dev);
}

/*
 * Returns valid pointer to a sg_device object on success or a negated
 * errno value on failure. Does not return NULL.
 */
static struct sg_device *
sg_get_dev(int min_dev)
{
	struct sg_device *sdp;
	unsigned long iflags;

	read_lock_irqsave(&sg_index_lock, iflags);
	sdp = sg_lookup_dev(min_dev);
	if (unlikely(!sdp))
		sdp = ERR_PTR(-ENXIO);
	else if (SG_IS_DETACHING(sdp)) {
		/* If detaching, then the refcount may already be 0, in
		 * which case it would be a bug to do kref_get().
		 */
		sdp = ERR_PTR(-ENODEV);
	} else
		kref_get(&sdp->d_ref);
	read_unlock_irqrestore(&sg_index_lock, iflags);
	return sdp;
}

#if IS_ENABLED(SG_PROC_OR_DEBUG_FS)
static const char *
sg_rq_st_str(enum sg_rq_state rq_st, bool long_str)
{
	switch (rq_st) {		/* request state */
	case SG_RS_INACTIVE:
		return long_str ? "inactive" :  "ina";
	case SG_RS_INFLIGHT:	/* includes device command processing */
		return long_str ? "inflight" : "act";
	case SG_RS_AWAIT_RCV:
		return long_str ? "await_receive" : "rcv";
	case SG_RS_RCV_DONE:
		return long_str ? "receive_done" : "fin";
	case SG_RS_BUSY:	/* state transitioning */
		return long_str ? "busy" : "bsy";
	case SG_RS_SHR_SWAP:	/* master: awaiting slave hookup */
		return long_str ? "share swap" : "s_wp";
	case SG_RS_SHR_SLAVE:	/* master: waiting for inflight slave */
		return long_str ? "share slave active" : "sl_a";
	case SG_RS_SHR_SWAIT:	/* slave: waiting for inflight slave master */
		return long_str ? "share slave wait" : "sl_w";
	default:
		return long_str ? "unknown" : "unk";
	}
}

static const char *
sg_shr_str(enum sg_shr_var sh_var, bool long_str)
{
	switch (sh_var) {	/* share variety of request */
	case SG_SHR_NONE:
		return long_str ? "none" :  "-";
	case SG_SHR_MA_RQ:
		return long_str ? "master request" :  "m_rq";
	case SG_SHR_MA_NOT_SRQ:
		return long_str ? "master, not share request" :  "m_nsh";
	case SG_SHR_SL_RQ:
		return long_str ? "slave request" :  "s_rq";
	case SG_SHR_SL_NOT_SRQ:
		return long_str ? "slave, not share request" :  "s_nsh";
	default:
		return long_str ? "unknown" : "unk";
	}
}

#else

static const char *
sg_rq_st_str(enum sg_rq_state rq_st, bool long_str)
{
	return "";
}

static const char *
sg_shr_str(enum sg_shr_var sh_var, bool long_str)
{
	return "";
}
#endif

#if IS_ENABLED(SG_PROC_OR_DEBUG_FS)

#define SG_SNAPSHOT_DEV_MAX 4

/*
 * For snapshot_devs array, -1 or two adjacent the same is terminator.
 * -1 in first element of first two elements the same implies all.
 */
static struct sg_dfs_context_t {
	struct dentry *dfs_rootdir;
	int snapshot_devs[SG_SNAPSHOT_DEV_MAX];
} sg_dfs_cxt;

struct sg_proc_deviter {
	loff_t index;
	size_t max;
	int fd_index;
};

static int
sg_last_dev(void)
{
	int k = -1;
	unsigned long iflags;

	read_lock_irqsave(&sg_index_lock, iflags);
	idr_for_each(&sg_index_idr, sg_idr_max_id, &k);
	read_unlock_irqrestore(&sg_index_lock, iflags);
	return k + 1;		/* origin 1 */
}

static void *
dev_seq_start(struct seq_file *s, loff_t *pos)
{
	struct sg_proc_deviter *it = kzalloc(sizeof(*it), GFP_KERNEL);

	s->private = it;
	if (unlikely(!it))
		return NULL;

	it->index = *pos;
	it->max = sg_last_dev();
	if (it->index >= (int)it->max)
		return NULL;
	return it;
}

static void *
dev_seq_next(struct seq_file *s, void *v, loff_t *pos)
{
	struct sg_proc_deviter *it = s->private;

	*pos = ++it->index;
	return (it->index < (int)it->max) ? it : NULL;
}

static void
dev_seq_stop(struct seq_file *s, void *v)
{
	kfree(s->private);
}

#endif			/* SG_PROC_OR_DEBUG_FS */

#if IS_ENABLED(CONFIG_SCSI_PROC_FS)     /* around 100 lines */

static int
sg_proc_seq_show_int(struct seq_file *s, void *v)
{
	seq_printf(s, "%d\n", *((int *)s->private));
	return 0;
}

static int
sg_proc_single_open_adio(struct inode *inode, struct file *filp)
{
	return single_open(filp, sg_proc_seq_show_int, &sg_allow_dio);
}

/*
 * This is the write(2) system call entry point. v4 interface disallowed.
 * Returns count or a negated errno value.
 */
static ssize_t
sg_proc_write_adio(struct file *filp, const char __user *buffer,
		   size_t count, loff_t *off)
{
	int err;
	unsigned long num;

	if (unlikely(!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO)))
		return -EACCES;
	err = kstrtoul_from_user(buffer, count, 0, &num);
	if (unlikely(err))
		return err;
	sg_allow_dio = num ? 1 : 0;
	return count;
}

static int
sg_proc_single_open_dressz(struct inode *inode, struct file *filp)
{
	return single_open(filp, sg_proc_seq_show_int, &sg_big_buff);
}

static ssize_t
sg_proc_write_dressz(struct file *filp, const char __user *buffer,
		     size_t count, loff_t *off)
{
	int err;
	unsigned long k = ULONG_MAX;

	if (unlikely(!capable(CAP_SYS_ADMIN) || !capable(CAP_SYS_RAWIO)))
		return -EACCES;

	err = kstrtoul_from_user(buffer, count, 0, &k);
	if (unlikely(err))
		return err;
	if (likely(k <= 1048576)) {	/* limit "big buff" to 1 MB */
		sg_big_buff = k;
		return count;
	}
	return -EDOM;
}

static int
sg_proc_seq_show_version(struct seq_file *s, void *v)
{
	seq_printf(s, "%d\t%s [%s]\n", sg_version_num, SG_VERSION_STR,
		   sg_version_date);
	return 0;
}

static int
sg_proc_seq_show_devhdr(struct seq_file *s, void *v)
{
	seq_puts(s, "host\tchan\tid\tlun\ttype\topens\tqdepth\tbusy\tonline\n");
	return 0;
}

static int
sg_proc_seq_show_dev(struct seq_file *s, void *v)
{
	struct sg_proc_deviter *it = (struct sg_proc_deviter *)v;
	struct sg_device *sdp;
	struct scsi_device *scsidp;
	unsigned long iflags;

	read_lock_irqsave(&sg_index_lock, iflags);
	sdp = it ? sg_lookup_dev(it->index) : NULL;
	if (unlikely(!sdp || !sdp->device || SG_IS_DETACHING(sdp)))
		seq_puts(s, "-1\t-1\t-1\t-1\t-1\t-1\t-1\t-1\t-1\n");
	else {
		scsidp = sdp->device;
		seq_printf(s, "%d\t%d\t%d\t%llu\t%d\t%d\t%d\t%d\t%d\n",
			      scsidp->host->host_no, scsidp->channel,
			      scsidp->id, scsidp->lun, (int)scsidp->type,
			      1,
			      (int)scsidp->queue_depth,
			      (int)atomic_read(&scsidp->device_busy),
			      (int)scsi_device_online(scsidp));
	}
	read_unlock_irqrestore(&sg_index_lock, iflags);
	return 0;
}

static int
sg_proc_seq_show_devstrs(struct seq_file *s, void *v)
{
	struct sg_proc_deviter *it = (struct sg_proc_deviter *)v;
	struct sg_device *sdp;
	struct scsi_device *scsidp;
	unsigned long iflags;

	read_lock_irqsave(&sg_index_lock, iflags);
	sdp = it ? sg_lookup_dev(it->index) : NULL;
	scsidp = sdp ? sdp->device : NULL;
	if (sdp && scsidp && !SG_IS_DETACHING(sdp))
		seq_printf(s, "%8.8s\t%16.16s\t%4.4s\n",
			   scsidp->vendor, scsidp->model, scsidp->rev);
	else
		seq_puts(s, "<no active device>\n");
	read_unlock_irqrestore(&sg_index_lock, iflags);
	return 0;
}

#endif		/* CONFIG_SCSI_PROC_FS (~100 lines back) */

#if IS_ENABLED(SG_PROC_OR_DEBUG_FS)

/* Writes debug info for one sg_request in obp buffer */
static int
sg_proc_debug_sreq(struct sg_request *srp, int to, bool t_in_ns, char *obp,
		   int len)
		__must_hold(&sfp->rq_list_lock)
{
	bool is_v3v4, v4, is_dur, n_wq;
	int n = 0;
	u32 dur;
	enum sg_rq_state rq_st;
	const char *cp;
	const char *tp = t_in_ns ? "ns" : "ms";
	char b[32];

	if (unlikely(len < 1))
		return 0;
	v4 = test_bit(SG_FRQ_IS_V4I, srp->frq_bm);
	is_v3v4 = v4 ? true : (srp->s_hdr3.interface_id != '\0');
	sg_get_rsv_str(srp, "     ", "", sizeof(b), b);
	if (strlen(b) > 5)
		cp = (is_v3v4 && (srp->rq_flags & SG_FLAG_MMAP_IO)) ?
					"mmap" : "";
	else
		cp = (srp->rq_info & SG_INFO_DIRECT_IO_MASK) ? "dio" : "";
	rq_st = atomic_read(&srp->rq_st);
	dur = sg_get_dur(srp, &rq_st, t_in_ns, &is_dur);
	n += scnprintf(obp + n, len - n, "%s%s>> %s: dlen=%d/%d id=%d", b,
		       cp, sg_rq_st_str(rq_st, false), srp->sgatp->dlen,
		       srp->sgatp->buflen, (int)srp->pack_id);
	if (is_dur)	/* cmd/req has completed, waiting for ... */
		n += scnprintf(obp + n, len - n, " dur=%u%s", dur, tp);
	else if (dur < U32_MAX) { /* in-flight or busy (so ongoing) */
		if ((srp->rq_flags & SGV4_FLAG_YIELD_TAG) &&
		    srp->tag != SG_TAG_WILDCARD)
			n += scnprintf(obp + n, len - n, " tag=0x%x",
				       srp->tag);
		n += scnprintf(obp + n, len - n, " t_o/elap=%us/%u%s",
			       to / 1000, dur, tp);
	}
	if (srp->sh_var != SG_SHR_NONE)
		n += scnprintf(obp + n, len - n, " shr=%s",
			       sg_shr_str(srp->sh_var, false));
	n_wq = !!(srp->rq_flags & SGV4_FLAG_NO_WAITQ);
	n += scnprintf(obp + n, len - n, " sgat=%d%s op=0x%02x\n",
		       srp->sgatp->num_sgat,
		       (n_wq ? " n_wq" : ""), srp->cmd_opcode);
	return n;
}

/* Writes debug info for one sg fd (including its sg requests) in obp buffer */
static int
sg_proc_debug_fd(struct sg_fd *fp, char *obp, int len, bool reduced)
		__must_hold(&sfp->rq_list_lock)
{
	bool first_fl;
	bool t_in_ns = test_bit(SG_FFD_TIME_IN_NS, fp->ffd_bm);
	int n = 0;
	int to;
	const char *cp;
	struct sg_request *srp = fp->rsv_arr[0].rsv_srp;

	if (fp->shr_fd == SG_SHARE_FD_UNUSED)
		cp = "";
	else
		cp = (fp->shr_fd == SG_SHARE_FD_MASTER) ?
			" shr_mast" : " shr_slv";
	/* sgat=-1 means unavailable */
	to = jiffies_to_msecs(fp->timeout);
	if (to % 1000)
		n += scnprintf(obp + n, len - n, "timeout=%dms rs", to);
	else
		n += scnprintf(obp + n, len - n, "timeout=%ds rs", to / 1000);
	n += scnprintf(obp + n, len - n, "v_buflen=%d%s\n   cmd_q=%d ",
		       (srp ? srp->sgatp->buflen : -1), cp,
		       (int)!test_bit(SG_FFD_NO_CMD_Q, fp->ffd_bm));
	n += scnprintf(obp + n, len - n,
		       "f_packid=%d k_orphan=%d ffd_bm=0x%lx\n",
		       (int)test_bit(SG_FFD_FORCE_PACKID, fp->ffd_bm),
		       (int)test_bit(SG_FFD_KEEP_ORPHAN, fp->ffd_bm),
		       fp->ffd_bm[0]);
	n += scnprintf(obp + n, len - n, "   mmap_called=%d sum_fd_dlens=%u\n",
		       test_bit(SG_FFD_MMAP_CALLED, fp->ffd_bm),
		       atomic_read(&fp->sum_fd_dlens));
	n += scnprintf(obp + n, len - n,
		       "   submitted=%d waiting=%d   open thr_id=%d\n",
		       atomic_read(&fp->submitted),
		       atomic_read(&fp->waiting), fp->tid);
	if (reduced)
		return n;
	list_for_each_entry_rcu(srp, &fp->rq_list, rq_entry) {
		spin_lock(&srp->req_lck);
		n += sg_proc_debug_sreq(srp, fp->timeout, t_in_ns, obp + n,
					len - n);
		spin_unlock(&srp->req_lck);
	}
	if (list_empty(&fp->rq_list))
		n += scnprintf(obp + n, len - n, "     No requests active\n");
	first_fl = true;
	list_for_each_entry_rcu(srp, &fp->rq_fl, fl_entry) {
		if (first_fl) {
			n += scnprintf(obp + n, len - n, "   Free list:\n");
			first_fl = false;
		}
		spin_lock(&srp->req_lck);
		n += sg_proc_debug_sreq(srp, fp->timeout, t_in_ns,
					obp + n, len - n);
		spin_unlock(&srp->req_lck);
	}
	return n;
}

/* Writes debug info for one sg device (including its sg fds) in obp buffer */
static int
sg_proc_debug_sdev(struct sg_device *sdp, char *obp, int len,
		   int *fd_counterp, bool reduced)
		__must_hold(&sdp->sfd_lock)
{
	int n = 0;
	int my_count = 0;
	struct scsi_device *ssdp = sdp->device;
	struct sg_fd *fp;
	char *disk_name;
	int *countp;

	countp = fd_counterp ? fd_counterp : &my_count;
	disk_name = (sdp->disk ? sdp->disk->disk_name : "?_?");
	n += scnprintf(obp + n, len - n, " >>> device=%s ", disk_name);
	n += scnprintf(obp + n, len - n, "%d:%d:%d:%llu ", ssdp->host->host_no,
		       ssdp->channel, ssdp->id, ssdp->lun);
	n += scnprintf(obp + n, len - n,
		       "  max_sgat_sz,elems=2^%d,%d excl=%d open_cnt=%d\n",
		       ilog2(sdp->max_sgat_sz), sdp->max_sgat_elems,
		       SG_HAVE_EXCLUDE(sdp), atomic_read(&sdp->open_cnt));
	list_for_each_entry(fp, &sdp->sfds, sfd_entry) {
		++*countp;
		rcu_read_lock(); /* assume irqs disabled */
		n += scnprintf(obp + n, len - n, "  FD(%d): ", *countp);
		n += sg_proc_debug_fd(fp, obp + n, len - n, reduced);
		rcu_read_unlock();
	}
	return n;
}

static int
sg_proc_seq_show_debug(struct seq_file *s, void *v, bool reduced)
{
	bool found = false;
	bool trunc = false;
	const int bp_len = SG_PROC_DEBUG_SZ;
	int j, sd_n;
	int n = 0;
	int k = 0;
	unsigned long iflags;
	struct sg_proc_deviter *it = (struct sg_proc_deviter *)v;
	struct sg_device *sdp;
	int *fdi_p;
	const int *dev_arr = sg_dfs_cxt.snapshot_devs;
	char *bp;
	char *disk_name;
	char b1[128];

	b1[0] = '\0';
	if (it && (it->index == 0))
		seq_printf(s, "max_active_device=%d  def_reserved_size=%d\n",
			   (int)it->max, def_reserved_size);
	fdi_p = it ? &it->fd_index : &k;
	bp = kzalloc(bp_len, __GFP_NOWARN | GFP_KERNEL);
	if (unlikely(!bp)) {
		seq_printf(s, "%s: Unable to allocate %d on heap, finish\n",
			   __func__, bp_len);
		return -EPERM;
	}
	read_lock_irqsave(&sg_index_lock, iflags);
	sdp = it ? sg_lookup_dev(it->index) : NULL;
	if (unlikely(!sdp))
		goto skip;
	sd_n = dev_arr[0];
	if (sd_n != -1 && sd_n != sdp->index && sd_n != dev_arr[1]) {
		for (j = 1; j < SG_SNAPSHOT_DEV_MAX; ) {
			sd_n = dev_arr[j];
			if (sd_n < 0)
				goto skip;
			++j;
			if (j >= SG_SNAPSHOT_DEV_MAX) {
				if (sd_n == sdp->index) {
					found = true;
					break;
				}
			} else if (sd_n == dev_arr[j]) {
				goto skip;
			} else if (sd_n == sdp->index) {
				found = true;
				break;
			}
		}
		if (!found)
			goto skip;
		found = false;
	}
	read_lock(&sdp->sfd_lock);
	if (!list_empty(&sdp->sfds)) {
		found = true;
		disk_name = (sdp->disk ? sdp->disk->disk_name : "?_?");
		if (SG_IS_DETACHING(sdp)) {
			snprintf(b1, sizeof(b1), " >>> device=%s  %s\n",
				 disk_name, "detaching pending close\n");
		} else if (sdp->device) {
			n = sg_proc_debug_sdev(sdp, bp, bp_len, fdi_p,
					       reduced);
			if (n >= bp_len - 1) {
				trunc = true;
				if (bp[bp_len - 2] != '\n')
					bp[bp_len - 2] = '\n';
			}
		} else {
			snprintf(b1, sizeof(b1), " >>> device=%s  %s\n",
				 disk_name, "sdp->device==NULL, skip");
		}
	}
	read_unlock(&sdp->sfd_lock);
skip:
	read_unlock_irqrestore(&sg_index_lock, iflags);
	if (found) {
		if (n > 0) {
			seq_puts(s, bp);
			if (seq_has_overflowed(s))
				goto s_ovfl;
			if (trunc)
				seq_printf(s, "   >> Output truncated %s\n",
					   "due to buffer size");
		} else if (b1[0]) {
			seq_puts(s, b1);
			if (seq_has_overflowed(s))
				goto s_ovfl;
		}
	}
s_ovfl:
	kfree(bp);
	return 0;
}

static int
sg_proc_seq_show_debug_full(struct seq_file *s, void *v)
{
	return sg_proc_seq_show_debug(s, v, false);
}

static int
sg_proc_seq_show_debug_summ(struct seq_file *s, void *v)
{
	return sg_proc_seq_show_debug(s, v, true);
}

#endif		/* SG_PROC_OR_DEBUG_FS */

#if IS_ENABLED(CONFIG_SCSI_PROC_FS)

static const struct file_operations adio_fops = {
	.owner = THIS_MODULE,
	.open = sg_proc_single_open_adio,
	.read = seq_read,
	.llseek = seq_lseek,
	.write = sg_proc_write_adio,
	.release = single_release,
};

static const struct file_operations dressz_fops = {
	.owner = THIS_MODULE,
	.open = sg_proc_single_open_dressz,
	.read = seq_read,
	.llseek = seq_lseek,
	.write = sg_proc_write_dressz,
	.release = single_release,
};

static const struct seq_operations dev_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_dev,
};

static const struct seq_operations devstrs_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_devstrs,
};

static const struct seq_operations debug_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_debug_full,
};

static const struct seq_operations debug_summ_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_debug_summ,
};

static int
sg_proc_init(void)
{
	struct proc_dir_entry *p;

	p = proc_mkdir("scsi/sg", NULL);
	if (!p)
		return 1;

	proc_create("allow_dio", 0644, p, &adio_fops);
	proc_create_seq("debug", 0444, p, &debug_seq_ops);
	proc_create_seq("debug_summary", 0444, p, &debug_summ_seq_ops);
	proc_create("def_reserved_size", 0644, p, &dressz_fops);
	proc_create_single("device_hdr", 0444, p, sg_proc_seq_show_devhdr);
	proc_create_seq("devices", 0444, p, &dev_seq_ops);
	proc_create_seq("device_strs", 0444, p, &devstrs_seq_ops);
	proc_create_single("version", 0444, p, sg_proc_seq_show_version);
	return 0;
}

/* remove_proc_subtree("scsi/sg", NULL) in exit_sg() does cleanup */

#else

static int
sg_proc_init(void)
{
	return 0;
}

#endif			/* CONFIG_SCSI_PROC_FS */

#if IS_ENABLED(CONFIG_DEBUG_FS)

struct sg_dfs_attr {
	const char *name;
	umode_t mode;
	int (*show)(void *d, struct seq_file *m);
	ssize_t (*write)(void *d, const char __user *b, size_t s, loff_t *o);
	/* Set either .show or .seq_ops. */
	const struct seq_operations *seq_ops;
};

static int
sg_dfs_snapped_show(void *data, struct seq_file *m)
{
	mutex_lock(&snapped_mutex);
	if (snapped_buf && snapped_buf[0])
		seq_puts(m, snapped_buf);
	mutex_unlock(&snapped_mutex);
	return 0;
}

static ssize_t
sg_dfs_snapped_write(void *data, const char __user *buf, size_t count,
		     loff_t *ppos)
{
	/* Any write clears snapped buffer */
	mutex_lock(&snapped_mutex);
	kfree(snapped_buf);
	snapped_buf = NULL;
	mutex_unlock(&snapped_mutex);
	return count;
}

static int
sg_dfs_snapshot_devs_show(void *data, struct seq_file *m)
{
	bool last;
	int k, d;
	struct sg_dfs_context_t *ctxp = data;

	for (k = 0; k < SG_SNAPSHOT_DEV_MAX; ++k) {
		d = ctxp->snapshot_devs[k];
		last = (k + 1 == SG_SNAPSHOT_DEV_MAX);
		if (d < 0) {
			if (k == 0)
				seq_puts(m, "-1");
			break;
		}
		if (!last && d == ctxp->snapshot_devs[k + 1]) {
			if (k == 0)
				seq_puts(m, "-1");
			break;
		}
		if (k != 0)
			seq_puts(m, ",");
		seq_printf(m, "%d", d);
	}
	seq_puts(m, "\n");
	return 0;
}

static ssize_t
sg_dfs_snapshot_devs_write(void *data, const char __user *buf, size_t count,
			   loff_t *ppos)
{
	bool trailing_comma;
	int k, n;
	struct sg_dfs_context_t *cxtp = data;
	char lbuf[64] = { }, *cp, *c2p;

	if (count >= sizeof(lbuf)) {
		pr_err("%s: operation too long\n", __func__);
		return -EINVAL;
	}
	if (copy_from_user(lbuf, buf, count))
		return -EFAULT;
	for (k = 0, n = 0, cp = lbuf; k < SG_SNAPSHOT_DEV_MAX;
	     ++k, cp = c2p + 1) {
		c2p = strchr(cp, ',');
		if (c2p)
			*c2p = '\0';
		trailing_comma = !!c2p;
		/* sscanf is easier to use that this ... */
		if (kstrtoint(cp, 10, cxtp->snapshot_devs + k))
			break;
		++n;
		if (!trailing_comma)
			break;
	}
	if (n == 0) {
		return -EINVAL;
	} else if (k >= SG_SNAPSHOT_DEV_MAX && trailing_comma) {
		pr_err("%s: only %d elements in snapshot array\n", __func__,
		       SG_SNAPSHOT_DEV_MAX);
		return -EINVAL;
	}
	if (n < SG_SNAPSHOT_DEV_MAX)
		cxtp->snapshot_devs[n] = -1;
	return count;
}

static int
sg_dfs_show(struct seq_file *m, void *v)
{
	const struct sg_dfs_attr *attr = m->private;
	void *data = d_inode(m->file->f_path.dentry->d_parent)->i_private;

	return attr->show(data, m);
}

static ssize_t
sg_dfs_write(struct file *file, const char __user *buf, size_t count,
	     loff_t *ppos)
{
	struct seq_file *m = file->private_data;
	const struct sg_dfs_attr *attr = m->private;
	void *data = d_inode(file->f_path.dentry->d_parent)->i_private;

	/*
	 * Attributes that only implement .seq_ops are read-only and 'attr' is
	 * the same with 'data' in this case.
	 */
	if (attr == data || !attr->write)
		return -EPERM;
	return attr->write(data, buf, count, ppos);
}

static int
sg_dfs_open(struct inode *inode, struct file *file)
{
	const struct sg_dfs_attr *attr = inode->i_private;
	void *data = d_inode(file->f_path.dentry->d_parent)->i_private;
	struct seq_file *m;
	int ret;

	if (attr->seq_ops) {
		ret = seq_open(file, attr->seq_ops);
		if (!ret) {
			m = file->private_data;
			m->private = data;
		}
		return ret;
	}
	if (WARN_ON_ONCE(!attr->show))
		return -EPERM;
	return single_open(file, sg_dfs_show, inode->i_private);
}

static int
sg_dfs_release(struct inode *inode, struct file *file)
{
	const struct sg_dfs_attr *attr = inode->i_private;

	if (attr->show)
		return single_release(inode, file);
	return seq_release(inode, file);
}

static const struct file_operations sg_dfs_fops = {
	.owner		= THIS_MODULE,
	.open		= sg_dfs_open,
	.read		= seq_read,
	.write		= sg_dfs_write,
	.llseek		= seq_lseek,
	.release	= sg_dfs_release,
};

static void sg_dfs_mk_files(struct dentry *parent, void *data,
			    const struct sg_dfs_attr *attr)
{
	if (IS_ERR_OR_NULL(parent))
		return;

	d_inode(parent)->i_private = data;
	for (; attr->name; ++attr)
		debugfs_create_file(attr->name, attr->mode, parent,
				    (void *)attr, &sg_dfs_fops);
}

static const struct seq_operations sg_snapshot_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_debug_full,
};

static const struct seq_operations sg_snapshot_summ_seq_ops = {
	.start = dev_seq_start,
	.next  = dev_seq_next,
	.stop  = dev_seq_stop,
	.show  = sg_proc_seq_show_debug_summ,
};

static const struct sg_dfs_attr sg_dfs_attrs[] = {
	{"snapped", 0600, sg_dfs_snapped_show, sg_dfs_snapped_write},
	{"snapshot", 0400, .seq_ops = &sg_snapshot_seq_ops},
	{"snapshot_devs", 0600, sg_dfs_snapshot_devs_show,
	 sg_dfs_snapshot_devs_write},
	{"snapshot_summary", 0400, .seq_ops = &sg_snapshot_summ_seq_ops},
	{ },
};

static void
sg_dfs_init(void)
{
	/* create and populate /sys/kernel/debug/scsi_generic directory */
	if (!sg_dfs_cxt.dfs_rootdir) {
		sg_dfs_cxt.dfs_rootdir = debugfs_create_dir("scsi_generic",
							    NULL);
		sg_dfs_mk_files(sg_dfs_cxt.dfs_rootdir, &sg_dfs_cxt,
				sg_dfs_attrs);
	}
	sg_dfs_cxt.snapshot_devs[0] = -1;	/* show all sg devices */
}

static void
sg_dfs_exit(void)
{
	debugfs_remove_recursive(sg_dfs_cxt.dfs_rootdir);
	sg_dfs_cxt.dfs_rootdir = NULL;
}

#else		/* not  defined: CONFIG_DEBUG_FS */

static void sg_dfs_init(void) {}
static void sg_dfs_exit(void) {}

#endif		/* CONFIG_DEBUG_FS */

module_param_named(scatter_elem_sz, scatter_elem_sz, int, 0644);
module_param_named(def_reserved_size, def_reserved_size, int, 0644);
module_param_named(allow_dio, sg_allow_dio, int, 0644);

MODULE_AUTHOR("Douglas Gilbert");
MODULE_DESCRIPTION("SCSI generic (sg) driver");
MODULE_LICENSE("GPL");
MODULE_VERSION(SG_VERSION_STR);
MODULE_ALIAS_CHARDEV_MAJOR(SCSI_GENERIC_MAJOR);

MODULE_PARM_DESC(scatter_elem_sz, "scatter gather element size (default: max(SG_SCATTER_SZ, PAGE_SIZE))");
MODULE_PARM_DESC(def_reserved_size, "size of buffer reserved for each fd");
MODULE_PARM_DESC(allow_dio, "allow direct I/O (default: 0 (disallow))");
module_init(init_sg);
module_exit(exit_sg);
